@@ -1,22 +1,22 @@
 /*
  * Stellarium
  * Copyright (C) 2006 Fabien Chereau
- * 
+ *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
  * as published by the Free Software Foundation; either version 2
  * of the License, or (at your option) any later version.
- * 
+ *
  * This program is distributed in the hope that it will be useful,
  * but WITHOUT ANY WARRANTY; without even the implied warranty of
  * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  * GNU General Public License for more details.
- * 
+ *
  * You should have received a copy of the GNU General Public License
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
  */
- 
+
 #include <config.h>
 
 #include <QDebug>
@@ -31,7 +31,8 @@
 #include "StelIniParser.hpp"
 
 
-StelModuleMgr::StelModuleMgr()
+
+StelModuleMgr::StelModuleMgr() : callingListsToRegenerate(true), pluginDescriptorListLoaded(false)
 {
 	// Initialize empty call lists for each possible actions
 	callOrders[StelModule::ActionDraw]=QList<StelModule*>();
@@ -39,7 +40,6 @@ StelModuleMgr::StelModuleMgr()
 	callOrders[StelModule::ActionHandleMouseClicks]=QList<StelModule*>();
 	callOrders[StelModule::ActionHandleMouseMoves]=QList<StelModule*>();
 	callOrders[StelModule::ActionHandleKeys]=QList<StelModule*>();
-	callingListsToRegenerate = false;
 }
 
 StelModuleMgr::~StelModuleMgr()
@@ -61,13 +61,13 @@ void StelModuleMgr::registerModule(StelModule* m, bool fgenerateCallingLists)
 {
 	QString name = m->objectName();
 	if (modules.find(name) != modules.end())
-	{		
+	{
 		qWarning() << "Module \"" << name << "\" is already loaded.";
 		return;
 	}
 	modules.insert(name, m);
 	m->setParent(this);
-	
+
 	if (fgenerateCallingLists)
 		generateCallingLists();
 }
@@ -87,18 +87,22 @@ void StelModuleMgr::unloadModule(const QString& moduleID, bool alsoDelete)
 	m->setParent(NULL);
 	callingListsToRegenerate = true;
 	if (alsoDelete)
+	{
+		m->deinit();
 		m->deleteLater();
+	}
 }
 
 /*************************************************************************
  Get the corresponding module or NULL if can't find it.
 *************************************************************************/
-StelModule* StelModuleMgr::getModule(const QString& moduleID)
+StelModule* StelModuleMgr::getModule(const QString& moduleID, bool noWarning)
 {
 	QMap<QString, StelModule*>::const_iterator iter = modules.find(moduleID);
 	if (iter==modules.end())
 	{
-		qWarning() << "Warning can't find module called " << moduleID << ".";
+		if (noWarning==false)
+			qWarning() << "Warning can't find module called " << moduleID << ".";
 		return NULL;
 	}
 	return iter.value();
@@ -110,45 +114,19 @@ StelModule* StelModuleMgr::getModule(const QString& moduleID)
 *************************************************************************/
 StelModule* StelModuleMgr::loadPlugin(const QString& moduleID)
 {
-	QString moduleFullPath = "modules/" + moduleID + "/lib" + moduleID;
-#ifdef WIN32
-	moduleFullPath += ".dll";
-#else
-#ifdef MACOSX
-	moduleFullPath += ".dylib";
-#else
-	moduleFullPath += ".so";
-#endif
-#endif
-	try
+	foreach (const PluginDescriptor& desc, getPluginsList())
 	{
-		moduleFullPath = StelApp::getInstance().getFileMgr().findFile(moduleFullPath, StelFileMgr::File);
+		if (desc.info.id==moduleID)
+		{
+			Q_ASSERT(desc.pluginInterface);
+			StelModule* sMod = desc.pluginInterface->getStelModule();
+			qDebug() << "Loaded plugin " << moduleID << ".";
+			pluginDescriptorList[moduleID].loaded=true;
+			return sMod;
+		}
 	}
-	catch (std::runtime_error& e)
-	{
-		qWarning() << "ERROR while locating plugin path: " << e.what();
-	}
-
-	QPluginLoader loader(moduleFullPath);
-	if (!loader.load())
-	{
-		qWarning() << "Couldn't load the dynamic library: " << moduleFullPath << ": " << loader.errorString();
-		qWarning() << "Plugin " << moduleID << " will not be loaded.";
-		return NULL;
-	}
-	
-	QObject* obj = loader.instance();
-	if (!obj)
-	{
-		qWarning() << "Couldn't open the dynamic library: " << moduleFullPath << ": " << loader.errorString();
-		qWarning() << "Plugin " << moduleID << " will not be open.";
-		return NULL;
-	}
-	
-	StelPluginInterface* plugInt = qobject_cast<StelPluginInterface *>(obj);
-	StelModule* sMod = plugInt->getStelModule();
-	qDebug() << "Loaded plugin " << moduleID << ".";
-	return sMod;
+	qWarning() << "Can't find plugin called " << moduleID;
+	return NULL;
 }
 
 struct StelModuleOrderComparator
@@ -168,10 +146,20 @@ void StelModuleMgr::unloadAllPlugins()
 	while (i.hasPrevious())
 	{
 		const PluginDescriptor& d = i.previous();
-		if (d.loadAtStartup==false)
+		if (d.loaded==false)
 			continue;
-		unloadModule(d.key, true);
-		qDebug() << "Unloaded plugin " << d.key << ".";
+		unloadModule(d.info.id, true);
+		qDebug() << "Unloaded plugin " << d.info.id << ".";
+	}
+}
+
+void StelModuleMgr::setPluginLoadAtStartup(const QString& key, bool b)
+{
+	QSettings* conf = StelApp::getInstance().getSettings();
+	conf->setValue("plugins_load_at_startup/"+key, b);
+	if (pluginDescriptorList.contains(key))
+	{
+		pluginDescriptorList[key].loadAtStartup=b;
 	}
 }
 
@@ -201,39 +189,92 @@ void StelModuleMgr::generateCallingLists()
 *************************************************************************/
 QList<StelModuleMgr::PluginDescriptor> StelModuleMgr::getPluginsList()
 {
-	QList<StelModuleMgr::PluginDescriptor> result;
-	QSet<QString> moduleDirs;
-	
-	StelFileMgr& fileMan(StelApp::getInstance().getFileMgr());
-	
-	try
+	if (pluginDescriptorListLoaded)
 	{
-		moduleDirs = fileMan.listContents("modules",StelFileMgr::Directory);
+		return pluginDescriptorList.values();
 	}
-	catch(std::runtime_error& e)
+
+	// First list all static plugins.
+	// If a dynamic plugin with the same ID exists, it will take precedence on the static one.
+	foreach (QObject *plugin, QPluginLoader::staticInstances())
 	{
-		qWarning() << "ERROR while trying list list modules:" << e.what();	
-	}
-	
-	for (QSet<QString>::iterator dir=moduleDirs.begin(); dir!=moduleDirs.end(); dir++)
-	{
-		try
+		StelPluginInterface* pluginInterface = qobject_cast<StelPluginInterface*>(plugin);
+		if (pluginInterface)
 		{
 			StelModuleMgr::PluginDescriptor mDesc;
-			QSettings pd(fileMan.findFile("modules/" + *dir + "/module.ini"), StelIniFormat);
-			mDesc.key = *dir;
-			mDesc.name = pd.value("module/name").toString();
-			mDesc.author = pd.value("module/author").toString();
-			mDesc.contact = pd.value("module/contact").toString();
-			mDesc.description = pd.value("module/description").toString();
-			mDesc.loadAtStartup = pd.value("module/load_at_startup").toBool();
-			result.push_back(mDesc);
-		}
-		catch (std::runtime_error& e)
-		{
-			qWarning() << "WARNING: unable to successfully read module.ini file from plugin " << *dir;
+			mDesc.info = pluginInterface->getPluginInfo();
+			mDesc.pluginInterface = pluginInterface;
+			pluginDescriptorList.insert(mDesc.info.id, mDesc);
 		}
 	}
 
-	return result;
+	// Then list dynamic libraries from the modules/ directory
+	QSet<QString> moduleDirs;
+	try
+	{
+		moduleDirs = StelFileMgr::listContents("modules",StelFileMgr::Directory);
+	}
+	catch(std::runtime_error& e)
+	{
+		qWarning() << "ERROR while trying list list modules:" << e.what();
+	}
+
+	foreach (QString dir, moduleDirs)
+	{
+		QString moduleFullPath = QString("modules/") + dir + "/lib" + dir;
+#ifdef WIN32
+		moduleFullPath += ".dll";
+#else
+#ifdef MACOSX
+		moduleFullPath += ".dylib";
+#else
+		moduleFullPath += ".so";
+#endif
+#endif
+		try
+		{
+			moduleFullPath = StelFileMgr::findFile(moduleFullPath, StelFileMgr::File);
+		}
+		catch (std::runtime_error& e)
+		{
+			continue;
+		}
+
+		QPluginLoader loader(moduleFullPath);
+		if (!loader.load())
+		{
+			qWarning() << "Couldn't load the dynamic library: " << moduleFullPath << ": " << loader.errorString();
+			qWarning() << "Plugin " << dir << " will not be loaded.";
+			continue;
+		}
+
+		QObject* obj = loader.instance();
+		if (!obj)
+		{
+			qWarning() << "Couldn't open the dynamic library: " << moduleFullPath << ": " << loader.errorString();
+			qWarning() << "Plugin " << dir << " will not be open.";
+			continue;
+		}
+
+		StelPluginInterface* pluginInterface = qobject_cast<StelPluginInterface *>(obj);
+		if (pluginInterface)
+		{
+			StelModuleMgr::PluginDescriptor mDesc;
+			mDesc.info = pluginInterface->getPluginInfo();
+			mDesc.pluginInterface = pluginInterface;
+			pluginDescriptorList.insert(mDesc.info.id, mDesc);
+		}
+	}
+
+	// Load for each plugin if it should be loaded at startup
+	QSettings* conf = StelApp::getInstance().getSettings();
+	for (QMap<QString, StelModuleMgr::PluginDescriptor>::Iterator iter=pluginDescriptorList.begin();iter!=pluginDescriptorList.end();++iter)
+	{
+		if (!conf->contains("plugins_load_at_startup/"+iter.key()))
+			conf->setValue("plugins_load_at_startup/"+iter.key(), false);
+		iter->loadAtStartup = conf->value("plugins_load_at_startup/"+iter.key()).toBool();
+	}
+
+	pluginDescriptorListLoaded = true;
+	return pluginDescriptorList.values();
 }
