@@ -1,17 +1,17 @@
 /*
  * Stellarium
  * Copyright (C) 2007 Fabien Chereau
- * 
+ *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
  * as published by the Free Software Foundation; either version 2
  * of the License, or (at your option) any later version.
- * 
+ *
  * This program is distributed in the hope that it will be useful,
  * but WITHOUT ANY WARRANTY; without even the implied warranty of
  * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  * GNU General Public License for more details.
- * 
+ *
  * You should have received a copy of the GNU General Public License
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
@@ -23,6 +23,7 @@
 #include "StelApp.hpp"
 #include "StelCore.hpp"
 #include "StelNavigator.hpp"
+#include "StelScriptMgr.hpp"
 #include "StelUtils.hpp"
 
 #include <QString>
@@ -31,7 +32,7 @@
 #include <QKeyEvent>
 #include <QDebug>
 
-StelMovementMgr::StelMovementMgr(StelCore* acore) : core(acore), 
+StelMovementMgr::StelMovementMgr(StelCore* acore) : core(acore),
 	flagLockEquPos(false),
 	flagTracking(false),
 	isMouseMovingHoriz(false),
@@ -39,7 +40,7 @@ StelMovementMgr::StelMovementMgr(StelCore* acore) : core(acore),
 	flagEnableMouseNavigation(true),
 	keyMoveSpeed(0.00025),
 	flagMoveSlow(false),
-	flagAutoMove(0),
+	flagAutoMove(false),
 	deltaFov(0.),
 	deltaAlt(0.),
 	deltaAz(0.),
@@ -48,6 +49,7 @@ StelMovementMgr::StelMovementMgr(StelCore* acore) : core(acore),
 {
 	setObjectName("StelMovementMgr");
 	isDragging = false;
+	mountMode = MountAltAzimuthal;  // default
 }
 
 StelMovementMgr::~StelMovementMgr()
@@ -61,6 +63,7 @@ void StelMovementMgr::init()
 	Q_ASSERT(conf);
 	Q_ASSERT(objectMgr);
 
+	movementsSpeedFactor=1.;
 	flagEnableMoveAtScreenEdge = conf->value("navigation/flag_enable_move_at_screen_edge",false).toBool();
 	mouseZoomSpeed = conf->value("navigation/mouse_zoom",30).toInt();
 	flagEnableZoomKeys = conf->value("navigation/flag_enable_zoom_keys").toBool();
@@ -71,13 +74,46 @@ void StelMovementMgr::init()
 	flagManualZoom = conf->value("navigation/flag_manual_zoom").toBool();
 	flagAutoZoomOutResetsDirection = conf->value("navigation/auto_zoom_out_resets_direction", true).toBool();
 	flagEnableMouseNavigation = conf->value("navigation/flag_enable_mouse_navigation",true).toBool();
-	
+
 	minFov = 0.0001;
-	maxFov = 100.; 
+	maxFov = 100.;
 	initFov = conf->value("navigation/init_fov",60.).toDouble();
 	currentFov = initFov;
-}	
-	
+
+	initViewPos = StelUtils::strToVec3f(conf->value("navigation/init_view_pos").toString());
+	viewDirectionJ2000 = core->getNavigator()->altAzToJ2000(initViewPos);
+
+	QString tmpstr = conf->value("navigation/viewing_mode", "horizon").toString();
+	if (tmpstr=="equator")
+		setMountMode(StelMovementMgr::MountEquinoxEquatorial);
+	else
+	{
+		if (tmpstr=="horizon")
+			setMountMode(StelMovementMgr::MountAltAzimuthal);
+		else
+		{
+			qWarning() << "ERROR : Unknown viewing mode type : " << tmpstr;
+			setMountMode(StelMovementMgr::MountEquinoxEquatorial);
+		}
+	}
+}
+
+void StelMovementMgr::setMountMode(MountMode m)
+{
+	mountMode = m;
+	setViewDirectionJ2000(viewDirectionJ2000);
+}
+
+void StelMovementMgr::setFlagLockEquPos(bool b)
+{
+	flagLockEquPos=b;
+}
+
+Vec3d StelMovementMgr::getViewUpVectorJ2000() const
+{
+	return mountFrameToJ2000(Vec3d(0,0,1));
+}
+
 bool StelMovementMgr::handleMouseMoves(int x, int y, Qt::MouseButtons b)
 {
 	// Turn if the mouse is at the edge of the screen unless config asks otherwise
@@ -115,7 +151,7 @@ bool StelMovementMgr::handleMouseMoves(int x, int y, Qt::MouseButtons b)
 			isMouseMovingVert = false;
 		}
 	}
-	
+
 	if (isDragging && flagEnableMouseNavigation)
 	{
 		if (hasDragged || (std::sqrt((double)((x-previousX)*(x-previousX) +(y-previousY)*(y-previousY)))>4.))
@@ -237,7 +273,7 @@ void StelMovementMgr::handleMouseClicks(QMouseEvent* event)
 			{
 				if (objectMgr->getWasSelected())
 				{
-					moveTo(objectMgr->getSelectedObject()[0]->getEquinoxEquatorialPos(core->getNavigator()),autoMoveDuration);
+					moveToObject(objectMgr->getSelectedObject()[0],autoMoveDuration);
 					setFlagTracking(true);
 				}
 			}
@@ -247,9 +283,16 @@ void StelMovementMgr::handleMouseClicks(QMouseEvent* event)
 	return;
 }
 
+void StelMovementMgr::setInitViewDirectionToCurrent()
+{
+	initViewPos = core->getNavigator()->j2000ToAltAz(viewDirectionJ2000);
+	QString dirStr = QString("%1,%2,%3").arg(initViewPos[0]).arg(initViewPos[1]).arg(initViewPos[2]);
+	StelApp::getInstance().getSettings()->setValue("navigation/init_view_pos", dirStr);
+}
+
 /*************************************************************************
  The selected objects changed, follow it it we were already following another one
-*************************************************************************/ 
+*************************************************************************/
 void StelMovementMgr::selectedObjectChangeCallBack(StelModuleSelectAction action)
 {
 	// If an object was selected keep the earth following
@@ -326,12 +369,11 @@ void StelMovementMgr::zoomOut(bool s)
 // Increment/decrement smoothly the vision field and position
 void StelMovementMgr::updateMotion(double deltaTime)
 {
-	const StelProjectorP proj = core->getProjection(StelCore::FrameJ2000);
-	
 	updateVisionVector(deltaTime);
-	
+
+	const StelProjectorP proj = core->getProjection(StelCore::FrameJ2000);
 	// the more it is zoomed, the lower the moving speed is (in angle)
-	double depl=keyMoveSpeed*deltaTime*1000*currentFov; 
+	double depl=keyMoveSpeed*deltaTime*1000*currentFov;
 	double deplzoom=keyZoomSpeed*deltaTime*1000*proj->deltaZoom(currentFov*(M_PI/360.0))*(360.0/M_PI);
 
 	if (flagMoveSlow)
@@ -396,18 +438,110 @@ void StelMovementMgr::updateMotion(double deltaTime)
 }
 
 
+void StelMovementMgr::updateVisionVector(double deltaTime)
+{
+	if (flagAutoMove)
+	{
+		if (!move.targetObject.isNull())
+		{
+			// if zooming in, object may be moving so be sure to zoom to latest position
+			move.aim = move.targetObject->getJ2000EquatorialPos(core->getNavigator());
+			move.aim.normalize();
+			move.aim*=2.;
+		}
+
+		move.coef+=move.speed*deltaTime*1000;
+		if (move.coef>=1.)
+		{
+			flagAutoMove=false;
+			move.coef=1.;
+		}
+
+		// Use a smooth function
+		float smooth = 4.f;
+		double c;
+		if (zoomingMode==1)
+		{
+			if (move.coef>.9)
+			{
+				c = 1.;
+			}
+			else
+			{
+				c = 1. - pow(1.-1.11*move.coef,3.);
+			}
+		}
+		else if (zoomingMode==-1)
+		{
+			if (move.coef<0.1)
+			{
+				// keep in view at first as zoom out
+				c = 0;
+			}
+			else
+			{
+				c =  pow(1.11*(move.coef-.1),3.);
+			}
+		}
+		else
+			c = std::atan(smooth * 2.*move.coef-smooth)/std::atan(smooth)/2+0.5;
+
+		Vec3d tmpStart(j2000ToMountFrame(move.start));
+		Vec3d tmpAim(j2000ToMountFrame(move.aim));
+		double ra_aim, de_aim, ra_start, de_start;
+		StelUtils::rectToSphe(&ra_start, &de_start, tmpStart);
+		StelUtils::rectToSphe(&ra_aim, &de_aim, tmpAim);
+
+		// Trick to choose the good moving direction and never travel on a distance > PI
+		if (ra_aim-ra_start > M_PI)
+		{
+			ra_aim -= 2.*M_PI;
+		}
+		else if (ra_aim-ra_start < -M_PI)
+		{
+			ra_aim += 2.*M_PI;
+		}
+		const double de_now = de_aim*c + de_start*(1.-c);
+		const double ra_now = ra_aim*c + ra_start*(1.-c);
+		Vec3d tmp;
+		StelUtils::spheToRect(ra_now, de_now, tmp);
+		setViewDirectionJ2000(mountFrameToJ2000(tmp));
+	}
+	else
+	{
+		if (flagTracking && objectMgr->getWasSelected()) // Equatorial vision vector locked on selected object
+		{
+			setViewDirectionJ2000(objectMgr->getSelectedObject()[0]->getJ2000EquatorialPos(core->getNavigator()));
+		}
+		else
+		{
+			if (flagLockEquPos) // Equatorial vision vector locked
+			{
+				// Recalc local vision vector
+				setViewDirectionJ2000(viewDirectionJ2000);
+			}
+			else
+			{
+				// Vision vector locked to its position in the mountFrame
+				setViewDirectionJ2000(mountFrameToJ2000(viewDirectionMountFrame));
+			}
+		}
+	}
+}
+
 // Go and zoom to the selected object.
 void StelMovementMgr::autoZoomIn(float moveDuration, bool allowManualZoom)
 {
 	if (!objectMgr->getWasSelected())
 		return;
-		
-	float manualMoveDuration;
 
+	moveDuration /= movementsSpeedFactor;
+
+	float manualMoveDuration;
 	if (!getFlagTracking())
 	{
 		setFlagTracking(true);
-		moveTo(objectMgr->getSelectedObject()[0]->getEquinoxEquatorialPos(core->getNavigator()), moveDuration, false, 1);
+		moveToObject(objectMgr->getSelectedObject()[0], moveDuration, 1);
 		manualMoveDuration = moveDuration;
 	}
 	else
@@ -441,8 +575,8 @@ void StelMovementMgr::autoZoomIn(float moveDuration, bool allowManualZoom)
 // Unzoom and go to the init position
 void StelMovementMgr::autoZoomOut(float moveDuration, bool full)
 {
-	StelNavigator* nav = core->getNavigator();
-	
+	moveDuration /= movementsSpeedFactor;
+
 	if (objectMgr->getWasSelected() && !full)
 	{
 		// If the selected object has satellites, unzoom to satellites view
@@ -466,8 +600,8 @@ void StelMovementMgr::autoZoomOut(float moveDuration, bool full)
 	}
 
 	zoomTo(initFov, moveDuration);
-	if (flagAutoZoomOutResetsDirection) 
-		moveTo(nav->getInitViewingDirection(), moveDuration, true, -1);
+	if (flagAutoZoomOutResetsDirection)
+		moveToJ2000(core->getNavigator()->altAzToJ2000(getInitViewingDirection()), moveDuration, -1);
 	setFlagTracking(false);
 	setFlagLockEquPos(false);
 }
@@ -475,176 +609,95 @@ void StelMovementMgr::autoZoomOut(float moveDuration, bool full)
 
 void StelMovementMgr::setFlagTracking(bool b)
 {
-	if(!b || !objectMgr->getWasSelected())
+	if (!b || !objectMgr->getWasSelected())
 	{
 		flagTracking=false;
 	}
 	else
 	{
-		moveTo(objectMgr->getSelectedObject()[0]->getEquinoxEquatorialPos(core->getNavigator()), getAutoMoveDuration());
+		moveToObject(objectMgr->getSelectedObject()[0], getAutoMoveDuration());
 		flagTracking=true;
 	}
 }
 
 
 ////////////////////////////////////////////////////////////////////////////////
-// Move to the given equatorial position
-void StelMovementMgr::moveTo(const Vec3d& _aim, float moveDuration, bool _localPos, int zooming)
+// Move to the given J2000 equatorial position
+void StelMovementMgr::moveToJ2000(const Vec3d& aim, float moveDuration, int zooming)
 {
-	StelNavigator* nav = core->getNavigator();
+	moveDuration /= movementsSpeedFactor;
+
 	zoomingMode = zooming;
-	move.aim=_aim;
+	move.aim=aim;
 	move.aim.normalize();
 	move.aim*=2.;
-	if (_localPos)
-	{
-		move.start=nav->getAltAzVisionDirection();
-	}
-	else
-	{
-		move.start=nav->getEquinoxEquVisionDirection();
-	}
+	move.start=viewDirectionJ2000;
 	move.start.normalize();
 	move.speed=1.f/(moveDuration*1000);
 	move.coef=0.;
-	move.localPos = _localPos;
+	move.targetObject.clear();
 	flagAutoMove = true;
 }
 
-
-////////////////////////////////////////////////////////////////////////////////
-void StelMovementMgr::updateVisionVector(double deltaTime)
+void StelMovementMgr::moveToObject(const StelObjectP& target, float moveDuration, int zooming)
 {
-	StelNavigator* nav = core->getNavigator();
-	if (flagAutoMove)
-	{
-		double ra_aim, de_aim, ra_start, de_start, ra_now, de_now;
+	moveDuration /= movementsSpeedFactor;
 
-		if( zoomingMode == 1 && objectMgr->getWasSelected())
-		{
-			// if zooming in, object may be moving so be sure to zoom to latest position
-			move.aim = objectMgr->getSelectedObject()[0]->getEquinoxEquatorialPos(core->getNavigator());
-			move.aim.normalize();
-			move.aim*=2.;
-		}
-
-		// Use a smooth function
-		float smooth = 4.f;
-		double c;
-
-		if (zoomingMode == 1)
-		{
-			if( move.coef > .9 )
-			{
-				c = 1;
-			}
-			else
-			{
-				c = 1 - pow(1.-1.11*(move.coef),3);
-			}
-		}
-		else if(zoomingMode == -1)
-		{
-			if( move.coef < 0.1 )
-			{
-				// keep in view at first as zoom out
-				c = 0;
-
-				/* could track as moves too, but would need to know if start was actually
-				   a zoomed in view on the object or an extraneous zoom out command
-				   if(move.localPos) {
-				   move.start=equinoxEquToAltAz(selected.getEquinoxEquatorialPos(this));
-				   } else {
-				   move.start=selected.getEquinoxEquatorialPos(this);
-				   }
-				   move.start.normalize();
-				*/
-
-			}
-			else
-			{
-				c =  pow(1.11*(move.coef-.1),3);
-			}
-		}
-		else c = std::atan(smooth * 2.*move.coef-smooth)/std::atan(smooth)/2+0.5;
-
-
-		if (move.localPos)
-		{
-			StelUtils::rectToSphe(&ra_aim, &de_aim, move.aim);
-			StelUtils::rectToSphe(&ra_start, &de_start, move.start);
-		}
-		else
-		{
-			StelUtils::rectToSphe(&ra_aim, &de_aim, nav->equinoxEquToAltAz(move.aim));
-			StelUtils::rectToSphe(&ra_start, &de_start, nav->equinoxEquToAltAz(move.start));
-		}
-		
-		// Trick to choose the good moving direction and never travel on a distance > PI
-		if (ra_aim-ra_start > M_PI)
-		{
-			ra_aim -= 2.*M_PI;
-		}
-		else if (ra_aim-ra_start < -M_PI)
-		{
-			ra_aim += 2.*M_PI;
-		}
-		
-		de_now = de_aim*c + de_start*(1. - c);
-		ra_now = ra_aim*c + ra_start*(1. - c);
-		
-		Vec3d tmp;
-		StelUtils::spheToRect(ra_now, de_now, tmp);
-		nav->setEquinoxEquVisionDirection(nav->altAzToEquinoxEqu(tmp));
-
-		move.coef+=move.speed*deltaTime*1000;
-		if (move.coef>=1.)
-		{
-			flagAutoMove=0;
-			if (move.localPos)
-			{
-				nav->setAltAzVisionDirection(move.aim);
-			}
-			else
-			{
-				nav->setEquinoxEquVisionDirection(move.aim);
-			}
-		}
-	}
-	else
-	{
-		if (flagTracking && objectMgr->getWasSelected()) // Equatorial vision vector locked on selected object
-		{
-			nav->setEquinoxEquVisionDirection(objectMgr->getSelectedObject()[0]->getEquinoxEquatorialPos(core->getNavigator()));
-		}
-		else
-		{
-			if (flagLockEquPos) // Equatorial vision vector locked
-			{
-				// Recalc local vision vector
-				nav->setAltAzVisionDirection(nav->equinoxEquToAltAz(nav->getEquinoxEquVisionDirection()));
-			}
-			else // Local vision vector locked
-			{
-				// Recalc equatorial vision vector
-				nav->setEquinoxEquVisionDirection(nav->altAzToEquinoxEqu(nav->getAltAzVisionDirection()));
-			}
-		}
-	}
+	zoomingMode = zooming;
+	move.aim=Vec3d(0);
+	move.start=viewDirectionJ2000;
+	move.start.normalize();
+	move.speed=1.f/(moveDuration*1000);
+	move.coef=0.;
+	move.targetObject = target;
+	flagAutoMove = true;
 }
 
+Vec3d StelMovementMgr::j2000ToMountFrame(const Vec3d& v) const
+{
+	switch (mountMode)
+	{
+		case MountAltAzimuthal:
+			return core->getNavigator()->j2000ToAltAz(v);
+		case MountEquinoxEquatorial:
+			return core->getNavigator()->j2000ToEquinoxEqu(v);
+		case MountGalactic:
+			return core->getNavigator()->j2000ToGalactic(v);
+	}
+	Q_ASSERT(0);
+	return Vec3d(0);
+}
+
+Vec3d StelMovementMgr::mountFrameToJ2000(const Vec3d& v) const
+{
+	switch (mountMode)
+	{
+		case MountAltAzimuthal:
+			return core->getNavigator()->altAzToJ2000(v);
+		case MountEquinoxEquatorial:
+			return core->getNavigator()->equinoxEquToJ2000(v);
+		case MountGalactic:
+			return core->getNavigator()->galacticToJ2000(v);
+	}
+	Q_ASSERT(0);
+	return Vec3d(0);
+}
+
+void StelMovementMgr::setViewDirectionJ2000(const Vec3d& v)
+{
+	core->getNavigator()->lookAtJ2000(v, getViewUpVectorJ2000());
+	viewDirectionJ2000 = v;
+	viewDirectionMountFrame = j2000ToMountFrame(v);
+}
 
-////////////////////////////////////////////////////////////////////////////////
 void StelMovementMgr::panView(double deltaAz, double deltaAlt)
 {
-	StelNavigator* nav = core->getNavigator();
 	double azVision, altVision;
-
-	if( nav->getViewingMode() == StelNavigator::ViewEquator) StelUtils::rectToSphe(&azVision,&altVision,nav->getEquinoxEquVisionDirection());
-	else StelUtils::rectToSphe(&azVision,&altVision,nav->getAltAzVisionDirection());
+	StelUtils::rectToSphe(&azVision,&altVision,j2000ToMountFrame(viewDirectionJ2000));
 
 	// if we are moving in the Azimuthal angle (left/right)
-	if (deltaAz) azVision-=deltaAz;
+	if (deltaAz)
+		azVision-=deltaAz;
 	if (deltaAlt)
 	{
 		if (altVision+deltaAlt <= M_PI_2 && altVision+deltaAlt >= -M_PI_2) altVision+=deltaAlt;
@@ -656,19 +709,9 @@ void StelMovementMgr::panView(double deltaAz, double deltaAlt)
 	if (deltaAz || deltaAlt)
 	{
 		setFlagTracking(false);
-		if( nav->getViewingMode() == StelNavigator::ViewEquator)
-		{
-			Vec3d tmp;
-			StelUtils::spheToRect(azVision, altVision, tmp);
-			nav->setAltAzVisionDirection(nav->equinoxEquToAltAz(tmp));
-		}
-		else
-		{
-			Vec3d tmp;
-			StelUtils::spheToRect(azVision, altVision, tmp);
-			// Calc the equatorial coordinate of the direction of vision wich was in Altazimuthal coordinate
-			nav->setEquinoxEquVisionDirection(nav->altAzToEquinoxEqu(tmp));
-		}
+		Vec3d tmp;
+		StelUtils::spheToRect(azVision, altVision, tmp);
+		setViewDirectionJ2000(mountFrameToJ2000(tmp));
 	}
 }
 
@@ -676,20 +719,13 @@ void StelMovementMgr::panView(double deltaAz, double deltaAlt)
 //! Make the first screen position correspond to the second (useful for mouse dragging)
 void StelMovementMgr::dragView(int x1, int y1, int x2, int y2)
 {
-	StelNavigator* nav = core->getNavigator();
-	
 	Vec3d tempvec1, tempvec2;
 	double az1, alt1, az2, alt2;
-	const StelProjectorP prj = nav->getViewingMode()==StelNavigator::ViewHorizon ? core->getProjection(StelCore::FrameAltAz) :
-		core->getProjection(StelCore::FrameEquinoxEqu);
-		
-//johannes: StelApp already gives appropriate x/y coordinates
-//	proj->unProject(x2,proj->getViewportHeight()-y2, tempvec2);
-//	proj->unProject(x1,proj->getViewportHeight()-y1, tempvec1);
+	const StelProjectorP prj = core->getProjection(StelCore::FrameJ2000);
 	prj->unProject(x2,y2, tempvec2);
 	prj->unProject(x1,y1, tempvec1);
-	StelUtils::rectToSphe(&az1, &alt1, tempvec1);
-	StelUtils::rectToSphe(&az2, &alt2, tempvec2);
+	StelUtils::rectToSphe(&az1, &alt1, j2000ToMountFrame(tempvec1));
+	StelUtils::rectToSphe(&az2, &alt2, j2000ToMountFrame(tempvec2));
 	panView(az2-az1, alt1-alt2);
 	setFlagTracking(false);
 	setFlagLockEquPos(false);
@@ -728,11 +764,13 @@ void StelMovementMgr::updateAutoZoom(double deltaTime)
 // Zoom to the given field of view
 void StelMovementMgr::zoomTo(double aim_fov, float moveDuration)
 {
+	moveDuration /= movementsSpeedFactor;
+
 	zoomMove.aim=aim_fov;
-    zoomMove.start=currentFov;
-    zoomMove.speed=1.f/(moveDuration*1000);
-    zoomMove.coef=0.;
-    flagAutoZoom = true;
+	zoomMove.start=currentFov;
+	zoomMove.speed=1.f/(moveDuration*1000);
+	zoomMove.coef=0.;
+	flagAutoZoom = true;
 }
 
 void StelMovementMgr::changeFov(double deltaFov)
