@@ -16,6 +16,7 @@
  * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
  */
 
+#include "StelTextureMgr.hpp"
 #include "StelSkyImageTile.hpp"
 #include "StelApp.hpp"
 #include "StelFileMgr.hpp"
@@ -24,18 +25,15 @@
 #include "StelProjector.hpp"
 #include "StelToneReproducer.hpp"
 #include "StelCore.hpp"
-#include "StelTextureMgr.hpp"
 #include "StelSkyDrawer.hpp"
 #include "StelPainter.hpp"
 
 #include <QDebug>
 
-#ifdef DEBUG_STELSKYIMAGE_TILE
- #include "StelFont.hpp"
-#include "StelFontMgr.hpp"
-#include "StelLocaleMgr.hpp"
- StelFont* StelSkyImageTile::debugFont = NULL;
-#endif
+StelSkyImageTile::StelSkyImageTile()
+{
+	initCtor();
+}
 
 void StelSkyImageTile::initCtor()
 {
@@ -69,20 +67,20 @@ StelSkyImageTile::StelSkyImageTile(const QVariantMap& map, StelSkyImageTile* par
 	}
 	initFromQVariantMap(map);
 }
-	
+
 // Destructor
 StelSkyImageTile::~StelSkyImageTile()
 {
 }
-	
-void StelSkyImageTile::draw(StelCore* core, const StelPainter& sPainter)
+
+void StelSkyImageTile::draw(StelCore* core, StelPainter& sPainter, float opacity)
 {
 	const StelProjectorP prj = core->getProjection(StelCore::FrameJ2000);
-	
+
 	const float limitLuminance = core->getSkyDrawer()->getLimitLuminance();
 	QMultiMap<double, StelSkyImageTile*> result;
 	getTilesToDraw(result, core, prj->getViewportConvexPolygon(0, 0), limitLuminance, true);
-	
+
 	int numToBeLoaded=0;
 	foreach (StelSkyImageTile* t, result)
 		if (t->isReadyToDisplay()==false)
@@ -90,7 +88,7 @@ void StelSkyImageTile::draw(StelCore* core, const StelPainter& sPainter)
 	updatePercent(result.size(), numToBeLoaded);
 
 	// Draw in the good order
-	glEnable(GL_TEXTURE_2D);
+	sPainter.enableTexture2d(true);
 	glBlendFunc(GL_ONE, GL_ONE);
 	QMap<double, StelSkyImageTile*>::Iterator i = result.end();
 	while (i!=result.begin())
@@ -98,14 +96,14 @@ void StelSkyImageTile::draw(StelCore* core, const StelPainter& sPainter)
 		--i;
 		i.value()->drawTile(core, sPainter);
 	}
-	
+
 	deleteUnusedSubTiles();
 }
-	
+
 // Return the list of tiles which should be drawn.
-void StelSkyImageTile::getTilesToDraw(QMultiMap<double, StelSkyImageTile*>& result, StelCore* core, const StelGeom::ConvexPolygon& viewPortPoly, float limitLuminance, bool recheckIntersect)
+void StelSkyImageTile::getTilesToDraw(QMultiMap<double, StelSkyImageTile*>& result, StelCore* core, const SphericalRegionP& viewPortPoly, float limitLuminance, bool recheckIntersect)
 {
-	
+
 #ifndef NDEBUG
 	// When this method is called, we can assume that:
 	// - the parent tile min resolution was reached
@@ -117,29 +115,29 @@ void StelSkyImageTile::getTilesToDraw(QMultiMap<double, StelSkyImageTile*>& resu
 		Q_ASSERT(isDeletionScheduled()==false);
 		const double degPerPixel = 1./core->getProjection(StelCore::FrameJ2000)->getPixelPerRadAtCenter()*180./M_PI;
 		Q_ASSERT(degPerPixel<parent->minResolution);
-		
+
 		Q_ASSERT(parent->isDeletionScheduled()==false);
 	}
 #endif
-		
+
 	// An error occured during loading
 	if (errorOccured)
 		return;
-	
+
 	// The JSON file is currently being downloaded
 	if (downloading)
 	{
 		//qDebug() << "Downloading " << contructorUrl;
 		return;
 	}
-	
+
 	if (luminance>0 && luminance<limitLuminance)
 	{
 		// Schedule a deletion
 		scheduleChildsDeletion();
 		return;
 	}
-	
+
 	// Check that we are in the screen
 	bool fullInScreen = true;
 	bool intersectScreen = false;
@@ -153,16 +151,16 @@ void StelSkyImageTile::getTilesToDraw(QMultiMap<double, StelSkyImageTile*>& resu
 		}
 		else
 		{
-			foreach (const StelGeom::ConvexPolygon poly, skyConvexPolygons)
+			foreach (const SphericalRegionP& poly, skyConvexPolygons)
 			{
-				if (contains(viewPortPoly, poly))
+				if (viewPortPoly->contains(poly))
 				{
 					intersectScreen = true;
 				}
 				else
 				{
 					fullInScreen = false;
-					if (intersect(viewPortPoly, poly))
+					if (viewPortPoly->intersects(poly))
 						intersectScreen = true;
 				}
 			}
@@ -175,23 +173,18 @@ void StelSkyImageTile::getTilesToDraw(QMultiMap<double, StelSkyImageTile*>& resu
 		scheduleChildsDeletion();
 		return;
 	}
-	
+
 	// The tile is in screen, and it is a precondition that its resolution is higher than the limit
 	// make sure that it's not going to be deleted
 	cancelDeletion();
-	
+
 	if (noTexture==false)
 	{
 		if (!tex)
 		{
 			// The tile has an associated texture, but it is not yet loaded: load it now
 			StelTextureMgr& texMgr=StelApp::getInstance().getTextureManager();
-			texMgr.setDefaultParams();
- 			texMgr.setMipmapsMode(true);
-			texMgr.setMinFilter(GL_LINEAR);
- 			texMgr.setMagFilter(GL_LINEAR);
- 			texMgr.setWrapMode(GL_CLAMP_TO_EDGE);
-			tex = texMgr.createTextureThread(absoluteImageURI);
+			tex = texMgr.createTextureThread(absoluteImageURI, StelTexture::StelTextureParams(true));
 			if (!tex)
 			{
 				qWarning() << "WARNING : Can't create tile: " << absoluteImageURI;
@@ -199,11 +192,11 @@ void StelSkyImageTile::getTilesToDraw(QMultiMap<double, StelSkyImageTile*>& resu
 				return;
 			}
 		}
-		
+
 		// The tile is in screen and has a texture: every test passed :) The tile will be displayed
 		result.insert(minResolution, this);
 	}
-	
+
 	// Check if we reach the resolution limit
 	const double degPerPixel = 1./core->getProjection(StelCore::FrameJ2000)->getPixelPerRadAtCenter()*180./M_PI;
 	if (degPerPixel < minResolution)
@@ -239,84 +232,60 @@ void StelSkyImageTile::getTilesToDraw(QMultiMap<double, StelSkyImageTile*>& resu
 
 // Draw the image on the screen.
 // Assume GL_TEXTURE_2D is enabled
-bool StelSkyImageTile::drawTile(StelCore* core, const StelPainter& sPainter)
-{	
+bool StelSkyImageTile::drawTile(StelCore* core, StelPainter& sPainter)
+{
 	if (!tex->bind())
-	{
 		return false;
-	}
-	
+
 	if (!texFader)
 	{
 		texFader = new QTimeLine(1000, this);
 		texFader->start();
 	}
-	
-	const StelProjectorP prj = core->getProjection(StelCore::FrameJ2000);
-	const float factorX = tex->getCoordinates()[2][0];
-	const float factorY = tex->getCoordinates()[2][1];
 
 	// Draw the real texture for this image
 	const float ad_lum = (luminance>0) ? core->getToneReproducer()->adaptLuminanceScaled(luminance) : 1.f;
+	Vec4f color;
 	if (alphaBlend==true || texFader->state()==QTimeLine::Running)
 	{
 		if (!alphaBlend)
 			glBlendFunc(GL_SRC_ALPHA, GL_ONE_MINUS_SRC_ALPHA); // Normal transparency mode
 		glEnable(GL_BLEND);
-		glColor4f(ad_lum,ad_lum,ad_lum, texFader->currentValue());
+		color.set(ad_lum,ad_lum,ad_lum, texFader->currentValue());
 	}
 	else
 	{
 		glDisable(GL_BLEND);
-		glColor3f(ad_lum,ad_lum,ad_lum);
+		color.set(ad_lum,ad_lum,ad_lum, 1.);
 	}
-	
-	for (int p=0;p<skyConvexPolygons.size();++p)
+
+	sPainter.setColor(color[0], color[1], color[2], color[3]);
+	sPainter.enableTexture2d(true);
+	foreach (const SphericalRegionP& poly, skyConvexPolygons)
 	{
-		const StelGeom::Polygon& poly = skyConvexPolygons.at(p).asPolygon();
-		const QList<Vec2f>& texCoords = textureCoords.at(p);
-				
-		Q_ASSERT((int)poly.size()==texCoords.size());
-						
-		Vec3d win;
-		const int N=poly.size()-1;
-		int idx=N;
-		int diff = 0;
-		// Using TRIANGLE STRIP requires to use the following vertex order N-0,0,N-1,1,N-2,2 etc..
-		glBegin(GL_TRIANGLE_STRIP);
-		for (int i=0;i<=N;++i)
-		{
-			idx = (diff==0 ? N-i/2 : i/2);
-			++diff;
-			if (diff>1) diff=0;
-					
-			glTexCoord2d(texCoords[idx][0]*factorX, texCoords[idx][1]*factorY);
-			prj->project(poly[idx],win);
-			glVertex3dv(win);
-		}
-		glEnd();
+		sPainter.drawSphericalRegion(poly.data(), StelPainter::SphericalPolygonDrawModeTextureFill);
 	}
+
 #ifdef DEBUG_STELSKYIMAGE_TILE
 	if (debugFont==NULL)
 	{
 		debugFont = &StelApp::getInstance().getFontManager().getStandardFont(StelApp::getInstance().getLocaleMgr().getSkyLanguage(), 12);
 	}
-	glColor3f(1.0,0.5,0.5);
-	foreach (const StelGeom::ConvexPolygon& poly, skyConvexPolygons)
+	color.set(1.0,0.5,0.5,1.0);
+	foreach (const SphericalRegionP& poly, skyConvexPolygons)
 	{
 		Vec3d win;
-		Vec3d bary = poly.getBarycenter();
-		prj->project(bary,win);
+		Vec3d bary = poly->getPointInside();
+		sPainter.getProjector()->project(bary,win);
 		sPainter.drawText(debugFont, win[0], win[1], getAbsoluteImageURI());
-		
-		glDisable(GL_TEXTURE_2D);
-		sPainter.drawPolygon(poly);
-		glEnable(GL_TEXTURE_2D);
+		sPainter.enableTexture2d(false);
+		sPainter.drawSphericalRegion(poly.get(), StelPainter::SphericalPolygonDrawModeBoundary, &color);
+		sPainter.enableTexture2d(true);
 	}
 #endif
 	if (!alphaBlend)
 		glBlendFunc(GL_ONE, GL_ONE); // Revert
-	
+
 	return true;
 }
 
@@ -343,13 +312,34 @@ void StelSkyImageTile::loadFromQVariantMap(const QVariantMap& map)
 		serverCredits.fullCredits = sCredits.value("full").toString();
 		serverCredits.infoURL = sCredits.value("infoUrl").toString();
 	}
-	
+	if (map.contains("description"))
+	{
+		htmlDescription = map.value("description").toString();
+		if (parent()==NULL)
+		{
+			htmlDescription+= "<h3>URL: "+contructorUrl+"</h3>";
+		}
+	}
+	else
+	{
+		if (parent()==NULL)
+		{
+			htmlDescription= "<h3>URL: "+contructorUrl+"</h3>";
+		}
+	}
+
 	shortName = map.value("shortName").toString();
+	if (shortName.isEmpty())
+		shortName = "no name";
 	bool ok=false;
+	if (!map.contains("minResolution"))
+		throw std::runtime_error(qPrintable(QString("minResolution is mandatory")));
 	minResolution = map.value("minResolution").toDouble(&ok);
 	if (!ok)
-		throw std::runtime_error("minResolution expect a double value");
-	
+	{
+		throw std::runtime_error(qPrintable(QString("minResolution expect a double value, found: \"%1\"").arg(map.value("minResolution").toString())));
+	}
+
 	if (map.contains("luminance"))
 	{
 		luminance = map.value("luminance").toDouble(&ok);
@@ -357,7 +347,7 @@ void StelSkyImageTile::loadFromQVariantMap(const QVariantMap& map)
 			throw std::runtime_error("luminance expect a float value");
 		qWarning() << "luminance in preview JSON files is deprecated. Replace with maxBrightness.";
 	}
-	
+
 	if (map.contains("maxBrightness"))
 	{
 		luminance = map.value("maxBrightness").toDouble(&ok);
@@ -365,22 +355,29 @@ void StelSkyImageTile::loadFromQVariantMap(const QVariantMap& map)
 			throw std::runtime_error("maxBrightness expect a float value");
 		luminance = StelApp::getInstance().getCore()->getSkyDrawer()->surfacebrightnessToLuminance(luminance);
 	}
-	
+
 	if (map.contains("alphaBlend"))
 	{
 		alphaBlend = map.value("alphaBlend").toBool();
 	}
-	
+
 	// Load the convex polygons (if any)
 	QVariantList polyList = map.value("skyConvexPolygons").toList();
 	if (polyList.empty())
 		polyList = map.value("worldCoords").toList();
 	else
 		qWarning() << "skyConvexPolygons in preview JSON files is deprecated. Replace with worldCoords.";
-	foreach (const QVariant& polyRaDec, polyList)
+
+	// Load the matching textures positions (if any)
+	QVariantList texCoordList = map.value("textureCoords").toList();
+	if (!texCoordList.isEmpty() && polyList.size()!=texCoordList.size())
+			throw std::runtime_error("the number of convex polygons does not match the number of texture space polygon");
+
+	for (int i=0;i<polyList.size();++i)
 	{
-		QList<Vec3d> vertices;
-		foreach (QVariant vRaDec, polyRaDec.toList())
+		const QVariant& polyRaDec = polyList.at(i);
+		QVector<Vec3d> vertices;
+		foreach (const QVariant& vRaDec, polyRaDec.toList())
 		{
 			const QVariantList vl = vRaDec.toList();
 			Vec3d v;
@@ -390,25 +387,32 @@ void StelSkyImageTile::loadFromQVariantMap(const QVariantMap& map)
 			vertices.append(v);
 		}
 		Q_ASSERT(vertices.size()==4);
-		skyConvexPolygons.append(StelGeom::ConvexPolygon(vertices[0], vertices[1], vertices[2], vertices[3]));
-	}
-	
-	// Load the matching textures positions (if any)
-	polyList = map.value("textureCoords").toList();
-	foreach (const QVariant& polyXY, polyList)
-	{
-		QList<Vec2f> vertices;
-		foreach (QVariant vXY, polyXY.toList())
+
+		if (!texCoordList.isEmpty())
 		{
-			const QVariantList vl = vXY.toList();
-			vertices.append(Vec2f(vl.at(0).toDouble(&ok), vl.at(1).toDouble(&ok)));
-			if (!ok)
-				throw std::runtime_error("wrong X and Y, expect a double value");
+			const QVariant& polyXY = texCoordList.at(i);
+			QVector<Vec2f> texCoords;
+			foreach (const QVariant& vXY, polyXY.toList())
+			{
+				const QVariantList vl = vXY.toList();
+				texCoords.append(Vec2f(vl.at(0).toDouble(&ok), vl.at(1).toDouble(&ok)));
+				if (!ok)
+					throw std::runtime_error("wrong X and Y, expect a double value");
+			}
+			Q_ASSERT(texCoords.size()==4);
+
+			SphericalTexturedConvexPolygon* pol = new SphericalTexturedConvexPolygon(vertices, texCoords);
+			Q_ASSERT(pol->checkValid());
+			skyConvexPolygons.append(SphericalRegionP(pol));
+		}
+		else
+		{
+			SphericalConvexPolygon* pol = new SphericalConvexPolygon(vertices);
+			Q_ASSERT(pol->checkValid());
+			skyConvexPolygons.append(SphericalRegionP(pol));
 		}
-		Q_ASSERT(vertices.size()==4);
-		textureCoords.append(vertices);
 	}
-	
+
 	if (map.contains("imageUrl"))
 	{
 		QString imageUrl = map.value("imageUrl").toString();
@@ -420,7 +424,7 @@ void StelSkyImageTile::loadFromQVariantMap(const QVariantMap& map)
 		{
 			try
 			{
-				absoluteImageURI = StelApp::getInstance().getFileMgr().findFile(baseUrl+imageUrl);
+				absoluteImageURI = StelFileMgr::findFile(baseUrl+imageUrl);
 			}
 			catch (std::runtime_error& er)
 			{
@@ -428,12 +432,10 @@ void StelSkyImageTile::loadFromQVariantMap(const QVariantMap& map)
 				absoluteImageURI = imageUrl;
 			}
 		}
-		if (skyConvexPolygons.size()!=textureCoords.size())
-			throw std::runtime_error("the number of convex polygons does not match the number of texture space polygon");
 	}
 	else
 		noTexture = true;
-	
+
 	// This is a list of URLs to the child tiles or a list of already loaded map containing child information
 	// (in this later case, the StelSkyImageTile objects will be created later)
 	subTilesUrls = map.value("subTiles").toList();
@@ -459,7 +461,7 @@ void StelSkyImageTile::loadFromQVariantMap(const QVariantMap& map)
 QVariantMap StelSkyImageTile::toQVariantMap() const
 {
 	QVariantMap res;
-	
+
 	// Image credits
 	QVariantMap imCredits;
 	if (!dataSetCredits.shortCredits.isEmpty())
@@ -470,7 +472,7 @@ QVariantMap StelSkyImageTile::toQVariantMap() const
 		imCredits["infoUrl"]=dataSetCredits.infoURL;
 	if (!imCredits.empty())
 		res["imageCredits"]=imCredits;
-	
+
 	// Server credits
 	QVariantMap serCredits;
 	if (!serverCredits.shortCredits.isEmpty())
@@ -481,10 +483,10 @@ QVariantMap StelSkyImageTile::toQVariantMap() const
 		imCredits["infoUrl"]=serverCredits.infoURL;
 	if (!serCredits.empty())
 		res["serverCredits"]=serCredits;
-	
+
 	// Misc
 	if (!shortName.isEmpty())
-		res["shortName"] = shortName;	
+		res["shortName"] = shortName;
 	if (minResolution>0)
 		 res["minResolution"]=minResolution;
 	if (luminance>0)
@@ -493,51 +495,17 @@ QVariantMap StelSkyImageTile::toQVariantMap() const
 		res["alphaBlend"]=true;
 	if (noTexture==false)
 		res["imageUrl"]=absoluteImageURI;
-	
+
 	// Polygons
-	if (!skyConvexPolygons.isEmpty())
-	{
-		QVariantList polygsL;
-		foreach (const StelGeom::ConvexPolygon& poly, skyConvexPolygons)
-		{
-			QVariantList polyL;
-			for (size_t i=0;i<poly.asPolygon().size();++i)
-			{
-				double ra, dec;
-				StelUtils::rectToSphe(&ra, &dec, poly[i]);
-				QVariantList vL;
-				vL.append(ra);
-				vL.append(dec);
-				polyL.append(vL);
-			}
-			polygsL.append(polyL);
-		}
-		res["worldCoords"]=polygsL;
-	}
-	
+	// TODO
+
 	// textures positions
-	if (!textureCoords.empty())
-	{
-		QVariantList polygsL;
-		foreach (const QList<Vec2f>& poly, textureCoords)
-		{
-			QVariantList polyL;
-			foreach (Vec2f v, poly)
-			{
-				QVariantList vL;
-				vL.append(v[0]);
-				vL.append(v[1]);
-				polyL.append(vL);
-			}
-			polygsL.append(polyL);
-		}
-		res["textureCoords"]=polygsL;
-	}
-	
+	// TODO
+
 	if (!subTilesUrls.empty())
 	{
 		res["subTiles"] = subTilesUrls;
 	}
-	
+
 	return res;
 }
