@@ -26,7 +26,7 @@
 
 #include <cmath>
 #include <QString>
- 
+
 template<class T> class Vector2;
 template<class T> class Vector3;
 template<class T> class Vector4;
@@ -71,29 +71,29 @@ typedef Matrix4<float>	Mat4f;
 template<class T> class Vector2
 {
 public:
-    inline Vector2();
-    inline Vector2(const Vector2<T>&);
-    inline Vector2(T, T);
+	inline Vector2();
+	inline Vector2(const Vector2<T>&);
+	inline Vector2(T, T);
 
 	inline Vector2& operator=(const Vector2<T>&);
 	inline Vector2& operator=(const T*);
-    inline void set(T, T);
+	inline void set(T, T);
 
 	inline bool operator==(const Vector2<T>&) const;
 	inline bool operator!=(const Vector2<T>&) const;
 
 	inline const T& operator[](int x) const;
-    inline T& operator[](int);
+	inline T& operator[](int);
 	inline operator const T*() const;
 	inline operator T*();
 
-    inline Vector2& operator+=(const Vector2<T>&);
-    inline Vector2& operator-=(const Vector2<T>&);
-    inline Vector2& operator*=(T);
+	inline Vector2& operator+=(const Vector2<T>&);
+	inline Vector2& operator-=(const Vector2<T>&);
+	inline Vector2& operator*=(T);
 	inline Vector2& operator/=(T);
 
-    inline Vector2 operator-(const Vector2<T>&) const;
-    inline Vector2 operator+(const Vector2<T>&) const;
+	inline Vector2 operator-(const Vector2<T>&) const;
+	inline Vector2 operator+(const Vector2<T>&) const;
 
 	inline Vector2 operator-() const;
 	inline Vector2 operator+() const;
@@ -102,47 +102,50 @@ public:
 	inline Vector2 operator/(T) const;
 
 
-    inline T dot(const Vector2<T>&) const;
+	inline T dot(const Vector2<T>&) const;
 
-    inline T length() const;
-    inline T lengthSquared() const;
-    inline void normalize();
+	inline T length() const;
+	inline T lengthSquared() const;
+	inline void normalize();
 
-    T v[2];
+	T v[2];
 };
 
+
 //! @class Vector3
 //! A templatized 3d vector compatible with openGL.
 //! Use Vec3d or Vec3f typdef for vectors of double and float respectively.
 template<class T> class Vector3
 {
 public:
-    inline Vector3();
-    inline Vector3(const Vector3&);
+	inline Vector3();
+	inline Vector3(const Vector3&);
 	template <class T2> inline Vector3(const Vector3<T2>&);
-    inline Vector3(T, T, T);
+	inline Vector3(T, T, T);
 	inline Vector3(T);
-	
+
 	inline Vector3& operator=(const Vector3&);
 	inline Vector3& operator=(const T*);
 	template <class T2> inline Vector3& operator=(const Vector3<T2>&);
-    inline void set(T, T, T);
+	inline void set(T, T, T);
 
 	inline bool operator==(const Vector3<T>&) const;
 	inline bool operator!=(const Vector3<T>&) const;
 
-    inline T& operator[](int);
-    inline const T& operator[](int) const;
+	inline T& operator[](int);
+	inline const T& operator[](int) const;
 	inline operator const T*() const;
 	inline operator T*();
+	inline const T* data() const {return v;}
+	inline T* data() {return v;}
 
-    inline Vector3& operator+=(const Vector3<T>&);
-    inline Vector3& operator-=(const Vector3<T>&);
-    inline Vector3& operator*=(T);
+	inline Vector3& operator+=(const Vector3<T>&);
+	inline Vector3& operator-=(const Vector3<T>&);
+	inline Vector3& operator*=(T);
 	inline Vector3& operator/=(T);
 
-    inline Vector3 operator-(const Vector3<T>&) const;
-    inline Vector3 operator+(const Vector3<T>&) const;
+	inline Vector3 operator-(const Vector3<T>&) const;
+	inline Vector3 operator+(const Vector3<T>&) const;
 
 	inline Vector3 operator-() const;
 	inline Vector3 operator+() const;
@@ -151,26 +154,27 @@ public:
 	inline Vector3 operator/(T) const;
 
 
-    inline T dot(const Vector3<T>&) const;
+	inline T dot(const Vector3<T>&) const;
 	inline Vector3 operator^(const Vector3<T>&) const;
 
 	// Return latitude in rad
 	inline T latitude() const;
 	// Return longitude in rad
 	inline T longitude() const;
-	
+
 	// Distance in radian between two
 	inline T angle(const Vector3<T>&) const;
 
-    inline T length() const;
-    inline T lengthSquared() const;
-    inline void normalize();
+	inline T length() const;
+	inline T lengthSquared() const;
+	inline void normalize();
 
 	inline void transfo4d(const Mat4d&);
 	inline void transfo4d(const Mat4f&);
 	T v[3];		// The 3 values
-	
+
 	QString toString() const {return QString("[%1, %2, %3]").arg(v[0]).arg(v[1]).arg(v[2]);}
+	QString toStringLonLat() const {return QString("[") + QString::number(longitude()*180./M_PI, 'g', 12) + "," + QString::number(latitude()*180./M_PI, 'g', 12)+"]";}
 };
 
 
@@ -180,8 +184,8 @@ public:
 template<class T> class Vector4
 {
 public:
-    inline Vector4();
-    inline Vector4(const Vector4<T>&);
+	inline Vector4();
+	inline Vector4(const Vector4<T>&);
 	inline Vector4(const Vector3<T>&);
 	inline Vector4(T, T, T);
 	inline Vector4(T, T, T, T);
@@ -189,23 +193,23 @@ public:
 	inline Vector4& operator=(const Vector4<T>&);
 	inline Vector4& operator=(const Vector3<T>&);
 	inline Vector4& operator=(const T*);
-    inline void set(T, T, T, T);
+	inline void set(T, T, T, T);
 
 	inline bool operator==(const Vector4<T>&) const;
 	inline bool operator!=(const Vector4<T>&) const;
 
-    inline T& operator[](int);
-    inline const T& operator[](int) const;
+	inline T& operator[](int);
+	inline const T& operator[](int) const;
 	inline operator T*();
 	inline operator const T*() const;
 
-    inline Vector4& operator+=(const Vector4<T>&);
-    inline Vector4& operator-=(const Vector4<T>&);
-    inline Vector4& operator*=(T);
+	inline Vector4& operator+=(const Vector4<T>&);
+	inline Vector4& operator-=(const Vector4<T>&);
+	inline Vector4& operator*=(T);
 	inline Vector4& operator/=(T);
 
-    inline Vector4 operator-(const Vector4<T>&) const;
-    inline Vector4 operator+(const Vector4<T>&) const;
+	inline Vector4 operator-(const Vector4<T>&) const;
+	inline Vector4 operator+(const Vector4<T>&) const;
 
 	inline Vector4 operator-() const;
 	inline Vector4 operator+() const;
@@ -214,11 +218,11 @@ public:
 	inline Vector4 operator/(T) const;
 
 
-    inline T dot(const Vector4<T>&) const;
+	inline T dot(const Vector4<T>&) const;
 
-    inline T length() const;
-    inline T lengthSquared() const;
-    inline void normalize();
+	inline T length() const;
+	inline T lengthSquared() const;
+	inline void normalize();
 
 	inline void transfo4d(const Mat4d&);
 
@@ -231,51 +235,60 @@ public:
 template<class T> class Matrix4
 {
  public:
-    Matrix4();
-    Matrix4(const Matrix4<T>& m);
-    Matrix4(T,T,T,T,T,T,T,T,T,T,T,T,T,T,T,T);
-    Matrix4(const T*);
-    Matrix4(const Vector4<T>&, const Vector4<T>&,
-            const Vector4<T>&, const Vector4<T>&);
+	Matrix4();
+	Matrix4(const Matrix4<T>& m);
+	Matrix4(T,T,T,T,T,T,T,T,T,T,T,T,T,T,T,T);
+	Matrix4(const T*);
+	Matrix4(const Vector4<T>&, const Vector4<T>&,
+			const Vector4<T>&, const Vector4<T>&);
 
 	inline Matrix4& operator=(const Matrix4<T>&);
 	inline Matrix4& operator=(const T*);
 	inline void set(T,T,T,T,T,T,T,T,T,T,T,T,T,T,T,T);
 
-    inline T* operator[](int);
+	inline T& operator[](int);
 	inline operator T*();
 	inline operator const T*() const;
 
-    inline Matrix4 operator-(const Matrix4<T>&) const;
-    inline Matrix4 operator+(const Matrix4<T>&) const;
-    inline Matrix4 operator*(const Matrix4<T>&) const;
+	inline Matrix4 operator-(const Matrix4<T>&) const;
+	inline Matrix4 operator+(const Matrix4<T>&) const;
+	inline Matrix4 operator*(const Matrix4<T>&) const;
 
-    inline Vector3<T> operator*(const Vector3<T>&) const;
-    inline Vector3<T> multiplyWithoutTranslation(const Vector3<T>& a) const;
+	inline Vector3<T> operator*(const Vector3<T>&) const;
+	inline Vector3<T> multiplyWithoutTranslation(const Vector3<T>& a) const;
 	inline Vector4<T> operator*(const Vector4<T>&) const;
-	
+
 	inline void transfo(Vector3<T>&) const;
-	
-    static Matrix4<T> identity();
-    static Matrix4<T> translation(const Vector3<T>&);
 
-    //    static Matrix4<T> rotation(const Vector3<T>&);
-    static Matrix4<T> rotation(const Vector3<T>&, T);
-    static Matrix4<T> xrotation(T);
-    static Matrix4<T> yrotation(T);
-    static Matrix4<T> zrotation(T);
-    static Matrix4<T> scaling(const Vector3<T>&);
-    static Matrix4<T> scaling(T);
+	static Matrix4<T> identity();
+	static Matrix4<T> translation(const Vector3<T>&);
 
-    Matrix4<T> transpose() const;
-    Matrix4<T> inverse() const;
+	//    static Matrix4<T> rotation(const Vector3<T>&);
+	static Matrix4<T> rotation(const Vector3<T>&, T);
+	static Matrix4<T> xrotation(T);
+	static Matrix4<T> yrotation(T);
+	static Matrix4<T> zrotation(T);
+	static Matrix4<T> scaling(const Vector3<T>&);
+	static Matrix4<T> scaling(T);
+
+	Matrix4<T> transpose() const;
+	Matrix4<T> inverse() const;
 
 	inline void print(void) const;
 
-    T r[16];
+	T r[16];
 };
 
+//! Serialization routines.
+template<class T> QDataStream& operator<<(QDataStream& out, const Vector2<T>& v) {out << v[0] << v[1]; return out;}
+template<class T> QDataStream& operator<<(QDataStream& out, const Vector3<T>& v) {out << v[0] << v[1] << v[2]; return out;}
+template<class T> QDataStream& operator<<(QDataStream& out, const Vector4<T>& v) {out << v[0] << v[1] << v[2] << v[3]; return out;}
+template<class T> QDataStream& operator<<(QDataStream& out, const Matrix4<T>& m) {out << m[0] << m[1] << m[2] << m[3] << m[4] << m[5] << m[6] << m[7] << m[8] << m[9] << m[10] << m[11] << m[12] << m[13] << m[14] << m[15]; return out;}
 
+template<class T> QDataStream& operator>>(QDataStream& in, Vector2<T>& v) {in >> v[0] >> v[1]; return in;}
+template<class T> QDataStream& operator>>(QDataStream& in, Vector3<T>& v) {in >> v[0] >> v[1] >> v[2]; return in;}
+template<class T> QDataStream& operator>>(QDataStream& in, Vector4<T>& v) {in >> v[0] >> v[1] >> v[2] >> v[3]; return in;}
+template<class T> QDataStream& operator>>(QDataStream& in, Matrix4<T>& m) {in >> m[0] >> m[1] >> m[2] >> m[3] >> m[4] >> m[5] >> m[6] >> m[7] >> m[8] >> m[9] >> m[10] >> m[11] >> m[12] >> m[13] >> m[14] >> m[15]; return in;}
 
 ////////////////////////// Vector2 class methods ///////////////////////////////
 
@@ -294,13 +307,13 @@ template<class T> Vector2<T>::Vector2(T x, T y)
 template<class T> Vector2<T>& Vector2<T>::operator=(const Vector2<T>& a)
 {
 	v[0]=a.v[0]; v[1]=a.v[1];
-    return *this;
+	return *this;
 }
 
 template<class T> Vector2<T>& Vector2<T>::operator=(const T* a)
 {
 	v[0]=a[0]; v[1]=a[1];
-    return *this;
+	return *this;
 }
 
 template<class T> void Vector2<T>::set(T x, T y)
@@ -342,77 +355,77 @@ template<class T> Vector2<T>::operator T*()
 
 template<class T> Vector2<T>& Vector2<T>::operator+=(const Vector2<T>& a)
 {
-    v[0] += a.v[0]; v[1] += a.v[1];
-    return *this;
+	v[0] += a.v[0]; v[1] += a.v[1];
+	return *this;
 }
 
 template<class T> Vector2<T>& Vector2<T>::operator-=(const Vector2<T>& a)
 {
-    v[0] -= a.v[0]; v[1] -= a.v[1];
-    return *this;
+	v[0] -= a.v[0]; v[1] -= a.v[1];
+	return *this;
 }
 
 template<class T> Vector2<T>& Vector2<T>::operator*=(T s)
 {
-    v[0] *= s; v[1] *= s;
-    return *this;
+	v[0] *= s; v[1] *= s;
+	return *this;
 }
 
 template<class T> Vector2<T> Vector2<T>::operator-() const
 {
-    return Vector2<T>(-v[0], -v[1]);
+	return Vector2<T>(-v[0], -v[1]);
 }
 
 template<class T> Vector2<T> Vector2<T>::operator+() const
 {
-    return *this;
+	return *this;
 }
 
 template<class T> Vector2<T> Vector2<T>::operator+(const Vector2<T>& b) const
 {
-    return Vector2<T>(v[0] + b.v[0], v[1] + b.v[1]);
+	return Vector2<T>(v[0] + b.v[0], v[1] + b.v[1]);
 }
 
 template<class T> Vector2<T> Vector2<T>::operator-(const Vector2<T>& b) const
 {
-    return Vector2<T>(v[0] - b.v[0], v[1] - b.v[1]);
+	return Vector2<T>(v[0] - b.v[0], v[1] - b.v[1]);
 }
 
 template<class T> Vector2<T> Vector2<T>::operator*(T s) const
 {
-    return Vector2<T>(s * v[0], s * v[1]);
+	return Vector2<T>(s * v[0], s * v[1]);
 }
 
 template<class T> Vector2<T> Vector2<T>::operator/(T s) const
 {
-    return Vector2<T>(v[0]/s, v[1]/s);
+	return Vector2<T>(v[0]/s, v[1]/s);
 }
 
 
 template<class T> T Vector2<T>::dot(const Vector2<T>& b) const
 {
-    return v[0] * b.v[0] + v[1] * b.v[1];
+	return v[0] * b.v[0] + v[1] * b.v[1];
 }
 
 
 template<class T> T Vector2<T>::length() const
 {
-    return (T) std::sqrt(v[0] * v[0] + v[1] * v[1]);
+	return (T) std::sqrt(v[0] * v[0] + v[1] * v[1]);
 }
 
 template<class T> T Vector2<T>::lengthSquared() const
 {
-    return v[0] * v[0] + v[1] * v[1];
+	return v[0] * v[0] + v[1] * v[1];
 }
 
 template<class T> void Vector2<T>::normalize()
 {
-    T s = (T) 1 / std::sqrt(v[0] * v[0] + v[1] * v[1]);
-    v[0] *= s;
-    v[1] *= s;
+	T s = (T) 1 / std::sqrt(v[0] * v[0] + v[1] * v[1]);
+	v[0] *= s;
+	v[1] *= s;
 }
 
-// template<class T> 
+// template<class T>
 // std::ostream& operator<<(std::ostream &o,const Vector2<T> &v) {
 //   return o << '[' << v[0] << ',' << v[1] << ']';
 // }
@@ -444,19 +457,19 @@ template<class T> Vector3<T>::Vector3(T x, T y, T z)
 template<class T> Vector3<T>& Vector3<T>::operator=(const Vector3& a)
 {
 	v[0]=a.v[0]; v[1]=a.v[1]; v[2]=a.v[2];
-    return *this;
+	return *this;
 }
 
 template<class T> template <class T2> Vector3<T>& Vector3<T>::operator=(const Vector3<T2>& a)
 {
 	v[0]=a.v[0]; v[1]=a.v[1]; v[2]=a.v[2];
-    return *this;
+	return *this;
 }
 
 template<class T> Vector3<T>& Vector3<T>::operator=(const T* a)
 {
 	v[0]=a[0]; v[1]=a[1]; v[2]=a[2];
-    return *this;
+	return *this;
 }
 
 template<class T> void Vector3<T>::set(T x, T y, T z)
@@ -498,20 +511,20 @@ template<class T> Vector3<T>::operator T*()
 
 template<class T> Vector3<T>& Vector3<T>::operator+=(const Vector3<T>& a)
 {
-    v[0] += a.v[0]; v[1] += a.v[1]; v[2] += a.v[2];
-    return *this;
+	v[0] += a.v[0]; v[1] += a.v[1]; v[2] += a.v[2];
+	return *this;
 }
 
 template<class T> Vector3<T>& Vector3<T>::operator-=(const Vector3<T>& a)
 {
-    v[0] -= a.v[0]; v[1] -= a.v[1]; v[2] -= a.v[2];
-    return *this;
+	v[0] -= a.v[0]; v[1] -= a.v[1]; v[2] -= a.v[2];
+	return *this;
 }
 
 template<class T> Vector3<T>& Vector3<T>::operator*=(T s)
 {
-    v[0] *= s; v[1] *= s; v[2] *= s;
-    return *this;
+	v[0] *= s; v[1] *= s; v[2] *= s;
+	return *this;
 }
 
 template<class T> Vector3<T>& Vector3<T>::operator/=(T s)
@@ -522,47 +535,47 @@ template<class T> Vector3<T>& Vector3<T>::operator/=(T s)
 
 template<class T> Vector3<T> Vector3<T>::operator-() const
 {
-    return Vector3<T>(-v[0], -v[1], -v[2]);
+	return Vector3<T>(-v[0], -v[1], -v[2]);
 }
 
 template<class T> Vector3<T> Vector3<T>::operator+() const
 {
-    return *this;
+	return *this;
 }
 
 template<class T> Vector3<T> Vector3<T>::operator+(const Vector3<T>& b) const
 {
-    return Vector3<T>(v[0] + b.v[0], v[1] + b.v[1], v[2] + b.v[2]);
+	return Vector3<T>(v[0] + b.v[0], v[1] + b.v[1], v[2] + b.v[2]);
 }
 
 template<class T> Vector3<T> Vector3<T>::operator-(const Vector3<T>& b) const
 {
-    return Vector3<T>(v[0] - b.v[0], v[1] - b.v[1], v[2] - b.v[2]);
+	return Vector3<T>(v[0] - b.v[0], v[1] - b.v[1], v[2] - b.v[2]);
 }
 
 template<class T> Vector3<T> Vector3<T>::operator*(T s) const
 {
-    return Vector3<T>(s * v[0], s * v[1], s * v[2]);
+	return Vector3<T>(s * v[0], s * v[1], s * v[2]);
 }
 
 template<class T> Vector3<T> Vector3<T>::operator/(T s) const
 {
-    return Vector3<T>(v[0]/s, v[1]/s, v[2]/s);
+	return Vector3<T>(v[0]/s, v[1]/s, v[2]/s);
 }
 
 
 template<class T> T Vector3<T>::dot(const Vector3<T>& b) const
 {
-    return v[0] * b.v[0] + v[1] * b.v[1] + v[2] * b.v[2];
+	return v[0] * b.v[0] + v[1] * b.v[1] + v[2] * b.v[2];
 }
 
 
 // cross product
 template<class T> Vector3<T> Vector3<T>::operator^(const Vector3<T>& b) const
 {
-    return Vector3<T>(v[1] * b.v[2] - v[2] * b.v[1],
-                      v[2] * b.v[0] - v[0] * b.v[2],
-                      v[0] * b.v[1] - v[1] * b.v[0]);
+	return Vector3<T>(v[1] * b.v[2] - v[2] * b.v[1],
+					  v[2] * b.v[0] - v[0] * b.v[2],
+					  v[0] * b.v[1] - v[1] * b.v[0]);
 }
 
 // Angle in radian between two normalized vectors
@@ -573,30 +586,38 @@ template<class T> T Vector3<T>::angle(const Vector3<T>& b) const
 
 template<class T> T Vector3<T>::length() const
 {
-    return (T) sqrt(v[0] * v[0] + v[1] * v[1] + v[2] * v[2]);
+	return (T) sqrt(v[0] * v[0] + v[1] * v[1] + v[2] * v[2]);
 }
 
 template<class T> T Vector3<T>::lengthSquared() const
 {
-    return v[0] * v[0] + v[1] * v[1] + v[2] * v[2];
+	return v[0] * v[0] + v[1] * v[1] + v[2] * v[2];
 }
 
 template<class T> void Vector3<T>::normalize()
 {
-    T s = (T) (1. / std::sqrt(v[0] * v[0] + v[1] * v[1] + v[2] * v[2]));
-    v[0] *= s;
-    v[1] *= s;
-    v[2] *= s;
+	T s = (T) (1. / std::sqrt(v[0] * v[0] + v[1] * v[1] + v[2] * v[2]));
+	v[0] *= s;
+	v[1] *= s;
+	v[2] *= s;
 }
 
 template<class T> void Vector3<T>::transfo4d(const Mat4d& m)
 {
-	(*this)=m*(*this);
+	const T v0 = v[0];
+	const T v1 = v[1];
+	v[0]=m.r[0]*v0 + m.r[4]*v1 + m.r[8]*v[2] + m.r[12];
+	v[1]=m.r[1]*v0 + m.r[5]*v1 +  m.r[9]*v[2] + m.r[13];
+	v[2]=m.r[2]*v0 + m.r[6]*v1 + m.r[10]*v[2] + m.r[14];
 }
 
 template<class T> void Vector3<T>::transfo4d(const Mat4f& m)
 {
-	(*this)=m*(*this);
+	const T v0 = v[0];
+	const T v1 = v[1];
+	v[0]=m.r[0]*v0 + m.r[4]*v1 + m.r[8]*v[2] + m.r[12];
+	v[1]=m.r[1]*v0 + m.r[5]*v1 +  m.r[9]*v[2] + m.r[13];
+	v[2]=m.r[2]*v0 + m.r[6]*v1 + m.r[10]*v[2] + m.r[14];
 }
 
 // Return latitude in rad
@@ -611,25 +632,7 @@ template<class T> T Vector3<T>::longitude() const
 	return std::atan2(v[1],v[0]);
 }
 
-// template<class T> 
-// std::ostream& operator<<(std::ostream &o,const Vector3<T> &v) {
-//   return o << '[' << v[0] << ',' << v[1] << ',' << v[2] << ']';
-// }
 
-// template<class T> 
-// std::istream& operator>> (std::istream& is, Vector3<T> &v) {
-// 	while(is.get()!='[' && !is.eof()) {;}
-// 	assert(!is.eof() && "Vector must start with a '['");
-// 	is >> v[0];
-// 	is.ignore(256, ',');
-// 	is >> v[1];
-// 	is.ignore(256, ',');
-// 	is >> v[2];
-// 	while(is.get()!=']' && !is.eof()) {;}
-// 	assert(!is.eof() && "Vector must be terminated by a ']'");
-// 	return is;
-// }
-		
 ////////////////////////// Vector4 class methods ///////////////////////////////
 
 template<class T> Vector4<T>::Vector4() {}
@@ -657,19 +660,19 @@ template<class T> Vector4<T>::Vector4(T x, T y, T z, T a = 1)
 template<class T> Vector4<T>& Vector4<T>::operator=(const Vector4<T>& a)
 {
 	v[0]=a.v[0]; v[1]=a.v[1]; v[2]=a.v[2]; v[3]=a.v[3];
-    return *this;
+	return *this;
 }
 
 template<class T> Vector4<T>& Vector4<T>::operator=(const Vector3<T>& a)
 {
 	v[0]=a.v[0]; v[1]=a.v[1]; v[2]=a.v[2]; v[3]=1;
-    return *this;
+	return *this;
 }
 
 template<class T> Vector4<T>& Vector4<T>::operator=(const T* a)
 {
 	v[0]=a[0]; v[1]=a[1]; v[2]=a[2]; v[3]=a[3];
-    return *this;
+	return *this;
 }
 
 template<class T> void Vector4<T>::set(T x, T y, T z, T a)
@@ -709,73 +712,73 @@ template<class T> Vector4<T>::operator const T*() const
 
 template<class T> Vector4<T>& Vector4<T>::operator+=(const Vector4<T>& a)
 {
-    v[0] += a.v[0]; v[1] += a.v[1]; v[2] += a.v[2]; v[3] += a.v[3];
-    return *this;
+	v[0] += a.v[0]; v[1] += a.v[1]; v[2] += a.v[2]; v[3] += a.v[3];
+	return *this;
 }
 
 template<class T> Vector4<T>& Vector4<T>::operator-=(const Vector4<T>& a)
 {
-    v[0] -= a.v[0]; v[1] -= a.v[1]; v[2] -= a.v[2]; v[3] -= a/v[3];
-    return *this;
+	v[0] -= a.v[0]; v[1] -= a.v[1]; v[2] -= a.v[2]; v[3] -= a/v[3];
+	return *this;
 }
 
 template<class T> Vector4<T>& Vector4<T>::operator*=(T s)
 {
-    v[0] *= s; v[1] *= s; v[2] *= s; v[3] *= s;
-    return *this;
+	v[0] *= s; v[1] *= s; v[2] *= s; v[3] *= s;
+	return *this;
 }
 
 template<class T> Vector4<T> Vector4<T>::operator-() const
 {
-    return Vector4<T>(-v[0], -v[1], -v[2], -v[3]);
+	return Vector4<T>(-v[0], -v[1], -v[2], -v[3]);
 }
 
 template<class T> Vector4<T> Vector4<T>::operator+() const
 {
-    return *this;
+	return *this;
 }
 
 template<class T> Vector4<T> Vector4<T>::operator+(const Vector4<T>& b) const
 {
-    return Vector4<T>(v[0] + b.v[0], v[1] + b.v[1], v[2] + b.v[2], v[3] + b.v[3]);
+	return Vector4<T>(v[0] + b.v[0], v[1] + b.v[1], v[2] + b.v[2], v[3] + b.v[3]);
 }
 
 template<class T> Vector4<T> Vector4<T>::operator-(const Vector4<T>& b) const
 {
-    return Vector4<T>(v[0] - b.v[0], v[1] - b.v[1], v[2] - b.v[2], v[3] - b.v[3]);
+	return Vector4<T>(v[0] - b.v[0], v[1] - b.v[1], v[2] - b.v[2], v[3] - b.v[3]);
 }
 
 template<class T> Vector4<T> Vector4<T>::operator*(T s) const
 {
-    return Vector4<T>(s * v[0], s * v[1], s * v[2], s * v[3]);
+	return Vector4<T>(s * v[0], s * v[1], s * v[2], s * v[3]);
 }
 
 template<class T> Vector4<T> Vector4<T>::operator/(T s) const
 {
-    return Vector4<T>(v[0]/s, v[1]/s, v[2]/s, v[3]/s);
+	return Vector4<T>(v[0]/s, v[1]/s, v[2]/s, v[3]/s);
 }
 
 template<class T> T Vector4<T>::dot(const Vector4<T>& b) const
 {
-    return v[0] * b.v[0] + v[1] * b.v[1] + v[2] * b.v[2] + v[3] * b.v[3];
+	return v[0] * b.v[0] + v[1] * b.v[1] + v[2] * b.v[2] + v[3] * b.v[3];
 }
 
 template<class T> T Vector4<T>::length() const
 {
-    return (T) sqrt(v[0] * v[0] + v[1] * v[1] + v[2] * v[2] + v[3] * v[3]);
+	return (T) sqrt(v[0] * v[0] + v[1] * v[1] + v[2] * v[2] + v[3] * v[3]);
 }
 
 template<class T> T Vector4<T>::lengthSquared() const
 {
-    return v[0] * v[0] + v[1] * v[1] + v[2] * v[2] + v[3] * v[3];
+	return v[0] * v[0] + v[1] * v[1] + v[2] * v[2] + v[3] * v[3];
 }
 
 template<class T> void Vector4<T>::normalize()
 {
-    T s = (T) (1. / sqrt(v[0] * v[0] + v[1] * v[1] + v[2] * v[2] + v[3] * v[3]));
-    v[0] *= s;
-    v[1] *= s;
-    v[2] *= s;
+	T s = (T) (1. / sqrt(v[0] * v[0] + v[1] * v[1] + v[2] * v[2] + v[3] * v[3]));
+	v[0] *= s;
+	v[1] *= s;
+	v[2] *= s;
 	v[3] *= s;
 }
 
@@ -784,7 +787,7 @@ template<class T> void Vector4<T>::transfo4d(const Mat4d& m)
 	(*this)=m*(*this);
 }
 /*
-template<class T> 
+template<class T>
 std::ostream& operator<<(std::ostream &o,const Vector4<T> &v) {
   return o << '[' << v[0] << ',' << v[1] << ',' << v[2] << ',' << v[3] << ']';
 }*/
@@ -811,26 +814,26 @@ template<class T> Matrix4<T>& Matrix4<T>::operator=(const Matrix4<T>& m)
 }
 
 template<class T> Matrix4<T>::Matrix4(const Vector4<T>& v0,
-                                      const Vector4<T>& v1,
-                                      const Vector4<T>& v2,
-                                      const Vector4<T>& v3)
+									  const Vector4<T>& v1,
+									  const Vector4<T>& v2,
+									  const Vector4<T>& v3)
 {
-    r[0] = v0.v[0];
-    r[1] = v0.v[1];
-    r[2] = v0.v[2];
-    r[3] = v0.v[3];
-    r[4] = v1.v[0];
-    r[5] = v1.v[1];
-    r[6] = v1.v[2];
-    r[7] = v1.v[3];
-    r[8] = v2.v[0];
-    r[9] = v2.v[1];
-    r[10] = v2.v[2];
-    r[11] = v2.v[3];
-    r[12] = v3.v[0];
-    r[13] = v3.v[1];
-    r[14] = v3.v[2];
-    r[15] = v3.v[3];
+	r[0] = v0.v[0];
+	r[1] = v0.v[1];
+	r[2] = v0.v[2];
+	r[3] = v0.v[3];
+	r[4] = v1.v[0];
+	r[5] = v1.v[1];
+	r[6] = v1.v[2];
+	r[7] = v1.v[3];
+	r[8] = v2.v[0];
+	r[9] = v2.v[1];
+	r[10] = v2.v[2];
+	r[11] = v2.v[3];
+	r[12] = v3.v[0];
+	r[13] = v3.v[1];
+	r[14] = v3.v[2];
+	r[15] = v3.v[3];
 }
 
 
@@ -846,9 +849,9 @@ template<class T> void Matrix4<T>::set(T a, T b, T c, T d, T e, T f, T g, T h, T
 	r[8]=i; r[9]=j; r[10]=k; r[11]=l; r[12]=m; r[13]=n; r[14]=o; r[15]=p;
 }
 
-template<class T> T* Matrix4<T>::operator[](int n)
+template<class T> T& Matrix4<T>::operator[](int n)
 {
-    return &(r[n*4]);
+	return r[n];
 }
 
 template<class T> Matrix4<T>::operator T*()
@@ -863,7 +866,7 @@ template<class T> Matrix4<T>::operator const T*() const
 
 template<class T> Matrix4<T> Matrix4<T>::identity()
 {
-    return Matrix4<T>(	1, 0, 0, 0,
+	return Matrix4<T>(	1, 0, 0, 0,
 						0, 1, 0, 0,
 						0, 0, 1, 0,
 						0, 0, 0, 1  );
@@ -872,136 +875,87 @@ template<class T> Matrix4<T> Matrix4<T>::identity()
 
 template<class T> Matrix4<T> Matrix4<T>::translation(const Vector3<T>& a)
 {
-    return Matrix4<T>(	1, 0, 0, 0,
+	return Matrix4<T>(	1, 0, 0, 0,
 						0, 1, 0, 0,
 						0, 0, 1, 0,
 						a.v[0], a.v[1], a.v[2], 1);
 }
 
-template<class T> Matrix4<T> Matrix4<T>::rotation(const Vector3<T>& a,
-                                                  T angle)
+
+template<class T> Matrix4<T> Matrix4<T>::rotation(const Vector3<T>& axis, T angle)
 {
-	Vec3d axis(a);
-	axis.normalize();
-//    T c = (T) cos(angle);
-//    T s = (T) sin(angle);
-//    T t = 1 - c;
-//
-//    return Matrix4<T>(Vector4<T>(t * axis.v[0] * axis.v[0] + c,
-//                                 t * axis.v[0] * axis.v[1] - s * axis.v[2],
-//                                 t * axis.v[0] * axis.v[2] + s * axis.v[1],
-//                                 0),
-//                      Vector4<T>(t * axis.v[0] * axis.v[1] + s * axis.v[2],
-//                                 t * axis.v[1] * axis.v[1] + c,
-//                                 t * axis.v[1] * axis.v[2] - s * axis.v[0],
-//                                 0),
-//                      Vector4<T>(t * axis.v[0] * axis.v[2] - s * axis.v[1],
-//                                 t * axis.v[1] * axis.v[2] + s * axis.v[0],
-//                                 t * axis.v[2] * axis.v[2] + c,
-//                                 0),
-//                      Vector4<T>(0, 0, 0, 1));
-                      
-	T sin_a = std::sin( angle / 2 );
-	T cos_a = std::cos( angle / 2 );
-	T X    = axis[0] * sin_a;
-	T Y    = axis[1] * sin_a;
-	T Z    = axis[2] * sin_a;
-	T W    = cos_a;
-	
-	T xx      = X * X;
-	T xy      = X * Y;
-	T xz      = X * Z;
-	T xw      = X * W;
-	
-	T yy      = Y * Y;
-	T yz      = Y * Z;
-	T yw      = Y * W;
-	
-	T zz      = Z * Z;
-	T zw      = Z * W;
-
-	return Matrix4<T>(
-	1. - 2. * ( yy + zz ), 2. * ( xy + zw ),      2. * ( xz - yw ),      0.,
-	2. * ( xy - zw ),      1. - 2. * ( xx + zz ), 2. * ( yz + xw ),      0., 
-	2. * ( xz + yw ),      2. * ( yz - xw ),      1. - 2. * ( xx + yy ), 0., 
-	0.,                    0.,                    0.,                    1.);
-}
-
-
-
-/*
-template<class T> Matrix4<T> Matrix4<T>::rotation(const Vector3<T>& a)
-{
-    T c = (T) cos(angle);
-    T s = (T) sin(angle);
-	T d = 1-c;
-	 return Matrix4<T>(	a.v[0]*a.v[0]*d+c, a.v[1]*a.v[0]*d+a.v[2]*s, a.v[0]*a.v[2]*d-a.v[1]*s, 0,
-						a.v[0]*a.v[1]*d-a.v[2]*s, a.v[1]*a.v[1]*d+c, a.v[1]*a.v[2]*d+a.v[0]*s, 0,
-						a.v[0]*a.v[2]*d+a.v[1]*s, a.v[1]*a.v[2]*d-a.v[0]*s, a.v[2]*a.v[2]*d+c, 0,
+	Vec3d a(axis);
+	a.normalize();
+	const T c = (T) cos(angle);
+	const T s = (T) sin(angle);
+	const T d = 1-c;
+	return Matrix4<T>(	a[0]*a[0]*d+c     , a[1]*a[0]*d+a[2]*s, a[0]*a[2]*d-a[1]*s, 0,
+						a[0]*a[1]*d-a[2]*s, a[1]*a[1]*d+c     , a[1]*a[2]*d+a[0]*s, 0,
+						a[0]*a[2]*d+a[1]*s, a[1]*a[2]*d-a[0]*s, a[2]*a[2]*d+c     , 0,
 						0,0,0,1	);
 }
-*/
+
 template<class T> Matrix4<T> Matrix4<T>::xrotation(T angle)
 {
-    T c = (T) cos(angle);
-    T s = (T) sin(angle);
+	T c = (T) cos(angle);
+	T s = (T) sin(angle);
 
-    return Matrix4<T>(1, 0, 0, 0,
-                      0, c, s, 0,
-                      0,-s, c, 0,
-                      0, 0, 0, 1 );
+	return Matrix4<T>(1, 0, 0, 0,
+					  0, c, s, 0,
+					  0,-s, c, 0,
+					  0, 0, 0, 1 );
 }
 
 
 template<class T> Matrix4<T> Matrix4<T>::yrotation(T angle)
 {
-    T c = (T) cos(angle);
-    T s = (T) sin(angle);
+	T c = (T) cos(angle);
+	T s = (T) sin(angle);
 
-    return Matrix4<T>( c, 0,-s, 0,
-                       0, 1, 0, 0,
-                       s, 0, c, 0,
-                       0, 0, 0, 1 );
+	return Matrix4<T>( c, 0,-s, 0,
+					   0, 1, 0, 0,
+					   s, 0, c, 0,
+					   0, 0, 0, 1 );
 }
 
 
 template<class T> Matrix4<T> Matrix4<T>::zrotation(T angle)
 {
-    T c = (T) cos(angle);
-    T s = (T) sin(angle);
+	T c = (T) cos(angle);
+	T s = (T) sin(angle);
 
-    return Matrix4<T>(c, s, 0, 0,
-                     -s, c, 0, 0,
-                      0, 0, 1, 0,
-                      0, 0, 0, 1 );
+	return Matrix4<T>(c, s, 0, 0,
+					 -s, c, 0, 0,
+					  0, 0, 1, 0,
+					  0, 0, 0, 1 );
 }
 
 
 template<class T> Matrix4<T> Matrix4<T>::scaling(const Vector3<T>& s)
 {
-    return Matrix4<T>(s[0], 0  , 0  , 0,
-                      0   ,s[1], 0  , 0,
-                      0   , 0  ,s[2], 0,
-                      0   , 0  , 0  , 1);
+	return Matrix4<T>(s[0], 0  , 0  , 0,
+					  0   ,s[1], 0  , 0,
+					  0   , 0  ,s[2], 0,
+					  0   , 0  , 0  , 1);
 }
 
 
 template<class T> Matrix4<T> Matrix4<T>::scaling(T scale)
 {
-    return scaling(Vector3<T>(scale, scale, scale));
+	return scaling(Vector3<T>(scale, scale, scale));
 }
 
 // multiply column vector by a 4x4 matrix in homogeneous coordinate (use a[3]=1)
 template<class T> Vector3<T> Matrix4<T>::operator*(const Vector3<T>& a) const
 {
-    return Vector3<T>(	r[0]*a.v[0] + r[4]*a.v[1] +  r[8]*a.v[2] + r[12],
+	return Vector3<T>(	r[0]*a.v[0] + r[4]*a.v[1] +  r[8]*a.v[2] + r[12],
 						r[1]*a.v[0] + r[5]*a.v[1] +  r[9]*a.v[2] + r[13],
 						r[2]*a.v[0] + r[6]*a.v[1] + r[10]*a.v[2] + r[14] );
 }
 
 template<class T> Vector3<T> Matrix4<T>::multiplyWithoutTranslation(const Vector3<T>& a) const
 {
-    return Vector3<T>(	r[0]*a.v[0] + r[4]*a.v[1] +  r[8]*a.v[2],
+	return Vector3<T>(	r[0]*a.v[0] + r[4]*a.v[1] +  r[8]*a.v[2],
 						r[1]*a.v[0] + r[5]*a.v[1] +  r[9]*a.v[2],
 						r[2]*a.v[0] + r[6]*a.v[1] + r[10]*a.v[2] );
 }
@@ -1009,7 +963,7 @@ template<class T> Vector3<T> Matrix4<T>::multiplyWithoutTranslation(const Vector
 // multiply column vector by a 4x4 matrix in homogeneous coordinate (considere a[3]=1)
 template<class T> Vector4<T> Matrix4<T>::operator*(const Vector4<T>& a) const
 {
-    return Vector4<T>(	r[0]*a.v[0] + r[4]*a.v[1] +  r[8]*a.v[2] + r[12]*a.v[3],
+	return Vector4<T>(	r[0]*a.v[0] + r[4]*a.v[1] +  r[8]*a.v[2] + r[12]*a.v[3],
 						r[1]*a.v[0] + r[5]*a.v[1] +  r[9]*a.v[2] + r[13]*a.v[3],
 						r[2]*a.v[0] + r[6]*a.v[1] + r[10]*a.v[2] + r[14]*a.v[3] );
 }
@@ -1023,7 +977,7 @@ template<class T> void Matrix4<T>::transfo(Vector3<T>& a) const
 
 template<class T> Matrix4<T> Matrix4<T>::transpose() const
 {
-    return Matrix4<T>(	r[0], r[4], r[8],  r[12],
+	return Matrix4<T>(	r[0], r[4], r[8],  r[12],
 						r[1], r[5], r[9],  r[13],
 						r[2], r[6], r[10], r[14],
 						r[3], r[7], r[11], r[15]);
@@ -1032,7 +986,7 @@ template<class T> Matrix4<T> Matrix4<T>::transpose() const
 template<class T> Matrix4<T> Matrix4<T>::operator*(const Matrix4<T>& a) const
 {
 #define MATMUL(R, C) (r[R] * a.r[C] + r[R+4] * a.r[C+1] + r[R+8] * a.r[C+2] + r[R+12] * a.r[C+3])
-    return Matrix4<T>(	MATMUL(0,0), MATMUL(1,0), MATMUL(2,0), MATMUL(3,0),
+	return Matrix4<T>(	MATMUL(0,0), MATMUL(1,0), MATMUL(2,0), MATMUL(3,0),
 						MATMUL(0,4), MATMUL(1,4), MATMUL(2,4), MATMUL(3,4),
 						MATMUL(0,8), MATMUL(1,8), MATMUL(2,8), MATMUL(3,8),
 						MATMUL(0,12), MATMUL(1,12), MATMUL(2,12), MATMUL(3,12) );
@@ -1042,7 +996,7 @@ template<class T> Matrix4<T> Matrix4<T>::operator*(const Matrix4<T>& a) const
 
 template<class T> Matrix4<T> Matrix4<T>::operator+(const Matrix4<T>& a) const
 {
-    return Matrix4<T>(	r[0]+a.r[0], r[1]+a.r[1], r[2]+a.r[2], r[3]+a.r[3],
+	return Matrix4<T>(	r[0]+a.r[0], r[1]+a.r[1], r[2]+a.r[2], r[3]+a.r[3],
 						r[4]+a.r[4], r[5]+a.r[5], r[6]+a.r[6], r[7]+a.r[7],
 						r[8]+a.r[8], r[9]+a.r[9], r[10]+a.r[10], r[11]+a.r[11],
 						r[12]+a.r[12], r[13]+a.r[13], r[14]+a.r[14], r[15]+a.r[15] );
@@ -1050,7 +1004,7 @@ template<class T> Matrix4<T> Matrix4<T>::operator+(const Matrix4<T>& a) const
 
 template<class T> Matrix4<T> Matrix4<T>::operator-(const Matrix4<T>& a) const
 {
-    return Matrix4<T>(	r[0]-a.r[0], r[1]-a.r[1], r[2]-a.r[2], r[3]-a.r[3],
+	return Matrix4<T>(	r[0]-a.r[0], r[1]-a.r[1], r[2]-a.r[2], r[3]-a.r[3],
 						r[4]-a.r[4], r[5]-a.r[5], r[6]-a.r[6], r[7]-a.r[7],
 						r[8]-a.r[8], r[9]-a.r[9], r[10]-a.r[10], r[11]-a.r[11],
 						r[12]-a.r[12], r[13]-a.r[13], r[14]-a.r[14], r[15]-a.r[15] );
@@ -1078,27 +1032,27 @@ template<class T> Matrix4<T> Matrix4<T>::inverse() const
    r0 = wtmp[0], r1 = wtmp[1], r2 = wtmp[2], r3 = wtmp[3];
 
    r0[0] = MAT(m, 0, 0), r0[1] = MAT(m, 0, 1),
-      r0[2] = MAT(m, 0, 2), r0[3] = MAT(m, 0, 3),
-      r0[4] = 1.0, r0[5] = r0[6] = r0[7] = 0.0,
-      r1[0] = MAT(m, 1, 0), r1[1] = MAT(m, 1, 1),
-      r1[2] = MAT(m, 1, 2), r1[3] = MAT(m, 1, 3),
-      r1[5] = 1.0, r1[4] = r1[6] = r1[7] = 0.0,
-      r2[0] = MAT(m, 2, 0), r2[1] = MAT(m, 2, 1),
-      r2[2] = MAT(m, 2, 2), r2[3] = MAT(m, 2, 3),
-      r2[6] = 1.0, r2[4] = r2[5] = r2[7] = 0.0,
-      r3[0] = MAT(m, 3, 0), r3[1] = MAT(m, 3, 1),
-      r3[2] = MAT(m, 3, 2), r3[3] = MAT(m, 3, 3),
-      r3[7] = 1.0, r3[4] = r3[5] = r3[6] = 0.0;
+	  r0[2] = MAT(m, 0, 2), r0[3] = MAT(m, 0, 3),
+	  r0[4] = 1.0, r0[5] = r0[6] = r0[7] = 0.0,
+	  r1[0] = MAT(m, 1, 0), r1[1] = MAT(m, 1, 1),
+	  r1[2] = MAT(m, 1, 2), r1[3] = MAT(m, 1, 3),
+	  r1[5] = 1.0, r1[4] = r1[6] = r1[7] = 0.0,
+	  r2[0] = MAT(m, 2, 0), r2[1] = MAT(m, 2, 1),
+	  r2[2] = MAT(m, 2, 2), r2[3] = MAT(m, 2, 3),
+	  r2[6] = 1.0, r2[4] = r2[5] = r2[7] = 0.0,
+	  r3[0] = MAT(m, 3, 0), r3[1] = MAT(m, 3, 1),
+	  r3[2] = MAT(m, 3, 2), r3[3] = MAT(m, 3, 3),
+	  r3[7] = 1.0, r3[4] = r3[5] = r3[6] = 0.0;
 
    /* choose pivot - or die */
    if (fabs(r3[0]) > fabs(r2[0]))
-      SWAP_ROWS(r3, r2);
+	  SWAP_ROWS(r3, r2);
    if (fabs(r2[0]) > fabs(r1[0]))
-      SWAP_ROWS(r2, r1);
+	  SWAP_ROWS(r2, r1);
    if (fabs(r1[0]) > fabs(r0[0]))
-      SWAP_ROWS(r1, r0);
+	  SWAP_ROWS(r1, r0);
    if (0.0 == r0[0])
-      return Matrix4<T>();
+	  return Matrix4<T>();
 
    /* eliminate first variable     */
    m1 = r1[0] / r0[0];
@@ -1118,36 +1072,36 @@ template<class T> Matrix4<T> Matrix4<T>::inverse() const
    r3[3] -= m3 * s;
    s = r0[4];
    if (s != 0.0) {
-      r1[4] -= m1 * s;
-      r2[4] -= m2 * s;
-      r3[4] -= m3 * s;
+	  r1[4] -= m1 * s;
+	  r2[4] -= m2 * s;
+	  r3[4] -= m3 * s;
    }
    s = r0[5];
    if (s != 0.0) {
-      r1[5] -= m1 * s;
-      r2[5] -= m2 * s;
-      r3[5] -= m3 * s;
+	  r1[5] -= m1 * s;
+	  r2[5] -= m2 * s;
+	  r3[5] -= m3 * s;
    }
    s = r0[6];
    if (s != 0.0) {
-      r1[6] -= m1 * s;
-      r2[6] -= m2 * s;
-      r3[6] -= m3 * s;
+	  r1[6] -= m1 * s;
+	  r2[6] -= m2 * s;
+	  r3[6] -= m3 * s;
    }
    s = r0[7];
    if (s != 0.0) {
-      r1[7] -= m1 * s;
-      r2[7] -= m2 * s;
-      r3[7] -= m3 * s;
+	  r1[7] -= m1 * s;
+	  r2[7] -= m2 * s;
+	  r3[7] -= m3 * s;
    }
 
    /* choose pivot - or die */
    if (fabs(r3[1]) > fabs(r2[1]))
-      SWAP_ROWS(r3, r2);
+	  SWAP_ROWS(r3, r2);
    if (fabs(r2[1]) > fabs(r1[1]))
-      SWAP_ROWS(r2, r1);
+	  SWAP_ROWS(r2, r1);
    if (0.0 == r1[1])
-      return Matrix4<T>();
+	  return Matrix4<T>();
 
    /* eliminate second variable */
    m2 = r2[1] / r1[1];
@@ -1158,39 +1112,39 @@ template<class T> Matrix4<T> Matrix4<T>::inverse() const
    r3[3] -= m3 * r1[3];
    s = r1[4];
    if (0.0 != s) {
-      r2[4] -= m2 * s;
-      r3[4] -= m3 * s;
+	  r2[4] -= m2 * s;
+	  r3[4] -= m3 * s;
    }
    s = r1[5];
    if (0.0 != s) {
-      r2[5] -= m2 * s;
-      r3[5] -= m3 * s;
+	  r2[5] -= m2 * s;
+	  r3[5] -= m3 * s;
    }
    s = r1[6];
    if (0.0 != s) {
-      r2[6] -= m2 * s;
-      r3[6] -= m3 * s;
+	  r2[6] -= m2 * s;
+	  r3[6] -= m3 * s;
    }
    s = r1[7];
    if (0.0 != s) {
-      r2[7] -= m2 * s;
-      r3[7] -= m3 * s;
+	  r2[7] -= m2 * s;
+	  r3[7] -= m3 * s;
    }
 
    /* choose pivot - or die */
    if (fabs(r3[2]) > fabs(r2[2]))
-      SWAP_ROWS(r3, r2);
+	  SWAP_ROWS(r3, r2);
    if (0.0 == r2[2])
-      return Matrix4<T>();
+	  return Matrix4<T>();
 
    /* eliminate third variable */
    m3 = r3[2] / r2[2];
    r3[3] -= m3 * r2[3], r3[4] -= m3 * r2[4],
-      r3[5] -= m3 * r2[5], r3[6] -= m3 * r2[6], r3[7] -= m3 * r2[7];
+	  r3[5] -= m3 * r2[5], r3[6] -= m3 * r2[6], r3[7] -= m3 * r2[7];
 
    /* last check */
    if (0.0 == r3[3])
-      return Matrix4<T>();
+	  return Matrix4<T>();
 
    s = 1.0 / r3[3];		/* now back substitute row 3 */
    r3[4] *= s;
@@ -1201,26 +1155,26 @@ template<class T> Matrix4<T> Matrix4<T>::inverse() const
    m2 = r2[3];			/* now back substitute row 2 */
    s = 1.0 / r2[2];
    r2[4] = s * (r2[4] - r3[4] * m2), r2[5] = s * (r2[5] - r3[5] * m2),
-      r2[6] = s * (r2[6] - r3[6] * m2), r2[7] = s * (r2[7] - r3[7] * m2);
+	  r2[6] = s * (r2[6] - r3[6] * m2), r2[7] = s * (r2[7] - r3[7] * m2);
    m1 = r1[3];
    r1[4] -= r3[4] * m1, r1[5] -= r3[5] * m1,
-      r1[6] -= r3[6] * m1, r1[7] -= r3[7] * m1;
+	  r1[6] -= r3[6] * m1, r1[7] -= r3[7] * m1;
    m0 = r0[3];
    r0[4] -= r3[4] * m0, r0[5] -= r3[5] * m0,
-      r0[6] -= r3[6] * m0, r0[7] -= r3[7] * m0;
+	  r0[6] -= r3[6] * m0, r0[7] -= r3[7] * m0;
 
    m1 = r1[2];			/* now back substitute row 1 */
    s = 1.0 / r1[1];
    r1[4] = s * (r1[4] - r2[4] * m1), r1[5] = s * (r1[5] - r2[5] * m1),
-      r1[6] = s * (r1[6] - r2[6] * m1), r1[7] = s * (r1[7] - r2[7] * m1);
+	  r1[6] = s * (r1[6] - r2[6] * m1), r1[7] = s * (r1[7] - r2[7] * m1);
    m0 = r0[2];
    r0[4] -= r2[4] * m0, r0[5] -= r2[5] * m0,
-      r0[6] -= r2[6] * m0, r0[7] -= r2[7] * m0;
+	  r0[6] -= r2[6] * m0, r0[7] -= r2[7] * m0;
 
    m0 = r0[1];			/* now back substitute row 0 */
    s = 1.0 / r0[0];
    r0[4] = s * (r0[4] - r1[4] * m0), r0[5] = s * (r0[5] - r1[5] * m0),
-      r0[6] = s * (r0[6] - r1[6] * m0), r0[7] = s * (r0[7] - r1[7] * m0);
+	  r0[6] = s * (r0[6] - r1[6] * m0), r0[7] = s * (r0[7] - r1[7] * m0);
 
    MAT(out, 0, 0) = r0[4];
    MAT(out, 0, 1) = r0[5], MAT(out, 0, 2) = r0[6];
@@ -1241,14 +1195,14 @@ template<class T> Matrix4<T> Matrix4<T>::inverse() const
 
 template<class T> void Matrix4<T>::print(void) const
 {
-    printf("[%5.2lf %5.2lf %5.2lf %17.12le]\n"
-           "[%5.2lf %5.2lf %5.2lf %17.12le]\n"
-           "[%5.2lf %5.2lf %5.2lf %17.12le]\n"
-           "[%5.2lf %5.2lf %5.2lf %17.12le]\n\n",
-    r[0],r[4],r[8],r[12],
-    r[1],r[5],r[9],r[13],
-    r[2],r[6],r[10],r[14],
-    r[3],r[7],r[11],r[15]);
+	printf("[%5.2lf %5.2lf %5.2lf %17.12le]\n"
+		   "[%5.2lf %5.2lf %5.2lf %17.12le]\n"
+		   "[%5.2lf %5.2lf %5.2lf %17.12le]\n"
+		   "[%5.2lf %5.2lf %5.2lf %17.12le]\n\n",
+	r[0],r[4],r[8],r[12],
+	r[1],r[5],r[9],r[13],
+	r[2],r[6],r[10],r[14],
+	r[3],r[7],r[11],r[15]);
 }
 
 
