@@ -1,17 +1,17 @@
 /*
  * Stellarium
  * Copyright (C) 2007 Matthew Gates
- * 
+ *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
  * as published by the Free Software Foundation; either version 2
  * of the License, or (at your option) any later version.
- * 
+ *
  * This program is distributed in the hope that it will be useful,
  * but WITHOUT ANY WARRANTY; without even the implied warranty of
  * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  * GNU General Public License for more details.
- * 
+ *
  * You should have received a copy of the GNU General Public License
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
@@ -38,8 +38,8 @@
 #include "StelApp.hpp"
 #include "StelFileMgr.hpp"
 #include "StelLocaleMgr.hpp"
-#include "StelMainGraphicsView.hpp"
 #include "StelStyle.hpp"
+#include "StelLogger.hpp"
 
 HelpDialog::HelpDialog()
 {
@@ -66,6 +66,11 @@ HelpDialog::HelpDialog()
 #ifdef MACOSX
 	setKey(group, "", "!06-ctrl+left-click", N_("Clear selection"));
 #endif
+
+	group = N_("When a Script is Running");
+	setKey(group, "", "J", N_("Slow down the script execution rate"));
+	setKey(group, "", "L", N_("Speed up the script execution rate"));
+	setKey(group, "", "K", N_("Set the normal script execution rate"));
 }
 
 HelpDialog::~HelpDialog()
@@ -94,25 +99,28 @@ void HelpDialog::styleChanged()
 void HelpDialog::createDialogContent()
 {
 	ui->setupUi(dialog);
-	ui->helpTabWidget->setCurrentIndex(0);
+	ui->stackedWidget->setCurrentIndex(0);
+	ui->stackListWidget->setCurrentRow(0);
 	connect(ui->closeStelWindow, SIGNAL(clicked()), this, SLOT(close()));
-	
+
 	updateText();
-	
-	ui->logPathLabel->setText(QString("%1/log.txt:").arg(StelApp::getInstance().getFileMgr().getUserDir()));
-	connect(ui->helpTabWidget, SIGNAL(currentChanged(int)), this, SLOT(updateLog(int)));
+
+	ui->logPathLabel->setText(QString("%1/log.txt:").arg(StelFileMgr::getUserDir()));
+	connect(ui->stackedWidget, SIGNAL(currentChanged(int)), this, SLOT(updateLog(int)));
 	connect(ui->refreshButton, SIGNAL(clicked()), this, SLOT(refreshLog()));
+
+	connect(ui->stackListWidget, SIGNAL(currentItemChanged(QListWidgetItem *, QListWidgetItem *)), this, SLOT(changePage(QListWidgetItem *, QListWidgetItem*)));
 }
 
 void HelpDialog::updateLog(int)
 {
-	if(ui->helpTabWidget->currentWidget() == ui->logTab)
+	if(ui->stackedWidget->currentWidget() == ui->page_3)
 		refreshLog();
 }
 
 void HelpDialog::refreshLog()
 {
-	ui->logBrowser->setPlainText(StelApp::getInstance().getLog());
+	ui->logBrowser->setPlainText(StelLogger::getLog());
 }
 
 void HelpDialog::setKey(QString group, QString oldKey, QString newKey, QString description)
@@ -130,7 +138,7 @@ void HelpDialog::setKey(QString group, QString oldKey, QString newKey, QString d
 		// 	this->updateText();
 		return;
 	}
-	
+
 	// Else delete the old entry if we can find it, and then insert the
 	// new entry.  Here's where the multimap makes us wince...
 	QMultiMap<QString, QPair<QString, QString> >::iterator i = keyData.begin();
@@ -175,17 +183,12 @@ QString HelpDialog::getFooterText(void)
 
 	footer += "<p>";
 	// TRANSLATORS: The text between braces is the text of an HTML link.
-	footer += Qt::escape(q_("{Support ticket system} - if you need help using Stellarium, post a support request here and we'll try to help.")).replace(a_rx, "<a href=\"http://sourceforge.net/tracker/?group_id=48857&atid=454374\">\\1</a>");
+	footer += Qt::escape(q_("{Support ticket system} - if you need help using Stellarium, post a support request here and we'll try to help.")).replace(a_rx, "<a href=\"http://answers.launchpad.net/stellarium/+addquestion\">\\1</a>");
 	footer += "</p>\n";
 
 	footer += "<p>";
 	// TRANSLATORS: The text between braces is the text of an HTML link.
-	footer += Qt::escape(q_("{Bug reporting system} - if something doesn't work properly and is not listed in the FAQ list, you can open bug reports here.")).replace(a_rx, "<a href=\"http://sourceforge.net/tracker/?group_id=48857&atid=454373\">\\1</a>");
-	footer += "</p>\n";
-
-	footer += "<p>";
-	// TRANSLATORS: The text between braces is the text of an HTML link.
-	footer += Qt::escape(q_("{Feature request system} - if you have an idea for a new feature, send it to us. We can't promise to implement every idea, but we appreciate the feedback and review the list when we are planning future features.")).replace(a_rx, "<a href=\"http://sourceforge.net/tracker/?group_id=48857&atid=454376\">\\1</a>");
+	footer += Qt::escape(q_("{Bug reporting and feature request system} - if something doesn't work properly or is missing and is not listed in the tracker, you can open bug reports here.")).replace(a_rx, "<a href=\"http://bugs.launchpad.net/stellarium/\">\\1</a>");
 	footer += "</p>\n";
 
 	footer += "<p>";
@@ -202,24 +205,24 @@ void HelpDialog::updateText(void)
 {
 	// Here's how we will build the help text for the keys:
 	// 1.  Get a unique list of groups by asking for the keys and then converting the
-	//     resulting QList into a QSet.  
+	//     resulting QList into a QSet.
 	// 2   Converet back to a QList, sort and move the empty string to the end of the
 	//     list if it is present (this is the miscellaneous group).
 	// 3   Iterate over the QSet of groups names doing:
 	// 3.1  add the group title
-	// 3.2  Use QMultiMap::values(key) to get a list of QPair<QString, QString> 
+	// 3.2  Use QMultiMap::values(key) to get a list of QPair<QString, QString>
 	//      which describe the key binding (QPair::first) and the help text for
 	//      that key binding (QPair::second).
 	// 3.3  Sort this list by the first value in the pait, courtesy of qSort and
 	//      HelpDialog::helpItemSort
 	// 3.4  Iterate over the sorted list adding key and description for each item
-	
+
 	QString newHtml(getHeaderText());
 	newHtml += "<table cellpadding=\"10%\">\n";
 
 	QList<QString> groups = keyData.keys().toSet().toList(); // 1 + 2
 	qSort(groups.begin(), groups.end(), HelpDialog::helpGroupSort);
-	
+
 	// 3
 	QString lastGroup;  // to group "" and "Miscellaneous into one
 	foreach (QString group, groups)
@@ -266,9 +269,9 @@ void HelpDialog::updateText(void)
 	ui->helpBrowser->scrollToAnchor("top");
 
 	// populate About tab
-	newHtml = "<h1>" + StelApp::getInstance().getApplicationName() + "</h1>";
+	newHtml = "<h1>" + StelUtils::getApplicationName() + "</h1>";
 	// Note: this legal notice is not suitable for traslation
-	newHtml += "<h3>Copyright &copy; 2000-2009 Stellarium Developers</h3>";
+	newHtml += "<h3>Copyright &copy; 2000-2010 Stellarium Developers</h3>";
 	newHtml += "<p>This program is free software; you can redistribute it and/or ";
 	newHtml += "modify it under the terms of the GNU General Public License ";
 	newHtml += "as published by the Free Software Foundation; either version 2 ";
@@ -301,7 +304,7 @@ void HelpDialog::updateText(void)
 
 bool HelpDialog::helpItemSort(const QPair<QString, QString>& p1, const QPair<QString, QString>& p2)
 {
-	// To be 100% proper, we should sort F1 F2 F11 F12 in that order, although 
+	// To be 100% proper, we should sort F1 F2 F11 F12 in that order, although
 	// right now we will get F1 F11 F12 F2.  However, at time of writing, no group
 	// of keys has F1-F9, and one from F10-F12 in it, so it doesn't really matter.
 	// -MNG 2008-06-01
@@ -327,3 +330,10 @@ bool HelpDialog::helpGroupSort(const QString& s1, const QString& s2)
 
 	return s1c < s2c;
 }
+
+void HelpDialog::changePage(QListWidgetItem *current, QListWidgetItem *previous)
+{
+	if (!current)
+		current = previous;
+	ui->stackedWidget->setCurrentIndex(ui->stackListWidget->row(current));
+}
