@@ -1,17 +1,17 @@
 /*
  * Stellarium
  * Copyright (C) 2002 Fabien Chereau
- * 
+ *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
  * as published by the Free Software Foundation; either version 2
  * of the License, or (at your option) any later version.
- * 
+ *
  * This program is distributed in the hope that it will be useful,
  * but WITHOUT ANY WARRANTY; without even the implied warranty of
  * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  * GNU General Public License for more details.
- * 
+ *
  * You should have received a copy of the GNU General Public License
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
@@ -38,13 +38,11 @@
 #include "StelProjector.hpp"
 #include "StelLoadingBar.hpp"
 #include "StelObjectMgr.hpp"
-#include "StelFontMgr.hpp"
 #include "StelLocaleMgr.hpp"
 #include "StelSkyCultureMgr.hpp"
 #include "StelModuleMgr.hpp"
 #include "StelFileMgr.hpp"
 #include "StelCore.hpp"
-#include "StelFont.hpp"
 #include "StelStyle.hpp"
 #include "StelPainter.hpp"
 
@@ -52,8 +50,6 @@ using namespace std;
 
 // constructor which loads all data from appropriate files
 ConstellationMgr::ConstellationMgr(StarMgr *_hip_stars) :
-	fontSize(15),
-	asterFont(NULL),
 	hipStarMgr(_hip_stars),
 	flagNames(0),
 	flagLines(0),
@@ -63,6 +59,7 @@ ConstellationMgr::ConstellationMgr(StarMgr *_hip_stars) :
 	setObjectName("ConstellationMgr");
 	Q_ASSERT(hipStarMgr);
 	isolateSelected = false;
+	asterFont.setPixelSize(15);
 }
 
 ConstellationMgr::~ConstellationMgr()
@@ -88,7 +85,7 @@ void ConstellationMgr::init()
 	Q_ASSERT(conf);
 
 	lastLoadedSkyCulture = "dummy";
-	fontSize = conf->value("viewing/constellation_font_size",16.).toDouble();
+	asterFont.setPixelSize(conf->value("viewing/constellation_font_size",16).toInt());
 	setFlagLines(conf->value("viewing/flag_constellation_drawing").toBool());
 	setFlagLabels(conf->value("viewing/flag_constellation_name").toBool());
 	setFlagBoundaries(conf->value("viewing/flag_constellation_boundaries",false).toBool());
@@ -96,7 +93,9 @@ void ConstellationMgr::init()
 	setArtFadeDuration(conf->value("viewing/constellation_art_fade_duration",2.).toDouble());
 	setFlagArt(conf->value("viewing/flag_constellation_art").toBool());
 	setFlagIsolateSelected(conf->value("viewing/flag_constellation_isolate_selected",
-				conf->value("viewing/flag_constellation_pick", false).toBool() ).toBool());
+						   conf->value("viewing/flag_constellation_pick", false).toBool() ).toBool());
+
+	GETSTELMODULE(StelObjectMgr)->registerStelObjectMgr(this);
 }
 
 /*************************************************************************
@@ -111,8 +110,6 @@ double ConstellationMgr::getCallOrder(StelModuleActionName actionName) const
 
 void ConstellationMgr::updateSkyCulture(const QString& skyCultureDir)
 {
-	StelFileMgr& fileMan = StelApp::getInstance().getFileMgr();
-	
 	// Check if the sky culture changed since last load, if not don't load anything
 	if (lastLoadedSkyCulture == skyCultureDir)
 		return;
@@ -122,38 +119,38 @@ void ConstellationMgr::updateSkyCulture(const QString& skyCultureDir)
 	QString conArtFile;
 	try
 	{
-		conArtFile = fileMan.findFile("skycultures/"+skyCultureDir+"/constellationsart.fab");
+		conArtFile = StelFileMgr::findFile("skycultures/"+skyCultureDir+"/constellationsart.fab");
 	}
 	catch (std::runtime_error& e)
 	{
-		qWarning() << "WARNING: no constellationsart.fab file found for sky culture " << skyCultureDir;
+		qDebug() << "No constellationsart.fab file found for sky culture " << skyCultureDir;
 	}
 
 	try
 	{
-		loadLinesAndArt(fileMan.findFile("skycultures/"+skyCultureDir+"/constellationship.fab"), conArtFile, skyCultureDir);
-			
+		loadLinesAndArt(StelFileMgr::findFile("skycultures/"+skyCultureDir+"/constellationship.fab"), conArtFile, skyCultureDir);
+
 		// load constellation names
-		loadNames(fileMan.findFile("skycultures/" + skyCultureDir + "/constellation_names.eng.fab"));
+		loadNames(StelFileMgr::findFile("skycultures/" + skyCultureDir + "/constellation_names.eng.fab"));
 
 		// Translate constellation names for the new sky culture
 		updateI18n();
-		
+
 		// as constellations have changed, clear out any selection and retest for match!
-		selectedObjectChangeCallBack(StelModule::ReplaceSelection);		
+		selectedObjectChangeCallBack(StelModule::ReplaceSelection);
 	}
 	catch (std::runtime_error& e)
 	{
-		qWarning() << "ERROR: while loading new constellation data for sky culture " 
-			<< skyCultureDir << ", reason: " << e.what() << endl;		
+		qWarning() << "ERROR: while loading new constellation data for sky culture "
+			<< skyCultureDir << ", reason: " << e.what() << endl;
 	}
-		
+
 	// TODO: do we need to have an else { clearBoundaries(); } ?
-	if (skyCultureDir=="western") 
+	if (skyCultureDir=="western")
 	{
 		try
 		{
-			loadBoundaries(fileMan.findFile("data/constellations_boundaries.dat"));
+			loadBoundaries(StelFileMgr::findFile("data/constellations_boundaries.dat"));
 		}
 		catch (std::runtime_error& e)
 		{
@@ -167,7 +164,7 @@ void ConstellationMgr::updateSkyCulture(const QString& skyCultureDir)
 void ConstellationMgr::setStelStyle(const StelStyle& style)
 {
 	QSettings* conf = StelApp::getInstance().getSettings();
-	
+
 	// Load colors from config file
 	QString defaultColor = conf->value(style.confSectionName+"/default_color").toString();
 	setLinesColor(StelUtils::strToVec3f(conf->value(style.confSectionName+"/const_lines_color",    defaultColor).toString()));
@@ -187,19 +184,16 @@ void ConstellationMgr::selectedObjectChangeCallBack(StelModuleSelectAction actio
 		// setSelected(NULL);
 		return;
 	}
-	
+
 	const QList<StelObjectP> newSelectedConst = omgr->getSelectedObject("Constellation");
 	if (!newSelectedConst.empty())
 	{
-//		const boost::intrusive_ptr<Constellation> c = boost::dynamic_pointer_cast<Constellation>(newSelectedConst[0]);
-//		omgr->setSelectedObject(c->getBrightestStarInConstellation());
-
 		// If removing this selection
 		if(action == StelModule::RemoveFromSelection) {
-			unsetSelectedConst((Constellation *)newSelectedConst[0].get());
+			unsetSelectedConst((Constellation *)newSelectedConst[0].data());
 		} else {
 			// Add constellation to selected list (do not select a star, just the constellation)
-			setSelectedConst((Constellation *)newSelectedConst[0].get());
+			setSelectedConst((Constellation *)newSelectedConst[0].data());
 		}
 	}
 	else
@@ -209,7 +203,7 @@ void ConstellationMgr::selectedObjectChangeCallBack(StelModuleSelectAction actio
 		{
 //			if (!added)
 //				setSelected(NULL);
-			setSelected(newSelectedStar[0].get());
+			setSelected(newSelectedStar[0].data());
 		}
 		else
 		{
@@ -229,34 +223,34 @@ Vec3f ConstellationMgr::getLinesColor() const
 	return Constellation::lineColor;
 }
 
-void ConstellationMgr::setBoundariesColor(const Vec3f& c) 
+void ConstellationMgr::setBoundariesColor(const Vec3f& c)
 {
 	Constellation::boundaryColor = c;
 }
 
-Vec3f ConstellationMgr::getBoundariesColor() const 
+Vec3f ConstellationMgr::getBoundariesColor() const
 {
 	return Constellation::boundaryColor;
 }
 
-void ConstellationMgr::setLabelsColor(const Vec3f& c) 
+void ConstellationMgr::setLabelsColor(const Vec3f& c)
 {
 	Constellation::labelColor = c;
 }
 
-Vec3f ConstellationMgr::getLabelsColor() const 
+Vec3f ConstellationMgr::getLabelsColor() const
 {
 	return Constellation::labelColor;
 }
 
 void ConstellationMgr::setFontSize(double newFontSize)
 {
-	asterFont = &StelApp::getInstance().getFontManager().getStandardFont(StelApp::getInstance().getLocaleMgr().getSkyLanguage(), fontSize);
+	asterFont.setPixelSize(newFontSize);
 }
 
 double ConstellationMgr::getFontSize() const
 {
-	return asterFont->getSize();
+	return asterFont.pixelSize();
 }
 
 // Load line and art data from files
@@ -272,7 +266,7 @@ void ConstellationMgr::loadLinesAndArt(const QString &fileName, const QString &a
 	int totalRecords=0;
 	QString record;
 	QRegExp commentRx("^(\\s*#.*|\\s*)$");
-	while (!in.atEnd()) 
+	while (!in.atEnd())
 	{
 		record = QString::fromUtf8(in.readLine());
 		if (!commentRx.exactMatch(record))
@@ -292,7 +286,7 @@ void ConstellationMgr::loadLinesAndArt(const QString &fileName, const QString &a
 	// read the file, adding a record per non-comment line
 	int currentLineNumber = 0;	// line in file
 	int readOk = 0;			// count of records processed OK
-	while (!in.atEnd()) 
+	while (!in.atEnd())
 	{
 		record = QString::fromUtf8(in.readLine());
 		currentLineNumber++;
@@ -302,6 +296,7 @@ void ConstellationMgr::loadLinesAndArt(const QString &fileName, const QString &a
 		cons = new Constellation;
 		if(cons->read(record, hipStarMgr))
 		{
+			cons->artFader.setMaxValue(artMaxIntensity);
 			asterisms.push_back(cons);
 			++readOk;
 		}
@@ -317,19 +312,21 @@ void ConstellationMgr::loadLinesAndArt(const QString &fileName, const QString &a
 	// Set current states
 	setFlagArt(flagArt);
 	setFlagLines(flagLines);
-	setFlagLabels(flagNames);	
-	setFlagBoundaries(flagBoundaries);	
+	setFlagLabels(flagNames);
+	setFlagBoundaries(flagBoundaries);
 
+	// It's possible to have no art - just constellations
+	if (artfileName.isNull() || artfileName.isEmpty())
+		return;
 	QFile fic(artfileName);
 	if (!fic.open(QIODevice::ReadOnly | QIODevice::Text))
 	{
 		qWarning() << "Can't open constellation art file" << fileName  << "for culture" << cultureName;
-		return; // possible to have no art - just constellations
-		Q_ASSERT(0);
+		return;
 	}
 
 	totalRecords=0;
-	while (!fic.atEnd()) 
+	while (!fic.atEnd())
 	{
 		record = QString::fromUtf8(fic.readLine());
 		if (!commentRx.exactMatch(record))
@@ -353,9 +350,7 @@ void ConstellationMgr::loadLinesAndArt(const QString &fileName, const QString &a
 	currentLineNumber = 0;	// line in file
 	readOk = 0;		// count of records processed OK
 
-	StelApp::getInstance().getTextureManager().setDefaultParams();
-	StelLoadingBar& lb = *StelApp::getInstance().getStelLoadingBar();
-	while (!fic.atEnd()) 
+	while (!fic.atEnd())
 	{
 		++currentLineNumber;
 		record = QString::fromUtf8(fic.readLine());
@@ -371,45 +366,44 @@ void ConstellationMgr::loadLinesAndArt(const QString &fileName, const QString &a
 			qWarning() << "ERROR parsing constellation art record at line" << currentLineNumber << "of art file for culture" << cultureName;
 			continue;
 		}
-		
+
 		// Draw loading bar
-		lb.SetMessage(q_("Loading Constellation Art: %1/%2").arg(currentLineNumber).arg(totalRecords));
-		lb.Draw((float)(currentLineNumber)/totalRecords);
-		
+// 		lb.SetMessage(q_("Loading Constellation Art: %1/%2").arg(currentLineNumber).arg(totalRecords));
+// 		lb.Draw((float)(currentLineNumber)/totalRecords);
+
 		cons = NULL;
 		cons = findFromAbbreviation(shortname);
 		if (!cons)
 		{
 			qWarning() << "ERROR in constellation art file at line" << currentLineNumber << "for culture" << cultureName
-			           << "constellation" << shortname << "unknown";
+					   << "constellation" << shortname << "unknown";
 		}
 		else
 		{
-			StelApp::getInstance().getTextureManager().setDefaultParams();
 			QString texturePath(texfile);
 			try
 			{
-				texturePath = StelApp::getInstance().getFileMgr().findFile("skycultures/"+cultureName+"/"+texfile);
+				texturePath = StelFileMgr::findFile("skycultures/"+cultureName+"/"+texfile);
 			}
 			catch (std::runtime_error& e)
 			{
 				// if the texture isn't found in the skycultures/[culture] directory,
 				// try the central textures diectory.
 				qWarning() << "WARNING, could not locate texture file " << texfile
-				     << " in the skycultures/" << cultureName
-				     << " directory...  looking in general textures/ directory...";
+					 << " in the skycultures/" << cultureName
+					 << " directory...  looking in general textures/ directory...";
 				try
 				{
-					texturePath = StelApp::getInstance().getFileMgr().findFile(QString("textures/")+texfile);
+					texturePath = StelFileMgr::findFile(QString("textures/")+texfile);
 				}
 				catch(exception& e2)
 				{
 					qWarning() << "ERROR: could not find texture, " << texfile << ": " << e2.what();
 				}
 			}
-			
+
 			cons->artTexture = StelApp::getInstance().getTextureManager().createTextureThread(texturePath);
-			
+
 			int texSizeX, texSizeY;
 			if (cons->artTexture==NULL || !cons->artTexture->getDimensions(texSizeX, texSizeY))
 			{
@@ -430,16 +424,23 @@ void ConstellationMgr::loadLinesAndArt(const QString &fileName, const QString &a
 			Mat4f A(x1, texSizeY - y1, 0.f, 1.f, x2, texSizeY - y2, 0.f, 1.f, x3, texSizeY - y3, 0.f, 1.f, x1, texSizeY - y1, texSizeX, 1.f);
 			Mat4f X = B * A.inverse();
 
-			cons->artVertex[0] = Vec3f(X * Vec3f(0, 0, 0));
-			cons->artVertex[1] = Vec3f(X * Vec3f(texSizeX / 2, 0, 0));
-			cons->artVertex[2] = Vec3f(X * Vec3f(texSizeX / 2, texSizeY / 2, 0));
-			cons->artVertex[3] = Vec3f(X * Vec3f(0, texSizeY / 2, 0));
-			cons->artVertex[4] = Vec3f(X * Vec3f(texSizeX / 2 + texSizeX / 2, 0, 0));
-			cons->artVertex[5] = Vec3f(X * Vec3f(texSizeX / 2 + texSizeX / 2, texSizeY / 2, 0));
-			cons->artVertex[6] = Vec3f(X * Vec3f(texSizeX / 2 + texSizeX / 2, texSizeY / 2 + texSizeY / 2, 0));
-			cons->artVertex[7] = Vec3f(X * Vec3f(texSizeX / 2 + 0, texSizeY / 2 + texSizeY / 2, 0));
-			cons->artVertex[8] = Vec3f(X * Vec3f(0, texSizeY / 2 + texSizeY / 2, 0));
+			QVector<Vec3d> contour(4);
+			contour[0] = X * Vec3f(0., 0., 0.);
+			contour[1] = X * Vec3f(texSizeX, 0., 0.);
+			contour[2] = X * Vec3f(texSizeX, texSizeY, 0.);
+			contour[3] = X * Vec3f(0, texSizeY, 0.);
+			contour[0].normalize();
+			contour[1].normalize();
+			contour[2].normalize();
+			contour[3].normalize();
 
+			QVector<Vec2f> texCoords(4);
+			texCoords[0].set(0,0);
+			texCoords[1].set(1,0);
+			texCoords[2].set(1,1);
+			texCoords[3].set(0,1);
+			cons->artPolygon.setContour(contour, texCoords);
+			Q_ASSERT(cons->artPolygon.checkValid());
 			++readOk;
 		}
 	}
@@ -453,60 +454,60 @@ void ConstellationMgr::draw(StelCore* core)
 	StelNavigator* nav = core->getNavigator();
 	const StelProjectorP prj = core->getProjection(StelCore::FrameJ2000);
 	StelPainter sPainter(prj);
-	
+	sPainter.setFont(asterFont);
 	drawLines(sPainter, nav);
 	drawNames(sPainter);
-	drawArt(prj, nav);
-	drawBoundaries(prj);
+	drawArt(sPainter);
+	drawBoundaries(sPainter);
 }
 
 // Draw constellations art textures
-void ConstellationMgr::drawArt(const StelProjectorP& prj, const StelNavigator * nav) const
+void ConstellationMgr::drawArt(StelPainter& sPainter) const
 {
 	glBlendFunc(GL_ONE, GL_ONE);
-	glEnable(GL_TEXTURE_2D);
+	sPainter.enableTexture2d(true);
 	glEnable(GL_BLEND);
 	glEnable(GL_CULL_FACE);
 
 	vector < Constellation * >::const_iterator iter;
+	SphericalRegionP region = sPainter.getProjector()->getViewportConvexPolygon();
 	for (iter = asterisms.begin(); iter != asterisms.end(); ++iter)
 	{
-		(*iter)->drawArtOptim(prj, nav);
+		(*iter)->drawArtOptim(sPainter, *region);
 	}
 
 	glDisable(GL_CULL_FACE);
 }
 
 // Draw constellations lines
-void ConstellationMgr::drawLines(const StelPainter& sPainter, const StelNavigator* nav) const
+void ConstellationMgr::drawLines(StelPainter& sPainter, const StelNavigator* nav) const
 {
-	glDisable(GL_TEXTURE_2D);
-	glDisable(GL_BLEND);
-	glEnable(GL_LINE_SMOOTH);
+	sPainter.enableTexture2d(false);
+	glEnable(GL_BLEND);
+	glBlendFunc(GL_SRC_ALPHA, GL_ONE_MINUS_SRC_ALPHA);
+	const SphericalCap viewportHalfspace = sPainter.getProjector()->getBoundingSphericalCap();
 	vector < Constellation * >::const_iterator iter;
 	for (iter = asterisms.begin(); iter != asterisms.end(); ++iter)
 	{
-		(*iter)->drawOptim(sPainter, nav);
+		(*iter)->drawOptim(sPainter, nav, viewportHalfspace);
 	}
-	glDisable(GL_LINE_SMOOTH);
 }
 
 // Draw the names of all the constellations
-void ConstellationMgr::drawNames(const StelPainter& sPainter) const
+void ConstellationMgr::drawNames(StelPainter& sPainter) const
 {
 	glEnable(GL_BLEND);
-	glEnable(GL_TEXTURE_2D);
-	
+	sPainter.enableTexture2d(true);
+
 	glBlendFunc(GL_ONE, GL_ONE);
 	// if (draw_mode == DM_NORMAL) glBlendFunc(GL_ONE, GL_ONE);
 	// else glBlendFunc(GL_SRC_ALPHA, GL_ONE_MINUS_SRC_ALPHA); // charting
-	
 	vector < Constellation * >::const_iterator iter;
 	for (iter = asterisms.begin(); iter != asterisms.end(); iter++)
 	{
 		// Check if in the field of view
 		if (sPainter.getProjector()->projectCheck((*iter)->XYZname, (*iter)->XYname))
-			(*iter)->drawName(asterFont, sPainter);
+			(*iter)->drawName(sPainter);
 	}
 }
 
@@ -516,7 +517,7 @@ Constellation *ConstellationMgr::isStarIn(const StelObject* s) const
 	for (iter = asterisms.begin(); iter != asterisms.end(); ++iter)
 	{
 		// Check if the star is in one of the constellation
-		if ((*iter)->isStarIn(s)) 
+		if ((*iter)->isStarIn(s))
 		{
 			return (*iter);
 		}
@@ -528,7 +529,7 @@ Constellation* ConstellationMgr::findFromAbbreviation(const QString& abbreviatio
 {
 	// search in uppercase only
 	QString tname = abbreviation.toUpper();
-	
+
 	vector < Constellation * >::const_iterator iter;
 	for (iter = asterisms.begin(); iter != asterisms.end(); ++iter)
 	{
@@ -548,7 +549,7 @@ void ConstellationMgr::loadNames(const QString& namesFile)
 {
 	// Constellation not loaded yet
 	if (asterisms.empty()) return;
-	
+
 	// clear previous names
 	vector < Constellation * >::const_iterator iter;
 	for (iter = asterisms.begin(); iter != asterisms.end(); ++iter)
@@ -566,11 +567,11 @@ void ConstellationMgr::loadNames(const QString& namesFile)
 
 	// Now parse the file
 	// lines to ignore which start with a # or are empty
-	QRegExp commentRx("^(\\s*#.*|\\s*)$"); 
+	QRegExp commentRx("^(\\s*#.*|\\s*)$");
 
 	// lines which look like records - we use the RE to extract the fields
 	// which will be available in recRx.capturedTexts()
-	QRegExp recRx("^\\s*(\\w+)\\s+\"(.*)\"\\s+_[(]\"(.*)\"[)]\\n");    
+	QRegExp recRx("^\\s*(\\w+)\\s+\"(.*)\"\\s+_[(]\"(.*)\"[)]\\n");
 
 	// Some more variables to use in the parsing
 	Constellation *aster;
@@ -578,9 +579,9 @@ void ConstellationMgr::loadNames(const QString& namesFile)
 
 	// keep track of how many records we processed.
 	int totalRecords=0;
-	int readOk=0; 
+	int readOk=0;
 	int lineNumber=0;
-	while (!commonNameFile.atEnd()) 
+	while (!commonNameFile.atEnd())
 	{
 		record = QString::fromUtf8(commonNameFile.readLine());
 		lineNumber++;
@@ -606,7 +607,7 @@ void ConstellationMgr::loadNames(const QString& namesFile)
 				aster->englishName = recRx.capturedTexts().at(3);
 				readOk++;
 			}
-			else 
+			else
 			{
 				qWarning() << "WARNING - constellation abbreviation" << shortName << "not found when loading constellation names";
 			}
@@ -618,13 +619,12 @@ void ConstellationMgr::loadNames(const QString& namesFile)
 
 void ConstellationMgr::updateI18n()
 {
-	StelTranslator trans(PACKAGE_NAME "-skycultures", StelApp::getInstance().getFileMgr().getLocaleDir(), StelApp::getInstance().getLocaleMgr().getSkyTranslator().getTrueLocaleName());
+	StelTranslator trans(PACKAGE_NAME "-skycultures", StelFileMgr::getLocaleDir(), StelApp::getInstance().getLocaleMgr().getSkyTranslator().getTrueLocaleName());
 	vector < Constellation * >::const_iterator iter;
 	for (iter = asterisms.begin(); iter != asterisms.end(); ++iter)
 	{
 		(*iter)->nameI18 = trans.qtranslate((*iter)->englishName);
 	}
-	asterFont = &StelApp::getInstance().getFontManager().getStandardFont(trans.getTrueLocaleName(), fontSize);
 }
 
 // update faders
@@ -727,13 +727,13 @@ StelObject* ConstellationMgr::getSelected(void) const {
 	return *selected.begin();  // TODO return all or just remove this method
 }
 
-void ConstellationMgr::setSelected(const QString& abbreviation) 
+void ConstellationMgr::setSelected(const QString& abbreviation)
 {
 	Constellation * c = findFromAbbreviation(abbreviation);
 	if(c != NULL) setSelectedConst(c);
 }
 
-StelObjectP ConstellationMgr::setSelectedStar(const QString& abbreviation) 
+StelObjectP ConstellationMgr::setSelectedStar(const QString& abbreviation)
 {
 	Constellation * c = findFromAbbreviation(abbreviation);
 	if(c != NULL) {
@@ -755,7 +755,7 @@ void ConstellationMgr::setSelectedConst(Constellation * c)
 		c->setFlagName(getFlagLabels());
 		c->setFlagArt(getFlagArt());
 		c->setFlagBoundaries(getFlagBoundaries());
-	
+
 		if (isolateSelected)
 		{
 			vector < Constellation * >::const_iterator iter;
@@ -766,10 +766,10 @@ void ConstellationMgr::setSelectedConst(Constellation * c)
 				vector < Constellation * >::const_iterator s_iter;
 				for (s_iter = selected.begin(); s_iter != selected.end(); ++s_iter)
 				{
-					if( (*iter)==(*s_iter) ) 
+					if( (*iter)==(*s_iter) )
 					{
 						match=true; // this is a selected constellation
-						break;  
+						break;
 					}
 				}
 
@@ -780,10 +780,10 @@ void ConstellationMgr::setSelectedConst(Constellation * c)
 					(*iter)->setFlagName(false);
 					(*iter)->setFlagArt(false);
 					(*iter)->setFlagBoundaries(false);
-		        	}
-             		}
+					}
+					}
 			Constellation::singleSelected = true;  // For boundaries
-        	}
+			}
 		else
 			Constellation::singleSelected = false; // For boundaries
 	}
@@ -819,7 +819,7 @@ void ConstellationMgr::unsetSelectedConst(Constellation * c)
 		int n=0;
 		for (iter = selected.begin(); iter != selected.end(); ++iter)
 		{
-			if( (*iter)->getEnglishName() == c->getEnglishName() ) 
+			if( (*iter)->getEnglishName() == c->getEnglishName() )
 			{
 				selected.erase(selected.begin()+n, selected.begin()+n+1);
 				iter--;
@@ -829,7 +829,7 @@ void ConstellationMgr::unsetSelectedConst(Constellation * c)
 		}
 
 		// If no longer any selection, restore all flags on all constellations
-		if (selected.begin() == selected.end()) { 
+		if (selected.begin() == selected.end()) {
 
 			// Otherwise apply standard flags to all constellations
 			for (iter = asterisms.begin(); iter != asterisms.end(); ++iter)
@@ -860,7 +860,7 @@ bool ConstellationMgr::loadBoundaries(const QString& boundaryFile)
 {
 	Constellation *cons = NULL;
 	unsigned int i, j;
-	
+
 	// delete existing boundaries if any exist
 	vector<vector<Vec3f> *>::iterator iter;
 	for (iter = allBoundarySegments.begin(); iter != allBoundarySegments.end(); ++iter)
@@ -888,7 +888,7 @@ bool ConstellationMgr::loadBoundaries(const QString& boundaryFile)
 	vector<Vec3f> *points = NULL;
 	QString consname;
 	i = 0;
-	while (!istr.atEnd())	
+	while (!istr.atEnd())
 	{
 		points = new vector<Vec3f>;
 
@@ -910,23 +910,23 @@ bool ConstellationMgr::loadBoundaries(const QString& boundaryFile)
 			StelUtils::spheToRect(RA,DE,XYZ);
 			points->push_back(XYZ);
 		}
-		
+
 		// this list is for the de-allocation
 		allBoundarySegments.push_back(points);
 
-		istr >> numc;  
+		istr >> numc;
 		// there are 2 constellations per boundary
-		
+
 		for (j=0;j<numc;j++)
 		{
 			istr >> consname;
 			// not used?
 			if (consname == "SER1" || consname == "SER2") consname = "SER";
-			
+
 			cons = findFromAbbreviation(consname);
 			if (!cons)
 				qWarning() << "ERROR while processing boundary file - cannot find constellation: " << consname;
-			else 
+			else
 				cons->isolatedBoundarySegments.push_back(points);
 		}
 
@@ -941,21 +941,22 @@ bool ConstellationMgr::loadBoundaries(const QString& boundaryFile)
 	return true;
 }
 
-void ConstellationMgr::drawBoundaries(const StelProjectorP& prj) const
+void ConstellationMgr::drawBoundaries(StelPainter& sPainter) const
 {
-	glDisable(GL_TEXTURE_2D);
+	sPainter.enableTexture2d(false);
 	glDisable(GL_BLEND);
-	glEnable(GL_LINE_SMOOTH);
+#ifndef USE_OPENGL_ES2
 	glLineStipple(2, 0x3333);
 	glEnable(GL_LINE_STIPPLE);
-
+#endif
 	vector < Constellation * >::const_iterator iter;
 	for (iter = asterisms.begin(); iter != asterisms.end(); ++iter)
 	{
-		(*iter)->drawBoundaryOptim(prj);
+		(*iter)->drawBoundaryOptim(sPainter);
 	}
+#ifndef USE_OPENGL_ES2
 	glDisable(GL_LINE_STIPPLE);
-	glDisable(GL_LINE_SMOOTH);
+#endif
 }
 
 ///unsigned int ConstellationMgr::getFirstSelectedHP(void) {
@@ -966,7 +967,7 @@ void ConstellationMgr::drawBoundaries(const StelProjectorP& prj) const
 StelObjectP ConstellationMgr::searchByNameI18n(const QString& nameI18n) const
 {
 	QString objw = nameI18n.toUpper();
-	
+
 	vector <Constellation*>::const_iterator iter;
 	for (iter = asterisms.begin(); iter != asterisms.end(); ++iter)
 	{
@@ -995,9 +996,9 @@ QStringList ConstellationMgr::listMatchingObjectsI18n(const QString& objPrefix,
 {
 	QStringList result;
 	if (maxNbItem==0) return result;
-		
+
 	QString objw = objPrefix.toUpper();
-	
+
 	vector < Constellation * >::const_iterator iter;
 	for (iter = asterisms.begin(); iter != asterisms.end(); ++iter)
 	{
