@@ -17,14 +17,11 @@
  * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
  */
 
-
 #include "LabelMgr.hpp"
 #include "StelObjectMgr.hpp"
 #include "StelApp.hpp"
 #include "StarMgr.hpp"
 #include "StelCore.hpp"
-#include "StelFont.hpp"
-#include "StelFontMgr.hpp"
 #include "StelLocaleMgr.hpp"
 #include "StelModuleMgr.hpp"
 #include "StelNavigator.hpp"
@@ -39,17 +36,18 @@
 #include <vector>
 #include <QString>
 #include <QDebug>
+#include <QtOpenGL>
 
 // Base class from which other label types inherit
 class StelLabel
 {
 public:
-	StelLabel(const QString& text, StelFont* font, const Vec3f& color);
+	StelLabel(const QString& text, const QFont& font, const Vec3f& color);
 	virtual ~StelLabel() {;}
 
 	//! draw the label on the sky
 	//! @param core the StelCore object
-	virtual bool draw(StelCore* core, const StelPainter& sPainter) = 0;
+	virtual bool draw(StelCore* core, StelPainter& sPainter) = 0;
 	//! update fade for on/off action
 	virtual void update(double deltaTime);
 	//! Set the duration used for the fade in / fade out of the label.
@@ -60,10 +58,12 @@ public:
 	virtual void setFlagShow(bool b);
 	//! Get value of flag used to turn on and off the label
 	virtual bool getFlagShow(void);
+	//! Get value of flag used to turn on and off the label
+	virtual void setText(const QString& newText);
 
 protected:
 	QString labelText;
-	StelFont* labelFont;
+	QFont labelFont;
 	Vec3f labelColor;
 	LinearFader labelFader;
 };
@@ -90,7 +90,7 @@ public:
 	//! @param distance the distance from the object to draw the label.  If < 0.0, placement is automatic.
 	//! @param style determines how the label is drawn
 	//! @param enclosureSize determines the size of the enclosure for styles Box and Circle
-	SkyLabel(const QString& text, StelObjectP bindObject, StelFont* font, Vec3f color,
+	SkyLabel(const QString& text, StelObjectP bindObject, const QFont& font, Vec3f color,
 			 QString side="NE", double distance=-1.0, SkyLabel::Style style=TextOnly, 
 	double enclosureSize=0.0);
 
@@ -100,7 +100,7 @@ public:
 	//! Draw the label on the sky
 	//! @param core the StelCore object
 	//! @param sPainter the StelPainter to use for drawing operations
-	virtual bool draw(StelCore* core, const StelPainter& sPainter);
+	virtual bool draw(StelCore* core, StelPainter& sPainter);
 
 	static SkyLabel::Style stringToStyle(const QString& s);
 	
@@ -120,16 +120,16 @@ public:
 	//! Constructor of a SkyLabel which is to be displayed at a fixed position on the screen.
 	//! @param text the text for the label
 	//! @param x the x-position on the screen (pixels from the left side)
-	//! @param y the y-position on the screen (pixels from the bottom side)
+	//! @param y the y-position on the screen (pixels from the top side)
 	//! @param font the font to use
 	//! @param color the color for the label
-	ScreenLabel(const QString& text, int x, int y, StelFont* font, Vec3f color);
+	ScreenLabel(const QString& text, int x, int y, const QFont& font, const Vec3f& color);
 	virtual ~ScreenLabel();
 
 	//! draw the label on the sky
 	//! @param core the StelCore object
 	//! @param sPainter the StelPainter to use for drawing operations
-	virtual bool draw(StelCore* core, const StelPainter& sPainter);
+	virtual bool draw(StelCore* core, StelPainter& sPainter);
 
 private:
 	int screenX;
@@ -139,7 +139,7 @@ private:
 /////////////////////
 // StelLabel class //
 /////////////////////
-StelLabel::StelLabel(const QString& text, StelFont* font, const Vec3f& color)
+StelLabel::StelLabel(const QString& text, const QFont& font, const Vec3f& color)
 	: labelText(text),
 	  labelFont(font),
 	  labelColor(color)
@@ -171,6 +171,11 @@ bool StelLabel::getFlagShow(void)
 	return labelFader;
 }
 
+void StelLabel::setText(const QString& newText)
+{
+	labelText = newText;
+}
+
 SkyLabel::Style SkyLabel::stringToStyle(const QString& s)
 {
 	if (s=="Line")
@@ -182,7 +187,7 @@ SkyLabel::Style SkyLabel::stringToStyle(const QString& s)
 ////////////////////
 // SkyLabel class //
 ////////////////////
-SkyLabel::SkyLabel(const QString& text, StelObjectP bindObject, StelFont* font,
+SkyLabel::SkyLabel(const QString& text, StelObjectP bindObject, const QFont& font,
                    Vec3f color, QString side, double distance, SkyLabel::Style style,
                    double enclosureSize)
 	: StelLabel(text, font, color),
@@ -198,7 +203,7 @@ SkyLabel::~SkyLabel()
 {
 }
 
-bool SkyLabel::draw(StelCore* core, const StelPainter& sPainter)
+bool SkyLabel::draw(StelCore* core, StelPainter& sPainter)
 {
 	if(labelFader.getInterstate() <= 0.0)
 		return false;
@@ -207,6 +212,8 @@ bool SkyLabel::draw(StelCore* core, const StelPainter& sPainter)
 	Vec3d labelXY;
 	sPainter.getProjector()->project(objectPos,labelXY);
 
+	sPainter.setFont(labelFont);
+			
 	double xOffset(0.);
 	double yOffset(0.);
 	char hJustify = 'c';
@@ -215,7 +222,7 @@ bool SkyLabel::draw(StelCore* core, const StelPainter& sPainter)
 	if (labelSide.toUpper().contains("N"))
 	{
 		yOffset = 1.0;
-		vJustify = 'b'; // bottom justufy text
+		vJustify = 'b'; // bottom justify text
 	}
 	else if (labelSide.toUpper().contains("S"))
 	{
@@ -250,21 +257,21 @@ bool SkyLabel::draw(StelCore* core, const StelPainter& sPainter)
 	double jxOffset(0.);
 	double jyOffset(0.); 
 	if (hJustify == 'r')
-		jxOffset = labelFont->getStrLen(labelText);
+		jxOffset = sPainter.getFontMetrics().width(labelText);
 	else if (hJustify == 'c')
-		jxOffset = labelFont->getStrLen(labelText) / 2.;
+		jxOffset = sPainter.getFontMetrics().width(labelText) / 2.;
 
 	if (vJustify == 't')
-		jyOffset = labelFont->getLineHeight();
+		jyOffset = sPainter.getFontMetrics().height();
 	else if (vJustify == 'c')
-		jyOffset = labelFont->getLineHeight() / 2.;
+		jyOffset = sPainter.getFontMetrics().height() / 2.;
 
-	glColor4f(labelColor[0], labelColor[1], labelColor[2], labelFader.getInterstate());
-	sPainter.drawText(labelFont, labelXY[0]+xOffset-jxOffset, labelXY[1]+yOffset-jyOffset, labelText, 0, 0, 0, false);
+	sPainter.setColor(labelColor[0], labelColor[1], labelColor[2], labelFader.getInterstate());
+	sPainter.drawText(labelXY[0]+xOffset-jxOffset, labelXY[1]+yOffset-jyOffset, labelText, 0, 0, 0, false);
 
 	if (labelStyle == SkyLabel::Line)
 	{
-		glDisable(GL_TEXTURE_2D);
+		sPainter.enableTexture2d(false);
 		glEnable(GL_BLEND);
 		glBlendFunc(GL_SRC_ALPHA, GL_ONE_MINUS_SRC_ALPHA);
 
@@ -285,14 +292,9 @@ bool SkyLabel::draw(StelCore* core, const StelPainter& sPainter)
 		else if (hJustify == 'r')
 			lineEndX += 5;
 				
-		glColor4f(labelColor[0], labelColor[1], labelColor[2], labelFader.getInterstate());
+		sPainter.setColor(labelColor[0], labelColor[1], labelColor[2], labelFader.getInterstate());
 		
-		glEnable(GL_LINE_SMOOTH);
-		glBegin(GL_LINES);
-			glVertex2f(lineEndX,lineEndY);
-			glVertex2f(objXY[0], objXY[1]);
-		glEnd();
-		glDisable(GL_LINE_SMOOTH);
+		sPainter.drawLine2d(lineEndX,lineEndY,objXY[0], objXY[1]);
 	}
 
 	return true;
@@ -302,24 +304,26 @@ bool SkyLabel::draw(StelCore* core, const StelPainter& sPainter)
 ///////////////////////
 // ScreenLabel class //
 ///////////////////////
-ScreenLabel::ScreenLabel(const QString& text, int x, int y, StelFont* font, Vec3f color)
+ScreenLabel::ScreenLabel(const QString& text, int x, int y, const QFont& font, const Vec3f& color)
 	: StelLabel(text, font, color),
-	  screenX(x),
-	  screenY(y)
+	  screenX(x)
 {
+	QFontMetrics metrics(font);
+	screenY = StelApp::getInstance().getCore()->getProjection2d()->getViewportHeight() - y - metrics.height();
 }
 
 ScreenLabel::~ScreenLabel()
 {
 }
 
-bool ScreenLabel::draw(StelCore* core, const StelPainter& sPainter)
+bool ScreenLabel::draw(StelCore* core, StelPainter& sPainter)
 {
-	if(labelFader.getInterstate() <= 0.0)
+	if (labelFader.getInterstate() <= 0.0)
 		return false;
 
-	glColor4f(labelColor[0], labelColor[1], labelColor[2], labelFader.getInterstate());
-	sPainter.drawText(labelFont, screenX, screenY, labelText, 0, 0, 0, false);
+	sPainter.setColor(labelColor[0], labelColor[1], labelColor[2], labelFader.getInterstate());
+	sPainter.setFont(labelFont);
+	sPainter.drawText(screenX, screenY, labelText, 0, 0, 0, false);
 	return true;
 }
 
@@ -356,8 +360,8 @@ int LabelMgr::labelObject(const QString& text,
                           double labelDistance,
                           const QString& style)
 {
-	StelFont* font = &StelApp::getInstance().getFontManager().getStandardFont(StelApp::getInstance().getLocaleMgr().getSkyLanguage(), fontSize);
-	Q_ASSERT(font);
+	QFont font;
+	font.setPixelSize(fontSize);
 	StelObjectP obj = GETSTELMODULE(StelObjectMgr)->searchByName(objectName);
 	if (!obj)
 	{
@@ -377,14 +381,14 @@ int LabelMgr::labelObject(const QString& text,
 }
 
 int LabelMgr::labelScreen(const QString& text,
-                             int x,
-                             int y,
-                             bool visible,
-                             float fontSize,
-                             const QString& fontColor)
+                          int x,
+                          int y,
+                          bool visible,
+                          float fontSize,
+                          const QString& fontColor)
 {
-	StelFont* font = &StelApp::getInstance().getFontManager().getStandardFont(StelApp::getInstance().getLocaleMgr().getSkyLanguage(), fontSize);
-	Q_ASSERT(font);
+	QFont font;
+	font.setPixelSize(fontSize);
 	ScreenLabel* l = new ScreenLabel(text, x, y, font, StelUtils::htmlColorToVec3f(fontColor));
 	if (l==NULL)
 		return -1;
@@ -409,6 +413,12 @@ void LabelMgr::setLabelShow(int id, bool show)
 	if (allLabels.at(id)!=NULL)
 		allLabels.at(id)->setFlagShow(show);
 }
+
+void LabelMgr::setLabelText(int id, const QString& newText)
+{
+	if (allLabels.at(id)!=NULL)
+		allLabels.at(id)->setText(newText);
+}
 	
 bool LabelMgr::deleteLabel(int id)
 {
