@@ -1,17 +1,17 @@
 /*
  * Stellarium
  * Copyright (C) 2002 Fabien Chereau
- * 
+ *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
  * as published by the Free Software Foundation; either version 2
  * of the License, or (at your option) any later version.
- * 
+ *
  * This program is distributed in the hope that it will be useful,
  * but WITHOUT ANY WARRANTY; without even the implied warranty of
  * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  * GNU General Public License for more details.
- * 
+ *
  * You should have received a copy of the GNU General Public License
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
@@ -37,7 +37,6 @@
 #include "StelLoadingBar.hpp"
 #include "StelTextureMgr.hpp"
 #include "StelObjectMgr.hpp"
-#include "StelFontMgr.hpp"
 #include "StelLocaleMgr.hpp"
 #include "StelSkyCultureMgr.hpp"
 #include "StelFileMgr.hpp"
@@ -47,8 +46,6 @@
 #include "StelSkyImageTile.hpp"
 #include "StelPainter.hpp"
 
-using namespace std;
-
 void NebulaMgr::setLabelsColor(const Vec3f& c) {Nebula::labelColor = c;}
 const Vec3f &NebulaMgr::getLabelsColor(void) const {return Nebula::labelColor;}
 void NebulaMgr::setCirclesColor(const Vec3f& c) {Nebula::circleColor = c;}
@@ -65,12 +62,6 @@ NebulaMgr::NebulaMgr(void) : nebGrid(200), displayNoTexture(false)
 NebulaMgr::~NebulaMgr()
 {
 	Nebula::texCircle = StelTextureSP();
-	
-	vector<Nebula *>::iterator iter;
-	for(iter=nebArray.begin();iter!=nebArray.end();iter++)
-	{
-		delete (*iter);
-	}
 }
 
 /*************************************************************************
@@ -98,15 +89,10 @@ void NebulaMgr::init()
 	QSettings* conf = StelApp::getInstance().getSettings();
 	Q_ASSERT(conf);
 
-	double fontSize = 12;
-	Nebula::nebulaFont = &StelApp::getInstance().getFontManager().getStandardFont(StelApp::getInstance().getLocaleMgr().getSkyLanguage(), fontSize);
-
-	StelApp::getInstance().getTextureManager().setDefaultParams();
-	StelApp::getInstance().getTextureManager().setMinFilter(GL_LINEAR);
+	nebulaFont.setPixelSize(13);
 	Nebula::texCircle = StelApp::getInstance().getTextureManager().createTexture("neb.png");   // Load circle texture
-
 	texPointer = StelApp::getInstance().getTextureManager().createTexture("pointeur5.png");   // Load pointer texture
-	
+
 	setFlagShow(conf->value("astro/flag_nebula",true).toBool());
 	setFlagHints(conf->value("astro/flag_nebula_name",false).toBool());
 	setHintsAmount(conf->value("astro/nebula_hints_amount", 3).toDouble());
@@ -115,85 +101,90 @@ void NebulaMgr::init()
 	setFlagDisplayNoTexture(conf->value("astro/flag_nebula_display_no_texture", false).toBool());
 
 	updateI18n();
-	
+
 	GETSTELMODULE(StelObjectMgr)->registerStelObjectMgr(this);
 }
 
+struct DrawNebulaFuncObject
+{
+	DrawNebulaFuncObject(float amaxMagHints, float amaxMagLabels, StelPainter* p, bool acheckMaxMagHints) : maxMagHints(amaxMagHints), maxMagLabels(amaxMagLabels), sPainter(p), checkMaxMagHints(acheckMaxMagHints)
+	{
+		angularSizeLimit = 5./sPainter->getProjector()->getPixelPerRadAtCenter()*180./M_PI;
+	}
+	void operator()(StelRegionObjectP obj)
+	{
+		Nebula* n = obj.staticCast<Nebula>().data();
+		if (n->angularSize>angularSizeLimit || (checkMaxMagHints && n->mag <= maxMagHints))
+		{
+			sPainter->getProjector()->project(n->XYZ,n->XY);
+			n->drawLabel(*sPainter, maxMagLabels);
+			n->drawHints(*sPainter, maxMagHints);
+		}
+	}
+	float maxMagHints;
+	float maxMagLabels;
+	StelPainter* sPainter;
+	float angularSizeLimit;
+	bool checkMaxMagHints;
+};
+
 // Draw all the Nebulae
 void NebulaMgr::draw(StelCore* core)
 {
 	const StelProjectorP prj = core->getProjection(StelCore::FrameJ2000);
 	StelPainter sPainter(prj);
-	
+
 	StelSkyDrawer* skyDrawer = core->getSkyDrawer();
-	
+
 	Nebula::hintsBrightness = hintsFader.getInterstate()*flagShow.getInterstate();
-	
-	glEnable(GL_TEXTURE_2D);
+
+	sPainter.enableTexture2d(true);
 	glEnable(GL_BLEND);
 	glBlendFunc(GL_ONE, GL_ONE);
 
-	Vec3f pXYZ;
-	
 	// Use a 1 degree margin
 	const double margin = 1.*M_PI/180.*prj->getPixelPerRadAtCenter();
-	const StelGeom::ConvexPolygon& p = prj->getViewportConvexPolygon(margin, margin);
-	nebGrid.filterIntersect(p);
-	
-	// Print all the nebulae of all the selected zones
-	Nebula* n;
+	const SphericalRegionP& p = prj->getViewportConvexPolygon(margin, margin);
 
-	// speed up the computation of n->getOnScreenSize(core)>5:
-	const float size_limit = 5./prj->getPixelPerRadAtCenter()*180./M_PI;
+	// Print all the nebulae of all the selected zones
 	float maxMagHints = skyDrawer->getLimitMagnitude()*1.2-2.+(hintsAmount*1.2f)-2.f;
 	float maxMagLabels = skyDrawer->getLimitMagnitude()-2.+(labelsAmount*1.2f)-2.f;
+	sPainter.setFont(nebulaFont);
+	DrawNebulaFuncObject func(maxMagHints, maxMagLabels, &sPainter, hintsFader.getInterstate()>0.0001);
+	nebGrid.processIntersectingRegions(p, func);
 	
-	for (StelTreeGrid::const_iterator iter = nebGrid.begin(); iter != nebGrid.end(); ++iter)
-	{
-		n = static_cast<Nebula*>(*iter);
-
-		// improve performance by skipping if too small to see
-		if (n->angularSize>size_limit || (hintsFader.getInterstate()>0.0001 && n->mag <= maxMagHints))
-		{
-			prj->project(n->XYZ,n->XY);
-			n->drawLabel(core, sPainter, maxMagLabels);
-			n->drawHints(sPainter, maxMagHints);
-		}
-	}
 	if (GETSTELMODULE(StelObjectMgr)->getFlagSelectedObjectPointer())
 		drawPointer(core, sPainter);
-	//nebGrid.draw(prj, p);
 }
 
-void NebulaMgr::drawPointer(const StelCore* core, const StelPainter& sPainter)
+void NebulaMgr::drawPointer(const StelCore* core, StelPainter& sPainter)
 {
 	const StelNavigator* nav = core->getNavigator();
 	const StelProjectorP prj = core->getProjection(StelCore::FrameJ2000);
-	
+
 	const QList<StelObjectP> newSelected = GETSTELMODULE(StelObjectMgr)->getSelectedObject("Nebula");
 	if (!newSelected.empty())
 	{
 		const StelObjectP obj = newSelected[0];
 		Vec3d pos=obj->getJ2000EquatorialPos(nav);
-		Vec3d screenpos;
-		
+
 		// Compute 2D pos and return if outside screen
-		if (!prj->project(pos, screenpos)) return;
-		glColor3f(0.4f,0.5f,0.8f);
+		if (!prj->projectInPlace(pos)) return;
+		sPainter.setColor(0.4f,0.5f,0.8f);
 		texPointer->bind();
-		
-		glEnable(GL_TEXTURE_2D);
-        glEnable(GL_BLEND);
-        glBlendFunc(GL_SRC_ALPHA, GL_ONE_MINUS_SRC_ALPHA); // Normal transparency mode
+
+		sPainter.enableTexture2d(true);
+		glEnable(GL_BLEND);
+		glBlendFunc(GL_SRC_ALPHA, GL_ONE_MINUS_SRC_ALPHA); // Normal transparency mode
 
 		// Size on screen
 		float size = obj->getAngularSize(core)*M_PI/180.*prj->getPixelPerRadAtCenter();
 
 		size+=20.f + 10.f*std::sin(2.f * StelApp::getInstance().getTotalRunTime());
-		sPainter.drawSprite2dMode(screenpos[0]-size/2, screenpos[1]-size/2, 20, 90);
-		sPainter.drawSprite2dMode(screenpos[0]-size/2, screenpos[1]+size/2, 20, 0);
-		sPainter.drawSprite2dMode(screenpos[0]+size/2, screenpos[1]+size/2, 20, -90);
-		sPainter.drawSprite2dMode(screenpos[0]+size/2, screenpos[1]-size/2, 20, -180);
+		sPainter.drawSprite2dMode(pos[0]-size/2, pos[1]-size/2, 10, 90);
+		sPainter.drawSprite2dMode(pos[0]-size/2, pos[1]+size/2, 10, 0);
+		sPainter.drawSprite2dMode(pos[0]+size/2, pos[1]+size/2, 10, -90);
+		sPainter.drawSprite2dMode(pos[0]+size/2, pos[1]-size/2, 10, -180);
 	}
 }
 
@@ -205,44 +196,43 @@ void NebulaMgr::setStelStyle(const StelStyle& style)
 	// Load colors from config file
 	QSettings* conf = StelApp::getInstance().getSettings();
 	QString section = style.confSectionName;
-	
+
 	QString defaultColor = conf->value(section+"/default_color").toString();
 	setLabelsColor(StelUtils::strToVec3f(conf->value(section+"/nebula_label_color", defaultColor).toString()));
-	setCirclesColor(StelUtils::strToVec3f(conf->value(section+"/nebula_circleColor", defaultColor).toString()));
+	setCirclesColor(StelUtils::strToVec3f(conf->value(section+"/nebula_circle_color", defaultColor).toString()));
 }
 
 // Search by name
-StelObject* NebulaMgr::search(const QString& name)
+NebulaP NebulaMgr::search(const QString& name)
 {
 	QString uname = name.toUpper();
-	vector <Nebula*>::const_iterator iter;
 
-	for (iter = nebArray.begin(); iter != nebArray.end(); ++iter)
+	foreach (const NebulaP& n, nebArray)
 	{
-		QString testName = (*iter)->getEnglishName().toUpper();
-		if (testName==uname) return *iter;
+		QString testName = n->getEnglishName().toUpper();
+		if (testName==uname) return n;
 	}
-	
+
 	// If no match found, try search by catalog reference
 	QRegExp catNumRx("^(M|NGC|IC)\\s*(\\d+)$");
 	if (catNumRx.exactMatch(uname))
 	{
 		QString cat = catNumRx.capturedTexts().at(1);
 		int num = catNumRx.capturedTexts().at(2).toInt();
-		
+
 		if (cat == "M") return searchM(num);
 		if (cat == "NGC") return searchNGC(num);
 		if (cat == "IC") return searchIC(num);
 	}
-	return NULL;
+	return NebulaP();
 }
 
 void NebulaMgr::loadNebulaSet(const QString& setName)
 {
 	try
 	{
-		loadNGC(StelApp::getInstance().getFileMgr().findFile("nebulae/" + setName + "/ngc2000.dat"));
-		loadNGCNames(StelApp::getInstance().getFileMgr().findFile("nebulae/" + setName + "/ngc2000names.dat"));
+		loadNGC(StelFileMgr::findFile("nebulae/" + setName + "/ngc2000.dat"));
+		loadNGCNames(StelFileMgr::findFile("nebulae/" + setName + "/ngc2000names.dat"));
 	}
 	catch (std::runtime_error& e)
 	{
@@ -251,25 +241,25 @@ void NebulaMgr::loadNebulaSet(const QString& setName)
 }
 
 // Look for a nebulae by XYZ coords
-StelObject* NebulaMgr::search(Vec3f Pos)
+NebulaP NebulaMgr::search(const Vec3d& apos)
 {
-	Pos.normalize();
-	vector<Nebula *>::iterator iter;
-	Nebula * plusProche=NULL;
+	Vec3d pos = apos;
+	pos.normalize();
+	NebulaP plusProche;
 	float anglePlusProche=0.;
-	for(iter=nebArray.begin();iter!=nebArray.end();iter++)
+	foreach (const NebulaP& n, nebArray)
 	{
-		if ((*iter)->XYZ[0]*Pos[0]+(*iter)->XYZ[1]*Pos[1]+(*iter)->XYZ[2]*Pos[2]>anglePlusProche)
+		if (n->XYZ*pos>anglePlusProche)
 		{
-			anglePlusProche=(*iter)->XYZ[0]*Pos[0]+(*iter)->XYZ[1]*Pos[1]+(*iter)->XYZ[2]*Pos[2];
-			plusProche=(*iter);
+			anglePlusProche=n->XYZ*pos;
+			plusProche=n;
 		}
 	}
 	if (anglePlusProche>0.999)
 	{
 		return plusProche;
 	}
-	else return NULL;
+	else return NebulaP();
 }
 
 // Return a stl vector containing the nebulas located inside the limFov circle around position v
@@ -278,66 +268,48 @@ QList<StelObjectP> NebulaMgr::searchAround(const Vec3d& av, double limitFov, con
 	QList<StelObjectP> result;
 	if (!getFlagShow())
 		return result;
-		
+
 	Vec3d v(av);
 	v.normalize();
 	double cosLimFov = cos(limitFov * M_PI/180.);
 	Vec3d equPos;
-
-	vector<Nebula*>::const_iterator iter = nebArray.begin();
-	while (iter != nebArray.end())
+	foreach (const NebulaP& n, nebArray)
 	{
-		equPos = (*iter)->XYZ;
+		equPos = n->XYZ;
 		equPos.normalize();
-		if (equPos[0]*v[0] + equPos[1]*v[1] + equPos[2]*v[2]>=cosLimFov)
+		if (equPos*v>=cosLimFov)
 		{
-
-			// NOTE: non-labeled nebulas are not returned!
-			// Otherwise cursor select gets invisible nebulas - Rob
-			//if((*iter)->getNameI18n() != L"") 
-				result.push_back(StelObjectP(*iter));
+			result.push_back(qSharedPointerCast<StelObject>(n));
 		}
-		iter++;
 	}
 	return result;
 }
 
-Nebula *NebulaMgr::searchM(unsigned int M)
+NebulaP NebulaMgr::searchM(unsigned int M)
 {
-	vector<Nebula *>::iterator iter;
-	for(iter=nebArray.begin();iter!=nebArray.end();iter++)
-	{
-		if ((*iter)->M_nb == M) return (*iter);
-	}
-
-	return NULL;
+	foreach (const NebulaP& n, nebArray)
+		if (n->M_nb == M)
+			return n;
+	return NebulaP();
 }
 
-Nebula *NebulaMgr::searchNGC(unsigned int NGC)
+NebulaP NebulaMgr::searchNGC(unsigned int NGC)
 {
-	vector<Nebula *>::iterator iter;
-	for(iter=nebArray.begin();iter!=nebArray.end();++iter)
-	{
-		if ((*iter)->NGC_nb == NGC)
-			return (*iter);
-	}
-	return NULL;
+	if (ngcIndex.contains(NGC))
+		return ngcIndex[NGC];
+	return NebulaP();
 }
 
-Nebula *NebulaMgr::searchIC(unsigned int IC)
+NebulaP NebulaMgr::searchIC(unsigned int IC)
 {
-	vector<Nebula *>::iterator iter;
-	for(iter=nebArray.begin();iter!=nebArray.end();iter++)
-	{
-		if ((*iter)->IC_nb == IC) return (*iter);
-	}
-
-	return NULL;
+	foreach (const NebulaP& n, nebArray)
+		if (n->IC_nb == IC) return n;
+	return NebulaP();
 }
 
-
+#if 0
 // read from stream
-bool NebulaMgr::loadNGC(const QString& catNGC)
+bool NebulaMgr::loadNGCOld(const QString& catNGC)
 {
 	StelLoadingBar& lb = *StelApp::getInstance().getStelLoadingBar();
 	QFile in(catNGC);
@@ -346,12 +318,10 @@ bool NebulaMgr::loadNGC(const QString& catNGC)
 
 	int totalRecords=0;
 	QString record;
-	QRegExp commentRx("^(\\s*#.*|\\s*)$");
-	while (!in.atEnd()) 
+	while (!in.atEnd())
 	{
-		record = QString::fromUtf8(in.readLine());
-		if (!commentRx.exactMatch(record))
-			totalRecords++;
+		in.readLine();
+		++totalRecords;
 	}
 
 	// rewind the file to the start
@@ -366,12 +336,11 @@ bool NebulaMgr::loadNGC(const QString& catNGC)
 		++currentLineNumber;
 
 		// skip comments
-		if (commentRx.exactMatch(record))
+		if (record.startsWith("//") || record.startsWith("#"))
 			continue;
-
 		++currentRecordNumber;
 
-		// Update the status bar every 200 record 
+		// Update the status bar every 200 record
 		if (!(currentRecordNumber%200) || (currentRecordNumber == totalRecords))
 		{
 			lb.SetMessage(q_("Loading NGC catalog: %1/%2").arg(currentRecordNumber).arg(totalRecords));
@@ -379,16 +348,17 @@ bool NebulaMgr::loadNGC(const QString& catNGC)
 		}
 
 		// Create a new Nebula record
-		Nebula *e = new Nebula;
+		NebulaP e = NebulaP(new Nebula);
 		if (!e->readNGC((char*)record.toLocal8Bit().data())) // reading error
 		{
-			delete e;
-			e = NULL;
+			e.clear();
 		}
 		else
 		{
-			nebArray.push_back(e);
-			nebGrid.insert(e);
+			nebArray.append(e);
+			nebGrid.insert(qSharedPointerCast<StelRegionObject>(e));
+			if (e->NGC_nb!=0)
+				ngcIndex.insert(e->NGC_nb, e);
 			++readOk;
 		}
 	}
@@ -396,7 +366,36 @@ bool NebulaMgr::loadNGC(const QString& catNGC)
 	qDebug() << "Loaded" << readOk << "/" << totalRecords << "NGC records";
 	return true;
 }
+#endif
 
+bool NebulaMgr::loadNGC(const QString& catNGC)
+{
+	StelLoadingBar& lb = *StelApp::getInstance().getStelLoadingBar();
+	QFile in(catNGC);
+	if (!in.open(QIODevice::ReadOnly))
+		return false;
+	QDataStream ins(&in);
+	ins.setVersion(QDataStream::Qt_4_5);
+	lb.SetMessage(q_("Loading NGC catalog"));
+	lb.Draw(0);
+	
+	int totalRecords=0;
+	while (!ins.atEnd())
+	{
+		// Create a new Nebula record
+		NebulaP e = NebulaP(new Nebula);
+		e->readNGC(ins);
+		
+		nebArray.append(e);
+		nebGrid.insert(qSharedPointerCast<StelRegionObject>(e));
+		if (e->NGC_nb!=0)
+			ngcIndex.insert(e->NGC_nb, e);
+		++totalRecords;
+	}
+	in.close();
+	qDebug() << "Loaded" << totalRecords << "NGC records";
+	return true;
+}
 
 bool NebulaMgr::loadNGCNames(const QString& catNGCNames)
 {
@@ -414,7 +413,7 @@ bool NebulaMgr::loadNGCNames(const QString& catNGCNames)
 	int lineNumber=0;
 	int readOk=0;
 	int nb;
-	Nebula *e;
+	NebulaP e;
 	QRegExp commentRx("^(\\s*#.*|\\s*)$");
 	while (!ngcNameFile.atEnd())
 	{
@@ -422,21 +421,21 @@ bool NebulaMgr::loadNGCNames(const QString& catNGCNames)
 		lineNumber++;
 		if (commentRx.exactMatch(record))
 			continue;
-	
+
 		totalRecords++;
 		nb = record.mid(38,4).toInt();
 		if (record[37] == 'I')
 		{
-			e = (Nebula*)searchIC(nb);
+			e = searchIC(nb);
 		}
 		else
 		{
-			e = (Nebula*)searchNGC(nb);
+			e = searchNGC(nb);
 		}
 
 		// get name, trimmed of whitespace
 		name = record.left(36).trimmed();
-		
+
 		if (e)
 		{
 			// If the name is not a messier number perhaps one is already
@@ -479,13 +478,8 @@ bool NebulaMgr::loadNGCNames(const QString& catNGCNames)
 void NebulaMgr::updateI18n()
 {
 	StelTranslator trans = StelApp::getInstance().getLocaleMgr().getSkyTranslator();
-	vector<Nebula*>::iterator iter;
-	for( iter = nebArray.begin(); iter < nebArray.end(); iter++ )
-	{
-		(*iter)->translateName(trans);
-	}
-	double fontSize = 12;
-	Nebula::nebulaFont = &StelApp::getInstance().getFontManager().getStandardFont(trans.getTrueLocaleName(), fontSize);
+	foreach (NebulaP n, nebArray)
+		n->translateName(trans);
 }
 
 
@@ -493,37 +487,36 @@ void NebulaMgr::updateI18n()
 StelObjectP NebulaMgr::searchByNameI18n(const QString& nameI18n) const
 {
 	QString objw = nameI18n.toUpper();
-	vector <Nebula*>::const_iterator iter;
-	
+
 	// Search by NGC numbers (possible formats are "NGC31" or "NGC 31")
 	if (objw.mid(0, 3) == "NGC")
 	{
-		for (iter = nebArray.begin(); iter != nebArray.end(); ++iter)
+		foreach (const NebulaP& n, nebArray)
 		{
-			if (QString("NGC%1").arg((*iter)->NGC_nb) == objw || QString("NGC %1").arg((*iter)->NGC_nb) == objw)
-				return *iter;
+			if (QString("NGC%1").arg(n->NGC_nb) == objw || QString("NGC %1").arg(n->NGC_nb) == objw)
+				return qSharedPointerCast<StelObject>(n);
 		}
 	}
-	
+
 	// Search by common names
-	for (iter = nebArray.begin(); iter != nebArray.end(); ++iter)
+	foreach (const NebulaP& n, nebArray)
 	{
-		QString objwcap = (*iter)->nameI18.toUpper();
-		if (objwcap==objw) 
-			return *iter;
+		QString objwcap = n->nameI18.toUpper();
+		if (objwcap==objw)
+			return qSharedPointerCast<StelObject>(n);
 	}
-	
+
 	// Search by Messier numbers (possible formats are "M31" or "M 31")
 	if (objw.mid(0, 1) == "M")
 	{
-		for (iter = nebArray.begin(); iter != nebArray.end(); ++iter)
+		foreach (const NebulaP& n, nebArray)
 		{
-			if (QString("M%1").arg((*iter)->M_nb) == objw || QString("M %1").arg((*iter)->M_nb) == objw)
-				return *iter;
+			if (QString("M%1").arg(n->M_nb) == objw || QString("M %1").arg(n->M_nb) == objw)
+				return qSharedPointerCast<StelObject>(n);
 		}
 	}
-	
-	return NULL;
+
+	return StelObjectP();
 }
 
 
@@ -532,36 +525,35 @@ StelObjectP NebulaMgr::searchByNameI18n(const QString& nameI18n) const
 StelObjectP NebulaMgr::searchByName(const QString& name) const
 {
 	QString objw = name.toUpper();
-	vector <Nebula*>::const_iterator iter;
-	
+
 	// Search by NGC numbers (possible formats are "NGC31" or "NGC 31")
 	if (objw.mid(0, 3) == "NGC")
 	{
-		for (iter = nebArray.begin(); iter != nebArray.end(); ++iter)
+		foreach (const NebulaP& n, nebArray)
 		{
-			if (QString("NGC%1").arg((*iter)->NGC_nb) == objw || QString("NGC %1").arg((*iter)->NGC_nb) == objw)
-				return *iter;
+			if (QString("NGC%1").arg(n->NGC_nb) == objw || QString("NGC %1").arg(n->NGC_nb) == objw)
+				return qSharedPointerCast<StelObject>(n);
 		}
 	}
-	
+
 	// Search by common names
-	for (iter = nebArray.begin(); iter != nebArray.end(); ++iter)
+	foreach (const NebulaP& n, nebArray)
 	{
-		QString objwcap = (*iter)->englishName.toUpper();
-		if (objwcap==objw) 
-			return *iter;
+		QString objwcap = n->englishName.toUpper();
+		if (objwcap==objw)
+			return qSharedPointerCast<StelObject>(n);
 	}
-	
+
 	// Search by Messier numbers (possible formats are "M31" or "M 31")
 	if (objw.mid(0, 1) == "M")
 	{
-		for (iter = nebArray.begin(); iter != nebArray.end(); ++iter)
+		foreach (const NebulaP& n, nebArray)
 		{
-			if (QString("M%1").arg((*iter)->M_nb) == objw || QString("M %1").arg((*iter)->M_nb) == objw)
-				return *iter;
+			if (QString("M%1").arg(n->M_nb) == objw || QString("M %1").arg(n->M_nb) == objw)
+				return qSharedPointerCast<StelObject>(n);
 		}
 	}
-	
+
 	return NULL;
 }
 
@@ -571,65 +563,57 @@ QStringList NebulaMgr::listMatchingObjectsI18n(const QString& objPrefix, int max
 {
 	QStringList result;
 	if (maxNbItem==0) return result;
-		
+
 	QString objw = objPrefix.toUpper();
-	
-	vector <Nebula*>::const_iterator iter;
-	
+
 	// Search by messier objects number (possible formats are "M31" or "M 31")
 	if (objw.size()>=1 && objw[0]=='M')
 	{
-		for (iter=nebArray.begin(); iter!=nebArray.end(); ++iter)
+		foreach (const NebulaP& n, nebArray)
 		{
-			if ((*iter)->M_nb==0) continue;
-			QString constw = QString("M%1").arg((*iter)->M_nb);
+			if (n->M_nb==0) continue;
+			QString constw = QString("M%1").arg(n->M_nb);
 			QString constws = constw.mid(0, objw.size());
 			if (constws==objw)
 			{
 				result << constw;
 				continue;	// Prevent adding both forms for name
 			}
-			constw = QString("M %1").arg((*iter)->M_nb);
+			constw = QString("M %1").arg(n->M_nb);
 			constws = constw.mid(0, objw.size());
 			if (constws==objw)
-			{
 				result << constw;
-			}
 		}
 	}
-	
+
 	// Search by NGC numbers (possible formats are "NGC31" or "NGC 31")
-	for (iter=nebArray.begin(); iter!=nebArray.end(); ++iter)
+	foreach (const NebulaP& n, nebArray)
 	{
-		if ((*iter)->NGC_nb==0) continue;
-		QString constw = QString("NGC%1").arg((*iter)->NGC_nb);
+		if (n->NGC_nb==0) continue;
+		QString constw = QString("NGC%1").arg(n->NGC_nb);
 		QString constws = constw.mid(0, objw.size());
 		if (constws==objw)
 		{
 			result << constw;
 			continue;
 		}
-		constw = QString("NGC %1").arg((*iter)->NGC_nb);
+		constw = QString("NGC %1").arg(n->NGC_nb);
 		constws = constw.mid(0, objw.size());
 		if (constws==objw)
-		{
 			result << constw;
-		}
 	}
-	
+
 	// Search by common names
-	for (iter=nebArray.begin(); iter!=nebArray.end(); ++iter)
+	foreach (const NebulaP& n, nebArray)
 	{
-		QString constw = (*iter)->nameI18.mid(0, objw.size()).toUpper();
+		QString constw = n->nameI18.mid(0, objw.size()).toUpper();
 		if (constw==objw)
-		{
-			result << (*iter)->nameI18;
-		}
+			result << n->nameI18;
 	}
-	
+
 	result.sort();
 	if (result.size()>maxNbItem) result.erase(result.begin()+maxNbItem, result.end());
-	
+
 	return result;
 }
 
