@@ -1,17 +1,17 @@
 /*
  * Stellarium
  * Copyright (C) 2002 Fabien Chereau
- * 
+ *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
  * as published by the Free Software Foundation; either version 2
  * of the License, or (at your option) any later version.
- * 
+ *
  * This program is distributed in the hope that it will be useful,
  * but WITHOUT ANY WARRANTY; without even the implied warranty of
  * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  * GNU General Public License for more details.
- * 
+ *
  * You should have received a copy of the GNU General Public License
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
@@ -22,6 +22,7 @@
 #include <QTextStream>
 #include <QString>
 #include <QDebug>
+#include <QVarLengthArray>
 
 #include "StelApp.hpp"
 #include "StelCore.hpp"
@@ -31,15 +32,14 @@
 #include "Planet.hpp"
 #include "StelNavigator.hpp"
 #include "StelProjector.hpp"
-#include "StelFont.hpp"
 #include "sideral_time.h"
 #include "StelTextureMgr.hpp"
 #include "StelModuleMgr.hpp"
 #include "StarMgr.hpp"
 #include "StelMovementMgr.hpp"
 #include "StelPainter.hpp"
+#include "StelTranslator.hpp"
 
-StelFont* Planet::planetNameFont = NULL;
 Vec3f Planet::labelColor = Vec3f(0.4,0.4,0.8);
 Vec3f Planet::orbitColor = Vec3f(1,0.6,1);
 Vec3f Planet::trailColor = Vec3f(1,0.7,0.7);
@@ -47,28 +47,30 @@ StelTextureSP Planet::hintCircleTex;
 StelTextureSP Planet::texEarthShadow;
 
 Planet::Planet(const QString& englishName,
-               int flagLighting,
-               double radius,
-               double oblateness,
-               Vec3f color,
-               float albedo,
-               const QString& atexMapName,
-               const QString& texHaloName,
-               posFuncType coordFunc,
-               OsulatingFunctType *osculatingFunc,
-               bool acloseOrbit,
-               bool hidden,
-	           bool hasAtmosphere) 
+			   int flagLighting,
+			   double radius,
+			   double oblateness,
+			   Vec3f color,
+			   float albedo,
+			   const QString& atexMapName,
+			   const QString& texHaloName,
+			   posFuncType coordFunc,
+			   void* auserDataPtr,
+			   OsulatingFunctType *osculatingFunc,
+			   bool acloseOrbit,
+			   bool hidden,
+			   bool hasAtmosphere)
 	: englishName(englishName),
 	  flagLighting(flagLighting),
 	  radius(radius), oneMinusOblateness(1.0-oblateness),
 	  color(color), albedo(albedo), axisRotation(0.), rings(NULL),
 	  sphereScale(1.f),
-	  lastJD(J2000), 
-	  coordFunc(coordFunc), 
-	  osculatingFunc(osculatingFunc), 
-	  parent(NULL), 
-	  hidden(hidden), 
+	  lastJD(J2000),
+	  coordFunc(coordFunc),
+	  userDataPtr(auserDataPtr),
+	  osculatingFunc(osculatingFunc),
+	  parent(NULL),
+	  hidden(hidden),
 	  atmosphere(hasAtmosphere)
 {
 	texMapName = atexMapName;
@@ -79,9 +81,7 @@ Planet::Planet(const QString& englishName,
 
 	eclipticPos=Vec3d(0.,0.,0.);
 	rotLocalToParent = Mat4d::identity();
-	StelApp::getInstance().getTextureManager().setDefaultParams();
-	StelApp::getInstance().getTextureManager().setWrapMode(GL_REPEAT);
-	texMap = StelApp::getInstance().getTextureManager().createTexture(texMapName);
+	texMap = StelApp::getInstance().getTextureManager().createTexture(texMapName, StelTexture::StelTextureParams(true, GL_LINEAR, GL_REPEAT));
 
 	// 60 day trails
 	DeltaTrail = 1;
@@ -93,7 +93,7 @@ Planet::Planet(const QString& englishName,
 	firstPoint = 1;
 
 	nameI18 = englishName;
-	if (englishName!="Pluto") 
+	if (englishName!="Pluto")
 	{
 		deltaJD = 0.001*JD_SECOND;
 	}
@@ -106,23 +106,18 @@ Planet::~Planet()
 		delete rings;
 }
 
-void Planet::setParent(Planet* newParent)
-{
-	Q_ASSERT(newParent);
-	newParent->satellites.push_back(this);
-	parent = newParent;
-}
+void Planet::translateName(StelTranslator& trans) {nameI18 = trans.qtranslate(englishName);}
 
 // Return the information string "ready to print" :)
 QString Planet::getInfoString(const StelCore* core, const InfoStringGroup& flags) const
 {
 	const StelNavigator* nav = core->getNavigator();
-	
+
 	QString str;
 	QTextStream oss(&str);
 
 	if (flags&Name)
-	{	
+	{
 		oss << "<h2>" << q_(englishName);  // UI translation can differ from sky translation
 		oss.setRealNumberNotation(QTextStream::FixedNotation);
 		oss.setRealNumberPrecision(1);
@@ -136,7 +131,7 @@ QString Planet::getInfoString(const StelCore* core, const InfoStringGroup& flags
 
 	if (flags&AbsoluteMagnitude)
 		oss << q_("Absolute Magnitude: %1").arg(getVMagnitude(nav)-5.*(std::log10(getJ2000EquatorialPos(nav).length()*AU/PARSEC)-1.), 0, 'f', 2) << "<br>";
-	
+
 	oss << getPositionInfoString(core, flags);
 
 	if (flags&Distance)
@@ -202,7 +197,7 @@ double Planet::getSatellitesFov(const StelNavigator * nav) const
 	return -1.;
 }
 
-double Planet::getParentSatellitesFov(const StelNavigator *nav) const 
+double Planet::getParentSatellitesFov(const StelNavigator *nav) const
 {
 	if (parent && parent->parent) return parent->getSatellitesFov(nav);
 	return -1.0;
@@ -222,18 +217,18 @@ void Planet::setRotationElements(float _period, float _offset, double _epoch, fl
 	deltaOrbitJD = re.siderealPeriod/ORBIT_SEGMENTS;
 }
 
-Vec3d Planet::getJ2000EquatorialPos(const StelNavigator *nav) const 
+Vec3d Planet::getJ2000EquatorialPos(const StelNavigator *nav) const
 {
 	return StelNavigator::matVsop87ToJ2000.multiplyWithoutTranslation(getHeliocentricEclipticPos() - nav->getObserverHeliocentricEclipticPos());
 }
 
 // Compute the position in the parent Planet coordinate system
 // Actually call the provided function to compute the ecliptical position
-void Planet::computePositionWithoutOrbits(const double date) 
+void Planet::computePositionWithoutOrbits(const double date)
 {
-	if (fabs(lastJD-date)>deltaJD) 
+	if (fabs(lastJD-date)>deltaJD)
 	{
-		coordFunc(date, eclipticPos);
+		coordFunc(date, eclipticPos, userDataPtr);
 		lastJD = date;
 	}
 }
@@ -241,7 +236,7 @@ void Planet::computePositionWithoutOrbits(const double date)
 void Planet::computePosition(const double date)
 {
 
-	if (deltaOrbitJD > 0 && (fabs(lastOrbitJD-date)>deltaOrbitJD || !orbitCached))
+	if (orbitFader.getInterstate()>0.000001 && deltaOrbitJD > 0 && (fabs(lastOrbitJD-date)>deltaOrbitJD || !orbitCached))
 	{
 
 		// calculate orbit first (for line drawing)
@@ -274,13 +269,13 @@ void Planet::computePosition(const double date)
 
 					// date increments between points will not be completely constant though
 					computeTransMatrix(calc_date);
-					if (osculatingFunc) 
+					if (osculatingFunc)
 					{
 						(*osculatingFunc)(date,calc_date,eclipticPos);
-					} 
-					else 
+					}
+					else
 					{
-						coordFunc(calc_date, eclipticPos);
+						coordFunc(calc_date, eclipticPos, userDataPtr);
 					}
 					orbit[d] = getHeliocentricEclipticPos();
 				}
@@ -305,10 +300,10 @@ void Planet::computePosition(const double date)
 					computeTransMatrix(calc_date);
 					if (osculatingFunc) {
 						(*osculatingFunc)(date,calc_date,eclipticPos);
-					} 
-					else 
+					}
+					else
 					{
-						coordFunc(calc_date, eclipticPos);
+						coordFunc(calc_date, eclipticPos, userDataPtr);
 					}
 					orbit[d] = getHeliocentricEclipticPos();
 				}
@@ -329,13 +324,13 @@ void Planet::computePosition(const double date)
 			{
 				calc_date = date + (d-ORBIT_SEGMENTS/2)*date_increment;
 				computeTransMatrix(calc_date);
-				if (osculatingFunc) 
+				if (osculatingFunc)
 				{
 					(*osculatingFunc)(date,calc_date,eclipticPos);
-				} 
-				else 
+				}
+				else
 				{
-					coordFunc(calc_date, eclipticPos);
+					coordFunc(calc_date, eclipticPos, userDataPtr);
 				}
 				orbit[d] = getHeliocentricEclipticPos();
 			}
@@ -346,7 +341,7 @@ void Planet::computePosition(const double date)
 
 
 		// calculate actual Planet position
-		coordFunc(date, eclipticPos);
+		coordFunc(date, eclipticPos, userDataPtr);
 
 		lastJD = date;
 
@@ -354,7 +349,7 @@ void Planet::computePosition(const double date)
 	else if (fabs(lastJD-date)>deltaJD)
 	{
 		// calculate actual Planet position
-		coordFunc(date, eclipticPos);
+		coordFunc(date, eclipticPos, userDataPtr);
 		lastJD = date;
 	}
 
@@ -367,29 +362,29 @@ void Planet::computeTransMatrix(double jd)
 
 	// Special case - heliocentric coordinates are on ecliptic,
 	// not solar equator...
-	if (parent) 
+	if (parent)
 	{
 		rotLocalToParent = Mat4d::zrotation(re.ascendingNode - re.precessionRate*(jd-re.epoch)) * Mat4d::xrotation(re.obliquity);
 	}
 }
 
-Mat4d Planet::getRotEquatorialToVsop87(void) const 
+Mat4d Planet::getRotEquatorialToVsop87(void) const
 {
 	Mat4d rval = rotLocalToParent;
 	if (parent)
 	{
-		for (const Planet *p=parent;p->parent;p=p->parent) 
+		for (PlanetP p=parent;p->parent;p=p->parent)
 			rval = p->rotLocalToParent * rval;
 	}
 	return rval;
 }
 
-void Planet::setRotEquatorialToVsop87(const Mat4d &m) 
+void Planet::setRotEquatorialToVsop87(const Mat4d &m)
 {
 	Mat4d a = Mat4d::identity();
 	if (parent)
 	{
-		for (const Planet *p=parent;p->parent;p=p->parent) 
+		for (PlanetP p=parent;p->parent;p=p->parent)
 			a = p->rotLocalToParent * a;
 	}
 	rotLocalToParent = a.transpose() * m;
@@ -403,7 +398,7 @@ double Planet::getSiderealTime(double jd) const
 	{
 		return get_apparent_sidereal_time(jd);
 	}
- 
+
 	double t = jd - re.epoch;
 	double rotations = t / (double) re.period;
 	double wholeRotations = floor(rotations);
@@ -422,7 +417,7 @@ Vec3d Planet::getEclipticPos() const
 Vec3d Planet::getHeliocentricEclipticPos() const
 {
 	Vec3d pos = eclipticPos;
-	const Planet *pp = parent;
+	PlanetP pp = parent;
 	if (pp)
 	{
 		while (pp->parent)
@@ -437,11 +432,14 @@ Vec3d Planet::getHeliocentricEclipticPos() const
 void Planet::setHeliocentricEclipticPos(const Vec3d &pos)
 {
 	eclipticPos = pos;
-	const Planet *p = parent;
-	if (p) while (p->parent)
+	PlanetP p = parent;
+	if (p)
 	{
-		eclipticPos -= p->eclipticPos;
-		p = p->parent;
+		while (p->parent)
+		{
+			eclipticPos -= p->eclipticPos;
+			p = p->parent;
+		}
 	}
 }
 
@@ -453,70 +451,106 @@ double Planet::computeDistance(const Vec3d& obsHelioPos)
 }
 
 // Get the phase angle for an observer at pos obsPos in the heliocentric coordinate (dist in AU)
-double Planet::getPhase(Vec3d obsPos) const 
+double Planet::getPhase(const Vec3d& obsPos) const
 {
-	const double sq = obsPos.lengthSquared();
-	const Vec3d heliopos = getHeliocentricEclipticPos();
-	const double Rq = heliopos.lengthSquared();
-	const double pq = (obsPos - heliopos).lengthSquared();
-	const double cos_chi = (pq + Rq - sq)/(2.0*sqrt(pq*Rq));
-	return (1.0 - acos(cos_chi)/M_PI) * cos_chi + sqrt(1.0 - cos_chi*cos_chi) / M_PI;
+	const double observerRq = obsPos.lengthSquared();
+	const Vec3d& planetHelioPos = getHeliocentricEclipticPos();
+	const double planetRq = planetHelioPos.lengthSquared();
+	const double observerPlanetRq = (obsPos - planetHelioPos).lengthSquared();
+	return std::acos(observerPlanetRq + planetRq - observerRq)/(2.0*sqrt(observerPlanetRq*planetRq));
 }
 
-float Planet::getVMagnitude(const StelNavigator * nav) const 
+// Computation of the visual magnitude (V band) of the planet.
+float Planet::getVMagnitude(const StelNavigator * nav) const
 {
-	Vec3d obsPos = nav->getObserverHeliocentricEclipticPos();
-	const double sq = obsPos.lengthSquared();
 	if (parent == 0)
 	{
 		// sun, compute the apparent magnitude for the absolute mag (4.83) and observer's distance
-		double distParsec = std::sqrt(sq)*AU/PARSEC;
+		const double distParsec = std::sqrt(nav->getObserverHeliocentricEclipticPos().lengthSquared())*AU/PARSEC;
 		return 4.83 + 5.*(std::log10(distParsec)-1.);
 	}
-	const Vec3d heliopos = getHeliocentricEclipticPos();
-	const double Rq = heliopos.lengthSquared();
-	const double pq = (obsPos - heliopos).lengthSquared();
-	const double cos_chi = (pq + Rq - sq)/(2.0*sqrt(pq*Rq));
-	const double phase = (1.0 - std::acos(cos_chi)/M_PI) * cos_chi + std::sqrt(1.0 - cos_chi*cos_chi) / M_PI;
-	double F = 2.0 * albedo * radius * radius * phase / (3.0*pq*Rq);
 
+	// Compute the angular phase
+	const Vec3d& observerHelioPos = nav->getObserverHeliocentricEclipticPos();
+	const double observerRq = observerHelioPos.lengthSquared();
+	const Vec3d& planetHelioPos = getHeliocentricEclipticPos();
+	const double planetRq = planetHelioPos.lengthSquared();
+	const double observerPlanetRq = (observerHelioPos - planetHelioPos).lengthSquared();
+	const double cos_chi = (observerPlanetRq + planetRq - observerRq)/(2.0*sqrt(observerPlanetRq*planetRq));
+	double phase = std::acos(cos_chi);
+
+	double shadowFactor = 1.;
 	// Check if the satellite is inside the inner shadow of the parent planet:
-	if (parent->parent != 0) {
-		const Vec3d parent_heliopos = parent->getHeliocentricEclipticPos();
-		const double parent_Rq = parent_heliopos.lengthSquared();
-		const double pos_times_parent_pos = heliopos * parent_heliopos;
-		if (pos_times_parent_pos > parent_Rq) {
+	if (parent->parent != 0)
+	{
+		const Vec3d& parentHeliopos = parent->getHeliocentricEclipticPos();
+		const double parent_Rq = parentHeliopos.lengthSquared();
+		const double pos_times_parent_pos = planetHelioPos * parentHeliopos;
+		if (pos_times_parent_pos > parent_Rq)
+		{
 			// The satellite is farther away from the sun than the parent planet.
 			const double sun_radius = parent->parent->radius;
 			const double sun_minus_parent_radius = sun_radius - parent->radius;
 			const double quot = pos_times_parent_pos/parent_Rq;
-			// compute d = distance from satellite center to border
-			// of inner shadow. d>0 means inside the shadow cone.
-			double d = sun_radius - sun_minus_parent_radius*quot
-			- std::sqrt( (1.0-sun_minus_parent_radius/sqrt(parent_Rq))
-			* (Rq-pos_times_parent_pos*quot) );
-			if (d >= radius) 
+
+			// Compute d = distance from satellite center to border of inner shadow.
+			// d>0 means inside the shadow cone.
+			double d = sun_radius - sun_minus_parent_radius*quot - std::sqrt((1.-sun_minus_parent_radius/sqrt(parent_Rq)) * (planetRq-pos_times_parent_pos*quot));
+			if (d>=radius)
 			{
 				// The satellite is totally inside the inner shadow.
-				F *= 1e-9;
-			} 
-			else if (d > -radius) 
+				shadowFactor = 1e-9;
+			}
+			else if (d>-radius)
 			{
 				// The satellite is partly inside the inner shadow,
 				// compute a fantasy value for the magnitude:
 				d /= radius;
-				F *= (0.5 - (std::asin(d)+d*std::sqrt(1.0-d*d))/M_PI);
+				shadowFactor = (0.5 - (std::asin(d)+d*std::sqrt(1.0-d*d))/M_PI);
 			}
 		}
 	}
 
-	const double rval = -26.73 - 2.5 * std::log10(F);
-	//qDebug() << "Planet(" << getEnglishName()
-	//         << ")::getVMagnitude(" << obsPos << "): "
-	//         << "phase: " << phase
-	//         << ",F: " << F
-	//         << ",rval: " << rval;
-	return rval;
+	// Use empirical formulae for main planets when seen from earth
+	// Algorithm provided by Pere Planesas (Observatorio Astronomico Nacional)
+	if (nav->getCurrentLocation().planetName=="Earth")
+	{
+		phase*=180./M_PI;
+		const double d = 5. * log10(sqrt(observerPlanetRq*planetRq));
+		double f1 = phase/100.;
+
+		if (englishName=="Mercury")
+		{
+			if ( phase > 150. ) f1 = 1.5;
+			return -0.36 + d + 3.8*f1 - 2.73*f1*f1 + 2*f1*f1*f1;
+		}
+		if (englishName=="Venus")
+			return -4.29 + d + 0.09*f1 + 2.39*f1*f1 - 0.65*f1*f1*f1;
+		if (englishName=="Mars")
+			return -1.52 + d + 0.016*phase;
+		if (englishName=="Jupiter")
+			return -9.25 + d + 0.005*phase;
+		if (englishName=="Saturn")
+		{
+			// TODO re-add rings computation
+			// double rings = -2.6*sinx + 1.25*sinx*sinx;
+			return -8.88 + d + 0.044*phase;// + rings;
+		}
+
+		if (englishName=="Uranus")
+			return -7.19 + d + 0.0028*phase;
+		if (englishName=="Neptune")
+			return -6.87 + d;
+		if (englishName=="Pluto")
+			return -1.01 + d + 0.041*phase;
+
+		phase/=180./M_PI;
+	}
+
+	// This formula seems to give wrong results
+	const double p = (1.0 - phase/M_PI) * cos_chi + std::sqrt(1.0 - cos_chi*cos_chi) / M_PI;
+	double F = 2.0 * albedo * radius * radius * p / (3.0*observerPlanetRq*planetRq) * shadowFactor;
+	return -26.73 - 2.5 * std::log10(F);
 }
 
 double Planet::getAngularSize(const StelCore* core) const
@@ -534,15 +568,15 @@ double Planet::getSpheroidAngularSize(const StelCore* core) const
 }
 
 // Draw the Planet and all the related infos : name, circle etc..
-void Planet::draw(StelCore* core, float maxMagLabels)
+void Planet::draw(StelCore* core, float maxMagLabels, const QFont& planetNameFont)
 {
 	if (hidden)
 		return;
 
 	StelNavigator* nav = core->getNavigator();
-		
+
 	Mat4d mat = Mat4d::translation(eclipticPos) * rotLocalToParent;
-	const Planet *p = parent;
+	PlanetP p = parent;
 	while (p && p->parent)
 	{
 		mat = Mat4d::translation(p->eclipticPos) * mat * p->rotLocalToParent;
@@ -552,8 +586,9 @@ void Planet::draw(StelCore* core, float maxMagLabels)
 	// This removed totally the Planet shaking bug!!!
 	mat = nav->getHeliocentricEclipticModelViewMat() * mat;
 
-	if (this == nav->getHomePlanet())
+	if (getEnglishName() == nav->getCurrentLocation().planetName)
 	{
+		// Draw the rings if we are located on a planet with rings, but not the planet itself.
 		if (rings)
 		{
 			StelPainter sPainter(core->getProjection(mat));
@@ -568,8 +603,8 @@ void Planet::draw(StelCore* core, float maxMagLabels)
 	float viewport_left = prj->getViewportPosX();
 	float viewport_bottom = prj->getViewportPosY();
 	if (prj->project(Vec3f(0,0,0), screenPos) &&
-	        screenPos[1]>viewport_bottom - screenSz && screenPos[1]<viewport_bottom + prj->getViewportHeight()+screenSz &&
-	        screenPos[0]>viewport_left - screenSz && screenPos[0]<viewport_left + prj->getViewportWidth() + screenSz)
+			screenPos[1]>viewport_bottom - screenSz && screenPos[1]<viewport_bottom + prj->getViewportHeight()+screenSz &&
+			screenPos[0]>viewport_left - screenSz && screenPos[0]<viewport_left + prj->getViewportWidth() + screenSz)
 	{
 		// Draw the name, and the circle if it's not too close from the body it's turning around
 		// this prevents name overlaping (ie for jupiter satellites)
@@ -589,7 +624,7 @@ void Planet::draw(StelCore* core, float maxMagLabels)
 		{
 			labelsFader=false;
 		}
-		drawHints(core);
+		drawHints(core, planetNameFont);
 
 		draw3dModel(core,mat,screenSz);
 	}
@@ -600,41 +635,40 @@ void Planet::draw3dModel(StelCore* core, const Mat4d& mat, float screenSz)
 {
 	// This is the main method drawing a planet 3d model
 	// Some work has to be done on this method to make the rendering nicer
-	
+
 	StelNavigator* nav = core->getNavigator();
-	
+
 	if (screenSz>1.)
 	{
 		StelPainter* sPainter = new StelPainter(core->getProjection(mat * Mat4d::zrotation(M_PI/180*(axisRotation + 90.))));
-	
+
 		if (flagLighting)
 		{
-			glEnable(GL_LIGHTING);
-			
+			sPainter->getLight().enable();
+
 			// Set the main source of light to be the sun
-			const Vec3d sunPos = core->getNavigator()->getHeliocentricEclipticModelViewMat()*Vec3d(0,0,0);
-			glLightfv(GL_LIGHT0,GL_POSITION,Vec4f(sunPos[0],sunPos[1],sunPos[2],1.f));
+			const Vec3d& sunPos = core->getNavigator()->getHeliocentricEclipticModelViewMat()*Vec3d(0,0,0);
+			sPainter->getLight().setPosition(Vec4f(sunPos[0],sunPos[1],sunPos[2],1.f));
 
-			const float diffuse[4] = {2.,2.,2.,1};
-			glLightfv(GL_LIGHT0,GL_DIFFUSE, diffuse);
-			
 			// Set the light parameters taking sun as the light source
-			const float zero[4] = {0,0,0,0};
-			const float ambient[4] = {0.02,0.02,0.02,0.02};
-			glLightfv(GL_LIGHT0,GL_AMBIENT, ambient);
-			glLightfv(GL_LIGHT0,GL_DIFFUSE, diffuse);
-			glLightfv(GL_LIGHT0,GL_SPECULAR,zero);
-			glMaterialfv(GL_FRONT,GL_AMBIENT,  ambient);
-			glMaterialfv(GL_FRONT,GL_EMISSION, zero);
-			glMaterialfv(GL_FRONT,GL_SHININESS,zero);
-			glMaterialfv(GL_FRONT,GL_SPECULAR, zero);
+			static const Vec4f diffuse = Vec4f(2.,2.,2.,1);
+			static const Vec4f zero = Vec4f(0,0,0,0);
+			static const Vec4f ambient = Vec4f(0.02,0.02,0.02,0.02);
+			sPainter->getLight().setAmbient(ambient);
+			sPainter->getLight().setDiffuse(diffuse);
+			sPainter->getLight().setSpecular(zero);
+
+			sPainter->getMaterial().setAmbient(ambient);
+			sPainter->getMaterial().setEmission(zero);
+			sPainter->getMaterial().setShininess(0);
+			sPainter->getMaterial().setSpecular(zero);
 		}
 		else
 		{
-			glDisable(GL_LIGHTING);
-			glColor3fv(Vec3f(1.f,1.f,1.f));
+			sPainter->getLight().disable();
+			sPainter->setColor(1.f,1.f,1.f);
 		}
-		
+
 		if (rings)
 		{
 			const double dist = getEquinoxEquatorialPos(nav).length();
@@ -644,19 +678,21 @@ void Planet::draw3dModel(StelCore* core, const Mat4d& mat, float screenSz)
 			double n,f;
 			core->getClippingPlanes(&n,&f); // Save clipping planes
 			core->setClippingPlanes(z_near,z_far);
+			glDepthMask(GL_TRUE);
 			glClear(GL_DEPTH_BUFFER_BIT);
 			glEnable(GL_DEPTH_TEST);
 			drawSphere(sPainter, screenSz);
-			glDisable(GL_LIGHTING);
+			glDepthMask(GL_FALSE);
+			sPainter->getLight().disable();
 			rings->draw(sPainter,mat,screenSz);
-			glEnable(GL_LIGHTING);
+			sPainter->getLight().enable();
 			glDisable(GL_DEPTH_TEST);
 			core->setClippingPlanes(n,f);  // Restore old clipping planes
 		}
 		else
 		{
 			SolarSystem* ssm = GETSTELMODULE(SolarSystem);
-			if (this==ssm->getMoon() && nav->getHomePlanet()==ssm->getEarth() && ssm->nearLunarEclipse())
+			if (this==ssm->getMoon() && nav->getCurrentLocation().planetName=="Earth" && ssm->nearLunarEclipse())
 			{
 				// Draw earth shadow over moon using stencil buffer if appropriate
 				// This effect curently only looks right from earth viewpoint
@@ -668,11 +704,9 @@ void Planet::draw3dModel(StelCore* core, const Mat4d& mat, float screenSz)
 				glEnable(GL_STENCIL_TEST);
 				drawSphere(sPainter, screenSz);
 				glDisable(GL_STENCIL_TEST);
-				
-				glDisable(GL_LIGHTING);
-				delete sPainter;
-				sPainter=NULL;
-				drawEarthShadow(core);
+
+				sPainter->getLight().disable();
+				drawEarthShadow(core, sPainter);
 			}
 			else
 			{
@@ -684,24 +718,24 @@ void Planet::draw3dModel(StelCore* core, const Mat4d& mat, float screenSz)
 			delete sPainter;
 		sPainter=NULL;
 	}
-	
+
 	// Draw the halo
-	
+
 	// Prepare openGL lighting parameters according to luminance
 	float surfArcMin2 = getSpheroidAngularSize(core)*60;
 	surfArcMin2 = surfArcMin2*surfArcMin2*M_PI; // the total illuminated area in arcmin^2
-	
+
 	StelPainter sPainter(core->getProjection(StelCore::FrameJ2000));
-	core->getSkyDrawer()->postDrawSky3dModel(screenPos[0],screenPos[1], surfArcMin2, getVMagnitude(core->getNavigator()), &sPainter, color);
+	core->getSkyDrawer()->postDrawSky3dModel(&sPainter, getJ2000EquatorialPos(nav), surfArcMin2, getVMagnitude(core->getNavigator()), color);
 }
 
 
-void Planet::drawSphere(const StelPainter* painter, float screenSz)
+void Planet::drawSphere(StelPainter* painter, float screenSz)
 {
-	glEnable(GL_TEXTURE_2D);
+	painter->enableTexture2d(true);
 	glDisable(GL_BLEND);
 	glEnable(GL_CULL_FACE);
-	
+
 	if (texMap)
 		texMap->bind();
 
@@ -710,19 +744,19 @@ void Planet::drawSphere(const StelPainter* painter, float screenSz)
 	int nb_facet = (int)(screenSz * 40/50);	// 40 facets for 1024 pixels diameter on screen
 	if (nb_facet<10) nb_facet = 10;
 	if (nb_facet>40) nb_facet = 40;
-	glShadeModel(GL_SMOOTH);
+	painter->setShadeModel(StelPainter::ShadeModelSmooth);
 	// Rotate and add an extra quarter rotation so that the planet texture map
 	// fits to the observers position. No idea why this is necessary,
 	// perhaps some openGl strangeness, or confusing sin/cos.
 	painter->sSphere(radius*sphereScale, oneMinusOblateness, nb_facet, nb_facet);
-	glShadeModel(GL_FLAT);
+	painter->setShadeModel(StelPainter::ShadeModelFlat);
 	glDisable(GL_CULL_FACE);
 }
 
 
 // draws earth shadow overlapping the moon using stencil buffer
 // umbra and penumbra are sized separately for accuracy
-void Planet::drawEarthShadow(StelCore* core)
+void Planet::drawEarthShadow(StelCore* core, StelPainter* sPainter)
 {
 	SolarSystem* ssm = GETSTELMODULE(SolarSystem);
 	Vec3d e = ssm->getEarth()->getEclipticPos();
@@ -748,71 +782,72 @@ void Planet::drawEarthShadow(StelCore* core)
 
 	// modify shadow location for scaled moon
 	Vec3d mdist = shadow - mh;
-	if(mdist.length() > r_penumbra + 2000./AU)
+	if (mdist.length() > r_penumbra + 2000./AU)
 		return;   // not visible so don't bother drawing
 
 	shadow = mh + mdist*mscale;
 	r_penumbra *= mscale;
 
-	const StelProjectorP prj = core->getProjection(StelCore::FrameHeliocentricEcliptic);
-	StelPainter sPainter(prj);
-	
-	glEnable(GL_TEXTURE_2D);
+	StelProjectorP saveProj = sPainter->getProjector();
+	sPainter->setProjector(core->getProjection(StelCore::FrameHeliocentricEcliptic));
+
+	sPainter->enableTexture2d(true);
 	glEnable(GL_BLEND);
 	glBlendFunc(GL_SRC_ALPHA, GL_ONE_MINUS_SRC_ALPHA);
-	glColor3f(1,1,1);
-	
+	sPainter->setColor(1,1,1);
+
 	glEnable(GL_STENCIL_TEST);
+	// We draw only where the stencil buffer is at 1, i.e. where the moon was drawn
 	glStencilFunc(GL_EQUAL, 0x1, 0x1);
+	// Don't change stencil buffer value
 	glStencilOp(GL_KEEP, GL_KEEP, GL_KEEP);
 
 	// shadow radial texture
 	texEarthShadow->bind();
 
-	Vec3d r, s;
+	Vec3d r;
 
-	// umbra first
-	glBegin(GL_TRIANGLE_FAN);
-      // johannes: work-around for nasty ATI rendering bug:
-      // use y-texture coordinate of 0.5 instead of 0.0
-	glTexCoord2f(0.f,0.5f);
-	sPainter.drawVertex3v(shadow);
+	// Draw umbra first
+	QVarLengthArray<Vec2f, 210> texCoordArray(210);
+	QVarLengthArray<Vec3d, 210> vertexArray(210);
+	texCoordArray[0].set(0.f, 0.5);
+	// johannes: work-around for nasty ATI rendering bug: use y-texture coordinate of 0.5 instead of 0.0
+	vertexArray[0]=shadow;
 
-	for (int i=0; i<=100; i++)
+	const Mat4d& rotMat = Mat4d::rotation(shadow, 2.*M_PI/100.);
+	r = upt;
+	for (int i=1; i<=101; ++i)
 	{
-		r = Mat4d::rotation(shadow, 2.*M_PI*i/100.) * upt;
-		s = shadow + r;
-
-		glTexCoord2f(0.6f,0.5f);  // position in texture of umbra edge
-		sPainter.drawVertex3v(s);
+		// position in texture of umbra edge
+		texCoordArray[i].set(0.6, 0.5);
+		r.transfo4d(rotMat);
+		vertexArray[i] = shadow + r;
 	}
-	glEnd();
-
+	sPainter->setArrays(vertexArray.constData(), texCoordArray.constData());
+	sPainter->drawFromArray(StelPainter::TriangleFan, 102);
 
 	// now penumbra
-	Vec3d u, sp;
-	glBegin(GL_TRIANGLE_STRIP);
-	for (int i=0; i<=100; i++)
+	Vec3d u;
+	r = rpt;
+	u = upt;
+	for (int i=0; i<=200; i+=2)
 	{
-		r = Mat4d::rotation(shadow, 2.*M_PI*i/100.) * rpt;
-		u = Mat4d::rotation(shadow, 2.*M_PI*i/100.) * upt;
-		s = shadow + r;
-		sp = shadow + u;
-
-		glTexCoord2f(0.6f,0.5f);
-		sPainter.drawVertex3v(sp);
-
-		glTexCoord2f(1.f,0.5f);  // position in texture of umbra edge
-		sPainter.drawVertex3v(s);
+		r.transfo4d(rotMat);
+		u.transfo4d(rotMat);
+		texCoordArray[i].set(0.6, 0.5);
+		texCoordArray[i+1].set(1., 0.5); // position in texture of umbra edge
+		vertexArray[i] = shadow + u;
+		vertexArray[i+1] = shadow + r;
 	}
-	glEnd();
-
+	sPainter->setArrays(vertexArray.constData(), texCoordArray.constData());
+	sPainter->drawFromArray(StelPainter::TriangleStrip, 202);
 	glDisable(GL_STENCIL_TEST);
 	glClearStencil(0x0);
 	glClear(GL_STENCIL_BUFFER_BIT);	// Clean again to let a clean buffer for later Qt display
+	sPainter->setProjector(saveProj);
 }
 
-void Planet::drawHints(const StelCore* core)
+void Planet::drawHints(const StelCore* core, const QFont& planetNameFont)
 {
 	if (labelsFader.getInterstate()<=0.f)
 		return;
@@ -820,50 +855,50 @@ void Planet::drawHints(const StelCore* core)
 	const StelNavigator* nav = core->getNavigator();
 	const StelProjectorP prj = core->getProjection(StelCore::FrameJ2000);
 	StelPainter sPainter(prj);
-			
+	sPainter.setFont(planetNameFont);
 	// Draw nameI18 + scaling if it's not == 1.
 	float tmp = 10.f + getAngularSize(core)*M_PI/180.*prj->getPixelPerRadAtCenter()/1.44; // Shift for nameI18 printing
-	glColor4f(labelColor[0], labelColor[1], labelColor[2],labelsFader.getInterstate());
-	sPainter.drawText(planetNameFont,screenPos[0],screenPos[1], getSkyLabel(nav), 0, tmp, tmp, false);
+	sPainter.setColor(labelColor[0], labelColor[1], labelColor[2],labelsFader.getInterstate());
+	sPainter.drawText(screenPos[0],screenPos[1], getSkyLabel(nav), 0, tmp, tmp, false);
 
 	// hint disapears smoothly on close view
 	if (hintFader.getInterstate()<=0)
 		return;
 	tmp -= 10.f;
 	if (tmp<1) tmp=1;
-	glColor4f(labelColor[0], labelColor[1], labelColor[2],labelsFader.getInterstate()*hintFader.getInterstate()/tmp*0.7);
+	sPainter.setColor(labelColor[0], labelColor[1], labelColor[2],labelsFader.getInterstate()*hintFader.getInterstate()/tmp*0.7);
 
 	// Draw the 2D small circle
 	glEnable(GL_BLEND);
-	glEnable(GL_TEXTURE_2D);
+	sPainter.enableTexture2d(true);
 	glBlendFunc(GL_SRC_ALPHA, GL_ONE_MINUS_SRC_ALPHA);
 	Planet::hintCircleTex->bind();
-	sPainter.drawSprite2dMode(screenPos[0], screenPos[1], 22);
+	sPainter.drawSprite2dMode(screenPos[0], screenPos[1], 11);
 }
 
 Ring::Ring(double radiusMin,double radiusMax,const QString &texname)
-     :radiusMin(radiusMin),radiusMax(radiusMax) 
+	 :radiusMin(radiusMin),radiusMax(radiusMax)
 {
 	tex = StelApp::getInstance().getTextureManager().createTexture(texname);
 }
 
-Ring::~Ring(void) 
+Ring::~Ring(void)
 {
 }
 
-void Ring::draw(const StelPainter* painter,const Mat4d& mat,double screenSz)
-{	
+void Ring::draw(StelPainter* sPainter,const Mat4d& mat,double screenSz)
+{
 	screenSz -= 50;
 	screenSz /= 250.0;
 	if (screenSz < 0.0) screenSz = 0.0;
 	else if (screenSz > 1.0) screenSz = 1.0;
 	const int slices = 128+(int)((256-128)*screenSz);
 	const int stacks = 8+(int)((32-8)*screenSz);
-	
+
 	// Normal transparency mode
 	glBlendFunc(GL_SRC_ALPHA, GL_ONE_MINUS_SRC_ALPHA);
-	glColor3f(1.f, 1.f, 1.f);
-	glEnable(GL_TEXTURE_2D);
+	sPainter->setColor(1.f, 1.f, 1.f);
+	sPainter->enableTexture2d(true);
 	glEnable(GL_CULL_FACE);
 	glEnable(GL_BLEND);
 
@@ -872,9 +907,9 @@ void Ring::draw(const StelPainter* painter,const Mat4d& mat,double screenSz)
 	  // solve the ring wraparound by culling:
 	  // decide if we are above or below the ring plane
 	const double h = mat.r[ 8]*mat.r[12]
-	               + mat.r[ 9]*mat.r[13]
-	               + mat.r[10]*mat.r[14];
-	painter->sRing(radiusMin,radiusMax,(h<0.0)?slices:-slices,stacks, 0);
+				   + mat.r[ 9]*mat.r[13]
+				   + mat.r[10]*mat.r[14];
+	sPainter->sRing(radiusMin,radiusMax,(h<0.0)?slices:-slices,stacks, 0);
 	glDisable(GL_CULL_FACE);
 }
 
@@ -890,62 +925,43 @@ void Planet::drawOrbit(const StelCore* core)
 	const StelProjectorP prj = core->getProjection(StelCore::FrameHeliocentricEcliptic);
 
 	StelPainter sPainter(prj);
-	
+
 	// Normal transparency mode
 	glBlendFunc(GL_SRC_ALPHA, GL_ONE_MINUS_SRC_ALPHA);
 	glEnable(GL_BLEND);
 
-	glColor4f(orbitColor[0], orbitColor[1], orbitColor[2], orbitFader.getInterstate());
-
-	int on=0;
-	int d;
+	sPainter.setColor(orbitColor[0], orbitColor[1], orbitColor[2], orbitFader.getInterstate());
 	Vec3d onscreen;
-	for( int n=0; n<=ORBIT_SEGMENTS; n++)
-	{
-		if( n==ORBIT_SEGMENTS )
-		{
-			d = 0;  // connect loop
-			if (!closeOrbit) break;
-		}
-		else
-		{
-			d = n;
-		}
+	// special case - use current Planet position as center vertex so that draws
+	// on it's orbit all the time (since segmented rather than smooth curve)
+	Vec3d savePos = orbit[ORBIT_SEGMENTS/2];
+	orbit[ORBIT_SEGMENTS/2]=getHeliocentricEclipticPos();
+	orbit[ORBIT_SEGMENTS]=orbit[0];
+	int nbIter = closeOrbit ? ORBIT_SEGMENTS : ORBIT_SEGMENTS-1;
+	QVarLengthArray<float, 1024> vertexArray;
 
-		// special case - use current Planet position as center vertex so that draws
-		// on it's orbit all the time (since segmented rather than smooth curve)
-		if( n == ORBIT_SEGMENTS/2 )
+	sPainter.enableClientStates(true, false, false);
+	for (int n=0; n<=nbIter; ++n)
+	{
+		if (prj->project(orbit[n],onscreen))
 		{
-			if(prj->project(getHeliocentricEclipticPos(), onscreen))
-			{
-				if(!on) glBegin(GL_LINE_STRIP);
-				glVertex3d(onscreen[0], onscreen[1], 0);
-				on=1;
-			}
-			else if( on )
-			{
-				glEnd();
-				on=0;
-			}
+			vertexArray.append(onscreen[0]);
+			vertexArray.append(onscreen[1]);
 		}
-		else
+		else if (!vertexArray.isEmpty())
 		{
-			if(prj->project(orbit[d],onscreen))
-			{
-				if(!on) glBegin(GL_LINE_STRIP);
-				glVertex3d(onscreen[0], onscreen[1], 0);
-				on=1;
-			}
-			else if( on )
-			{
-				glEnd();
-				on=0;
-			}
+			sPainter.setVertexPointer(2, GL_FLOAT, vertexArray.constData());
+			sPainter.drawFromArray(StelPainter::LineStrip, vertexArray.size()/2, 0, false);
+			vertexArray.clear();
 		}
 	}
-
-	if (on)
-		glEnd();
+	orbit[ORBIT_SEGMENTS/2]=savePos;
+	if (!vertexArray.isEmpty())
+	{
+		sPainter.setVertexPointer(2, GL_FLOAT, vertexArray.constData());
+		sPainter.drawFromArray(StelPainter::LineStrip, vertexArray.size()/2, 0, false);
+	}
+	sPainter.enableClientStates(false);
 }
 
 
@@ -957,46 +973,36 @@ void Planet::drawTrail(const StelCore* core)
 
 	const StelNavigator* nav = core->getNavigator();
 	const StelProjectorP prj = core->getProjection(StelCore::FrameJ2000);
-	
+
 	Vec3d onscreen1;
 	Vec3d onscreen2;
 
 	StelPainter sPainter(prj);
 	glEnable(GL_BLEND);
 
-	glColor3fv(trailColor*trailFader.getInterstate());
+	sPainter.setColor(trailColor[0]*trailFader.getInterstate(), trailColor[1]*trailFader.getInterstate(), trailColor[2]*trailFader.getInterstate(), 1.);
 
 	std::list<TrailPoint>::iterator iter;
 	std::list<TrailPoint>::iterator nextiter;
 	std::list<TrailPoint>::iterator begin = trail.begin();
 	//  begin++;
 
-	if(trail.begin() != trail.end())
+	if (trail.begin() != trail.end())
 	{
 		nextiter = trail.end();
 		nextiter--;
 
-		for( iter=nextiter; iter != begin; iter--)
+		for (iter=nextiter; iter != begin; --iter)
 		{
 			nextiter--;
-			if( prj->projectLineCheck( (*iter).point, onscreen1, (*(nextiter)).point, onscreen2) )
-			{
-				glBegin(GL_LINE_STRIP);
-				glVertex2d(onscreen1[0], onscreen1[1]);
-				glVertex2d(onscreen2[0], onscreen2[1]);
-				glEnd();
-			}
+			if (prj->projectLineCheck( (*iter).point, onscreen1, (*(nextiter)).point, onscreen2))
+				sPainter.drawLine2d(onscreen1[0], onscreen1[1], onscreen2[0], onscreen2[1]);
 		}
 	}
 
 	// draw final segment to finish at current Planet position
-	if( !firstPoint && prj->projectLineCheck( (*trail.begin()).point, onscreen1, getEquinoxEquatorialPos(nav), onscreen2) )
-	{
-		glBegin(GL_LINE_STRIP);
-		glVertex2d(onscreen1[0], onscreen1[1]);
-		glVertex2d(onscreen2[0], onscreen2[1]);
-		glEnd();
-	}
+	if (!firstPoint && prj->projectLineCheck( (*trail.begin()).point, onscreen1, getEquinoxEquatorialPos(nav), onscreen2))
+		sPainter.drawLine2d(onscreen1[0], onscreen1[1], onscreen2[0], onscreen2[1]);
 }
 
 // update trail points as needed
