@@ -17,6 +17,8 @@
  * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
  */
 
+#include "config.h"
+
 #include <QDebug>
 #include <QFile>
 #ifdef WIN32
@@ -25,20 +27,28 @@
 #endif
 
 #include "ZoneArray.hpp"
-
 #include "StelApp.hpp"
 #include "StelFileMgr.hpp"
 #include "StelGeodesicGrid.hpp"
 #include "StelObject.hpp"
 #include "StelNavigator.hpp"
 
+#ifdef HAVE_BYTESWAP_H
+#include <byteswap.h>
+#else
+static unsigned int bswap_32(unsigned int val) {
+  return (((val) & 0xff000000) >> 24) | (((val) & 0x00ff0000) >>  8) |
+	(((val) & 0x0000ff00) <<  8) | (((val) & 0x000000ff) << 24);
+}
+#endif
+
 namespace BigStarCatalogExtension
 {
 
 static const Vec3d north(0,0,1);
 
 void ZoneArray::initTriangle(int index, const Vec3d &c0, const Vec3d &c1,
-                             const Vec3d &c2)
+							 const Vec3d &c2)
 {
 	// initialize center,axis0,axis1:
 	ZoneData &z(zones[index]);
@@ -72,11 +82,9 @@ void ZoneArray::initTriangle(int index, const Vec3d &c0, const Vec3d &c1,
 	if (star_position_scale < h) star_position_scale = h;
 }
 
-static inline
-int ReadInt(QFile& file, unsigned int &x)
+static inline int ReadInt(QFile& file, unsigned int &x)
 {
-	const int rval = (4 == file.read((char*)&x, 4))
-			 ? 0 : -1;
+	const int rval = (4 == file.read((char*)&x, 4)) ? 0 : -1;
 	return rval;
 }
 
@@ -84,36 +92,25 @@ int ReadInt(QFile& file, unsigned int &x)
 #warning Star catalogue loading has only been tested with gcc
 #endif
 
-ZoneArray *ZoneArray::create(const QString& extended_file_name, bool use_mmap,
-                             StelLoadingBar &lb)
+ZoneArray* ZoneArray::create(const QString& catalogFilePath, bool use_mmap, StelLoadingBar* lb)
 {
-	QString fname(extended_file_name);
 	QString dbStr; // for debugging output.
-	try
+	QFile* file = new QFile(catalogFilePath);
+	if (!file->open(QIODevice::ReadOnly))
 	{
-		fname = StelApp::getInstance().getFileMgr().findFile("stars/default/"+fname);
-	}
-	catch (std::runtime_error &e)
-	{
-		qDebug() << "Loading" << extended_file_name << e.what();
+		qWarning() << "Error while loading " << catalogFilePath << ": failed to open file.";
 		return 0;
 	}
-	QFile* file = new QFile(fname);
-	if(!file->open(QIODevice::ReadOnly))
-	{
-		qDebug() << "Loading" << extended_file_name << "failed to open file.";
-		return 0;
-	}
-	dbStr = "Loading \"" + extended_file_name + "\": ";
+	dbStr = "Loading \"" + catalogFilePath + "\": ";
 	unsigned int magic,major,minor,type,level,mag_min,mag_range,mag_steps;
 	if (ReadInt(*file,magic) < 0 ||
-	        ReadInt(*file,type) < 0 ||
-	        ReadInt(*file,major) < 0 ||
-	        ReadInt(*file,minor) < 0 ||
-	        ReadInt(*file,level) < 0 ||
-	        ReadInt(*file,mag_min) < 0 ||
-	        ReadInt(*file,mag_range) < 0 ||
-	        ReadInt(*file,mag_steps) < 0)
+			ReadInt(*file,type) < 0 ||
+			ReadInt(*file,major) < 0 ||
+			ReadInt(*file,minor) < 0 ||
+			ReadInt(*file,level) < 0 ||
+			ReadInt(*file,mag_min) < 0 ||
+			ReadInt(*file,mag_range) < 0 ||
+			ReadInt(*file,mag_steps) < 0)
 	{
 		dbStr += "error - file format is bad.";
 		qDebug() << dbStr;
@@ -130,7 +127,7 @@ ZoneArray *ZoneArray::create(const QString& extended_file_name, bool use_mmap,
 			dbStr += "to native format ";
 #endif
 			dbStr += "before mmap loading";
-			qDebug() << dbStr;
+			qWarning() << dbStr;
 			return 0;
 		}
 		dbStr += "byteswap ";
@@ -150,7 +147,7 @@ ZoneArray *ZoneArray::create(const QString& extended_file_name, bool use_mmap,
 		{
 			// mmap only with gcc:
 			dbStr += "warning - you must convert catalogue "
-			      += "to native format before mmap loading";
+				  += "to native format before mmap loading";
 			qDebug(qPrintable(dbStr));
 
 			return 0;
@@ -168,11 +165,8 @@ ZoneArray *ZoneArray::create(const QString& extended_file_name, bool use_mmap,
 		return 0;
 	}
 	ZoneArray *rval = 0;
-	dbStr += QString("%1_%2v%3_%4; ").arg(level)
-	         .arg(type)
-	         .arg(major)
-	         .arg(minor);
-	
+	dbStr += QString("%1_%2v%3_%4; ").arg(level).arg(type).arg(major).arg(minor);
+
 	switch (type)
 	{
 	case 0:
@@ -243,7 +237,7 @@ ZoneArray *ZoneArray::create(const QString& extended_file_name, bool use_mmap,
 	if (rval && rval->isInitialized())
 	{
 		dbStr += QString("%1").arg(rval->getNrOfStars());
-		qDebug() << qPrintable(dbStr);
+		qDebug() << dbStr;
 	}
 	else
 	{
@@ -259,7 +253,7 @@ ZoneArray *ZoneArray::create(const QString& extended_file_name, bool use_mmap,
 }
 
 ZoneArray::ZoneArray(const QString& fname, QFile* file, int level, int mag_min,
-		     int mag_range, int mag_steps)
+			 int mag_range, int mag_steps)
 			: fname(fname), level(level), mag_min(mag_min),
 			  mag_range(mag_range), mag_steps(mag_steps),
 			  star_position_scale(0.0), zones(0), file(file)
@@ -268,8 +262,7 @@ ZoneArray::ZoneArray(const QString& fname, QFile* file, int level, int mag_min,
 	nr_of_stars = 0;
 }
 
-bool ZoneArray::readFileWithStelLoadingBar(QFile& file, void *data, qint64 size,
-        StelLoadingBar &lb)
+bool ZoneArray::readFileWithStelLoadingBar(QFile& file, void *data, qint64 size, StelLoadingBar*lb)
 {
 	int parts = 256;
 	int part_size = (size + (parts>>1)) / parts;
@@ -279,7 +272,8 @@ bool ZoneArray::readFileWithStelLoadingBar(QFile& file, void *data, qint64 size,
 		parts = (size + (part_size>>1)) / part_size;
 	}
 	float i = 0.f;
-	lb.Draw(i / parts);
+	if (lb)
+		lb->Draw(i / parts);
 	i += 1.f;
 	while (size > 0)
 	{
@@ -288,7 +282,8 @@ bool ZoneArray::readFileWithStelLoadingBar(QFile& file, void *data, qint64 size,
 		if (read_rc != to_read) return false;
 		size -= read_rc;
 		data = ((char*)data) + read_rc;
-		lb.Draw(i / parts);
+		if (lb)
+			lb->Draw(i / parts);
 		i += 1.f;
 	}
 	return true;
@@ -329,14 +324,15 @@ void SpecialZoneArray<Star>::scaleAxis(void)
 
 template<class Star>
 SpecialZoneArray<Star>::SpecialZoneArray(QFile* file, bool byte_swap,bool use_mmap,
-					 StelLoadingBar &lb, int level, int mag_min,
+					 StelLoadingBar*lb, int level, int mag_min,
 					 int mag_range, int mag_steps)
 		: ZoneArray(file->fileName(), file, level, mag_min, mag_range, mag_steps),
 		  stars(0), mmap_start(0)
 {
 	if (nr_of_zones > 0)
 	{
-		lb.Draw(0.f);
+		if (lb)
+			lb->Draw(0.f);
 		zones = new SpecialZoneData<Star>[nr_of_zones];
 		if (zones == 0)
 		{
@@ -344,7 +340,7 @@ SpecialZoneArray<Star>::SpecialZoneArray(QFile* file, bool byte_swap,bool use_mm
 				 << ")::SpecialZoneArray: no memory (1)";
 			exit(1);
 		}
-		
+
 		unsigned int *zone_size = new unsigned int[nr_of_zones];
 		if (zone_size == 0)
 		{
@@ -405,7 +401,7 @@ SpecialZoneArray<Star>::SpecialZoneArray(QFile* file, bool byte_swap,bool use_mm
 					Star *s = stars;
 					for (unsigned int z=0;z<nr_of_zones;z++)
 					{
-						
+
 						getZones()[z].stars = s;
 						s += getZones()[z].size;
 					}
@@ -440,9 +436,9 @@ SpecialZoneArray<Star>::SpecialZoneArray(QFile* file, bool byte_swap,bool use_mm
 					}
 					if (
 #if (!defined(__GNUC__))
-					    true
+						true
 #else
-					    byte_swap
+						byte_swap
 #endif
 					)
 					{
@@ -450,12 +446,12 @@ SpecialZoneArray<Star>::SpecialZoneArray(QFile* file, bool byte_swap,bool use_mm
 						for (unsigned int i=0;i<nr_of_stars;i++,s++)
 						{
 							s->repack(
-#ifdef WORDS_BIGENDIAN
-							    // need for byte_swap on a BE machine means that catalog is LE
-							    !byte_swap
+#if Q_BYTE_ORDER == Q_BIG_ENDIAN
+								// need for byte_swap on a BE machine means that catalog is LE
+								!byte_swap
 #else
-							    // need for byte_swap on a LE machine means that catalog is BE
-							    byte_swap
+								// need for byte_swap on a LE machine means that catalog is BE
+								byte_swap
 #endif
 							);
 						}
@@ -464,7 +460,8 @@ SpecialZoneArray<Star>::SpecialZoneArray(QFile* file, bool byte_swap,bool use_mm
 				file->close();
 			}
 		}
-		lb.Draw(1.f);
+		if (lb)
+			lb->Draw(1.f);
 	}
 }
 
@@ -494,35 +491,28 @@ SpecialZoneArray<Star>::~SpecialZoneArray(void)
 }
 
 template<class Star>
-void SpecialZoneArray<Star>::draw(int index, bool is_inside, const float *rcmag_table, StelCore* core, unsigned int maxMagStarName,
-				  float names_brightness, StelFont *starFont) const
+void SpecialZoneArray<Star>::draw(StelPainter* sPainter, int index, bool is_inside, const float *rcmag_table, StelCore* core, unsigned int maxMagStarName,
+				  float names_brightness) const
 {
 	StelSkyDrawer* drawer = core->getSkyDrawer();
-	const StelProjectorP& prj = core->getProjection(StelCore::FrameJ2000);
 	SpecialZoneData<Star> *const z = getZones() + index;
-	Vec3d xy;
+	Vec3d v;
 	const Star *const end = z->getStars() + z->size;
-	const double d2000 = 2451545.0;
+	static const double d2000 = 2451545.0;
 	const double movementFactor = (M_PI/180)*(0.0001/3600) * ((core->getNavigator()->getJDay()-d2000)/365.25) / star_position_scale;
+	const float* tmpRcmag;
 	for (const Star *s=z->getStars();s<end;++s)
 	{
-		if (is_inside ? prj->project(s->getJ2000Pos(z,movementFactor),xy) : prj->projectCheck(s->getJ2000Pos(z,movementFactor),xy))
+		tmpRcmag = rcmag_table+2*s->mag;
+		if (*tmpRcmag<=0.)
+			break;
+		s->getJ2000Pos(z,movementFactor, v);
+		if (drawer->drawPointSource(sPainter, v,tmpRcmag,s->bV, !is_inside) && s->hasName() && s->mag < maxMagStarName)
 		{
-			if (drawer->drawPointSource(xy[0],xy[1],rcmag_table + 2*(s->mag),s->bV)==false)
-			{
-				break;
-			}
-			if (s->mag < maxMagStarName)
-			{
-				const QString& starname = s->getNameI18n();
-				if (!starname.isEmpty())
-				{
-					const float offset = (rcmag_table + 2*(s->mag))[0]*0.7;
-					const Vec3f& colorr = (StelApp::getInstance().getVisionModeNight() ? Vec3f(0.8, 0.2, 0.2) : StelSkyDrawer::indexToColor(s->bV))*0.75;
-					glColor4f(colorr[0], colorr[1], colorr[2],names_brightness);
-					drawer->getPainter()->drawText(starFont,xy[0],xy[1], starname, 0, offset, offset, false);
-				}
-			}
+			const float offset = *tmpRcmag*0.7;
+			const Vec3f& colorr = (StelApp::getInstance().getVisionModeNight() ? Vec3f(0.8, 0.2, 0.2) : StelSkyDrawer::indexToColor(s->bV))*0.75;
+			sPainter->setColor(colorr[0], colorr[1], colorr[2],names_brightness);
+			sPainter->drawText(v, s->getNameI18n(), 0, offset, offset, false);
 		}
 	}
 }
@@ -531,18 +521,18 @@ template<class Star>
 void SpecialZoneArray<Star>::searchAround(const StelNavigator* nav, int index, const Vec3d &v, double cosLimFov,
 					  QList<StelObjectP > &result)
 {
-	const double d2000 = 2451545.0;
+	static const double d2000 = 2451545.0;
 	const double movementFactor = (M_PI/180.)*(0.0001/3600.) * ((nav->getJDay()-d2000)/365.25)/ star_position_scale;
 	const SpecialZoneData<Star> *const z = getZones()+index;
 	Vec3d tmp;
-	for (int i=0;i<z->size;i++)
+	for (const Star* s=z->getStars();s<z->getStars()+z->size;++s)
 	{
-		tmp = z->getStars()[i].getJ2000Pos(z,movementFactor);
+		s->getJ2000Pos(z,movementFactor, tmp);
 		tmp.normalize();
 		if (tmp*v >= cosLimFov)
 		{
 			// TODO: do not select stars that are too faint to display
-			result.push_back(z->getStars()[i].createStelObject(this,z));
+			result.push_back(s->createStelObject(this,z));
 		}
 	}
 }
